@@ -2,7 +2,7 @@
 
     File: adv.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,19 +48,46 @@
 #define INTER_ADV_Y	23
 #define INTER_ADV	15
 
-static int is_linux(const int part_type);
+static int is_linux(const t_partition *partition);
 
-static int is_linux(const int part_type)
+static int is_linux(const t_partition *partition)
 {
-  switch(part_type)
+  switch(partition->part_type_i386)
   {
     case P_LINUX:
       return 1;
   }
+  switch(partition->part_type_sun)
+  {
+    case PSUN_LINUX:
+      return 1;
+  }
+  switch(partition->part_type_mac)
+  {
+    case PMAC_LINUX:
+      return 1;
+  }
+  switch(partition->upart_type)
+  {
+    case UP_CRAMFS:
+    case UP_EXT2:
+    case UP_EXT3:
+    case UP_JFS:
+    case UP_RFS:
+    case UP_RFS2:
+    case UP_RFS3:
+    case UP_XFS:
+    case UP_XFS2:
+    case UP_XFS3:
+    case UP_XFS4:
+      return 1;
+    default:
+      break;
+  }
   return 0;
 }
 
-void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert)
+void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd)
 {
   int quit;
   int offset=0;
@@ -81,7 +108,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
   {
     static struct MenuItem menuAdv[]=
     {
-      {'t',"Type","Change type, this settings will not be saved"},
+      {'t',"Type","Change type, this settings will not be saved on disk"},
       {'b',"Boot","Boot sector recovery"},
       {'s',"Superblock","Locate EXT2/EXT3 backup superblock"},
       {'q',"Quit","Return to main menu"},
@@ -99,7 +126,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       rewrite=0;
     }
     for(i=0,element=list_part;(element!=NULL) && (i<offset);element=element->next,i++);
-    for(i=offset;(element!=NULL) && ((i-offset)<INTER_ADV);i++,element=element->next)
+    for(i=offset;(element!=NULL) && ((i-offset)<INTER_ADV);element=element->next,i++)
     {
       wmove(stdscr,5+2+i-offset,0);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
@@ -117,25 +144,64 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
     menu=0;
     if(current_element!=NULL)
     {
-      /* TODO: is_fat, is_ntfs, is_linux only works on i386 */
-      if(is_fat(current_element->part->part_type) ||
-	  is_ntfs(current_element->part->part_type))
+      if(is_fat(current_element->part) ||
+	  is_ntfs(current_element->part))
       {
 	options="tbq";
 	menu=1;
       }
-      else if(is_linux(current_element->part->part_type))
+      else if(is_linux(current_element->part))
       {
 	options="tsq";
 	menu=1;
       }
     }
     quit=0;
-    command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
-	MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
+    if(*current_cmd!=NULL)
+    {
+      int keep_asking;
+      command='Q';
+      do
+      {
+	keep_asking=0;
+	while(*current_cmd[0]==',')
+	  (*current_cmd)++;
+	if(strncmp(*current_cmd,"type",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='t';
+	}
+	else if(strncmp(*current_cmd,"boot",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='b';
+	}
+	else if(strncmp(*current_cmd,"superblock",10)==0)
+	{
+	  (*current_cmd)+=10;
+	  command='s';
+	}
+	else
+	{
+	  unsigned int order;
+	  order= atoi(*current_cmd);
+	  while(*current_cmd[0]!=',' && *current_cmd[0]!='\0')
+	    (*current_cmd)++;
+	  for(element=list_part;element!=NULL && element->part->order!=order;element=element->next);
+	  if(element!=NULL)
+	  {
+	    current_element=element;
+	    keep_asking=1;
+	  }
+	}
+      } while(keep_asking>0);
+    }
+    else
+      command = wmenuSelect(stdscr,INTER_ADV_Y, INTER_ADV_X, menuAdv, 8, options,
+	  MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu);
     switch(command)
     {
-      case MENU_UP:
+      case KEY_UP:
 	if(current_element!=NULL)
 	{
 	  if(current_element->prev!=NULL)
@@ -147,7 +213,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	    offset--;
 	}
 	break;
-      case MENU_DOWN:
+      case KEY_DOWN:
 	if(current_element!=NULL)
 	{
 	  if(current_element->next!=NULL)
@@ -167,19 +233,19 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
       case 'B':
 	if(current_element!=NULL)
 	{
-	  if(is_fat32(current_element->part->part_type))
+	  if(is_fat32(current_element->part))
 	  {
-	    fat32_boot_sector(disk_car, current_element->part, debug, dump_ind, expert);
+	    fat32_boot_sector(disk_car, current_element->part, debug, dump_ind, expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(is_fat(current_element->part->part_type))
+	  else if(is_fat(current_element->part))
 	  {
-	    fat1x_boot_sector(disk_car, current_element->part, debug, dump_ind,expert);
+	    fat1x_boot_sector(disk_car, current_element->part, debug, dump_ind,expert,current_cmd);
 	    rewrite=1;
 	  }
-	  else if(is_ntfs(current_element->part->part_type))
+	  else if(is_ntfs(current_element->part))
 	  {
-	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind);
+	    ntfs_boot_sector(disk_car, current_element->part, debug, dump_ind,current_cmd);
 	    rewrite=1;
 	  }
 	}
@@ -189,7 +255,7 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
 	if(current_element!=NULL)
 	{
 	  t_list_part *list_sb=search_superblock(disk_car,current_element->part,debug,dump_ind,1);
-	  interface_superblock(disk_car,list_sb);
+	  interface_superblock(disk_car,list_sb,current_cmd);
 	  delete_list_part(list_sb);
 	  rewrite=1;
 	}
@@ -207,13 +273,25 @@ void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, c
   delete_list_part(list_part);
 }
 
-int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert)
+int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char buffer_bs[0x200];
   const char *options="DR";
   int rescan=1;
+  struct MenuItem menu_fat1x[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Quit","Return to Advanced menu"},
+    { 'R', "Rebuild BS","Rebuild boot sector"},
+    { 'D', "Dump","Dump boot sector and backup boot sector"},
+    { 'C', "Repair FAT","Very Dangerous! Expert only"},
+    { 'I', "Init Root","Init root directory: Very Dangerous! Expert only"},
+    { 0, NULL, NULL }
+  };
   while(1)
   {
+    int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
     {
@@ -241,13 +319,43 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       }
       rescan=0;
     }
-    /*  mvwaddstr(stdscr,23,0,msg_PART_TYPE); */
-    switch(aff_buffer(BUFFER_DISPLAY,options,stdscr))
+    if(*current_cmd!=NULL)
+    {
+      command=0;
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"rebuildbs",9)==0)
+      {
+	(*current_cmd)+=9;
+	command='R';
+      }
+      else if(strncmp(*current_cmd,"dump",4)==0)
+      {
+	(*current_cmd)+=4;
+	command='D';
+      }
+      else if(strncmp(*current_cmd,"repairfat",8)==0)
+      {
+	(*current_cmd)+=8;
+	if(strchr(options,'C')!=NULL)
+	  command='C';
+      }
+      else if(strncmp(*current_cmd,"initroot",8)==0)
+      {
+	(*current_cmd)+=8;
+	if(strchr(options,'I')!=NULL)
+	    command='I';
+      }
+      screen_buffer_to_log();
+    }
+    else
+      command=screen_buffer_display(stdscr,options,menu_fat1x);
+    switch(command)
     {
       case 0:
 	return 0;
       case 'R': /* R : rebuild boot sector */
-	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert);
+	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
 	rescan=1;
 	break;
       case 'D':
@@ -279,14 +387,28 @@ int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
   }
 }
 
-int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert)
+int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd)
 {
   unsigned char buffer_bs[3*disk_car->sector_size];
   unsigned char buffer_backup_bs[3*disk_car->sector_size];
   const char *options="DRC";
   int rescan=1;
+  struct MenuItem menu_fat32[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Quit","Return to Advanced menu"},
+    { 'O', "Org. BS","Copy boot sector over backup sector"},
+    { 'B', "Backup BS","Copy backup boot sector over boot sector"},
+    { 'R', "Rebuild BS","Rebuild boot sector"},
+    { 'D', "Dump","Dump boot sector and backup boot sector"},
+    { 'C', "Repair FAT","Very Dangerous! Expert only"},
+    { 0, NULL, NULL }
+  };
+
   while(1)
   {
+    int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
     {
@@ -353,7 +475,44 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
       }
       rescan=0;
     }
-    switch(aff_buffer(BUFFER_DISPLAY,options,stdscr))
+    if(*current_cmd!=NULL)
+    {
+      command=0;
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"rebuildbs",9)==0)
+      {
+	(*current_cmd)+=9;
+	command='R';
+      }
+      else if(strncmp(*current_cmd,"dump",4)==0)
+      {
+	(*current_cmd)+=4;
+	command='D';
+      }
+      else if(strncmp(*current_cmd,"repairfat",8)==0)
+      {
+	(*current_cmd)+=8;
+	if(strchr(options,'C')!=NULL)
+	  command='C';
+      }
+      else if(strncmp(*current_cmd,"originalfat",11)==0)
+      {
+	(*current_cmd)+=11;
+	if(strchr(options,'O')!=NULL)
+	    command='O';
+      }
+      else if(strncmp(*current_cmd,"backupfat",9)==0)
+      {
+	(*current_cmd)+=9;
+	if(strchr(options,'B')!=NULL)
+	    command='B';
+      }
+      screen_buffer_to_log();
+    }
+    else
+      command=screen_buffer_display(stdscr,options,menu_fat32);
+    switch(command)
     {
       case 0:
 	return 0;
@@ -380,7 +539,7 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
 	}
 	break;
       case 'R': /* R : rebuild boot sector */
-	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert);
+	rebuild_FAT_BS(disk_car,partition,debug,dump_ind,1,expert,current_cmd);
 	rescan=1;
 	break;
       case 'D':
@@ -410,14 +569,27 @@ int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int
   }
 }
 
-int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind)
+int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, char **current_cmd)
 {
   unsigned char buffer_bs[DEFAULT_SECTOR_SIZE];
   unsigned char buffer_backup_bs[DEFAULT_SECTOR_SIZE];
   const char *options="";
   int rescan=1;
+  struct MenuItem menu_ntfs[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Quit","Return to Advanced menu"},
+    { 'O', "Org. BS","Copy boot sector over backup sector"},
+    { 'B', "Backup BS","Copy backup boot sector over boot sector"},
+    { 'R', "Rebuild BS","Rebuild boot sector"},
+    { 'D', "Dump","Dump boot sector and backup boot sector"},
+    { 0, NULL, NULL }
+  };
+
   while(1)
   {
+    int command;
     aff_buffer(BUFFER_RESET,"Q");
     if(rescan==1)
     {
@@ -488,7 +660,38 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
       }
       rescan=0;
     }
-    switch(aff_buffer(BUFFER_DISPLAY,options,stdscr))
+    if(*current_cmd!=NULL)
+    {
+      command=0;
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"rebuildbs",9)==0)
+      {
+	(*current_cmd)+=9;
+	command='R';
+      }
+      else if(strncmp(*current_cmd,"dump",4)==0)
+      {
+	(*current_cmd)+=4;
+	command='D';
+      }
+      else if(strncmp(*current_cmd,"originalntfs",11)==0)
+      {
+	(*current_cmd)+=11;
+	if(strchr(options,'O')!=NULL)
+	    command='O';
+      }
+      else if(strncmp(*current_cmd,"backupntfs",9)==0)
+      {
+	(*current_cmd)+=9;
+	if(strchr(options,'B')!=NULL)
+	    command='B';
+      }
+      screen_buffer_to_log();
+    }
+    else
+      command=screen_buffer_display(stdscr,options,menu_ntfs);
+    switch(command)
     {
       case 0:
 	return 0;
@@ -496,7 +699,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	if(ask_confirmation("Copy original NTFS boot sector over backup boot, confirm ? (Y/N)")!=0)
 	{
 	  ecrit_rapport("copy original boot sector over backup boot\n");
-	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+6*disk_car->sector_size)!=0)
+	  if(disk_car->write(disk_car,sizeof(buffer_bs), &buffer_bs, partition->part_offset+partition->part_size-disk_car->sector_size)!=0)
 	  {
 	    display_message("Write error: Can't overwrite NTFS backup boot sector\n");
 	  }
@@ -515,7 +718,7 @@ int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int d
 	}
 	break;
       case 'R': /* R : rebuild boot sector */
-	rebuild_NTFS_BS(disk_car,partition,debug,dump_ind,1);
+	rebuild_NTFS_BS(disk_car,partition,debug,dump_ind,1,current_cmd);
 	rescan=1;
 	break;
       case 'D':
