@@ -19,8 +19,8 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert);
-int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert);
-int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert);
-int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind);
+void interface_adv(t_param_disk *disk_car, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd);
+int fat1x_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd);
+int fat32_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, const unsigned int expert, char **current_cmd);
+int ntfs_boot_sector(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind, char **current_cmd);
 
