@@ -2,7 +2,7 @@
 
     File: bfs.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -50,7 +50,7 @@ int recover_BeFS(t_param_disk *disk_car, const struct disk_super_block *beos_blo
     return 1;
   set_BeFS_info(disk_car,beos_block,partition,debug,0);
   partition->part_size=beos_block->num_blocks*(1<<beos_block->block_shift);
-  partition->part_type=(unsigned char)P_BEOS;
+  partition->part_type_i386=(unsigned char)P_BEOS;
   return 0;
 }
 
