@@ -2,7 +2,7 @@
 
     File: bsd.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -61,7 +61,7 @@ int test_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_partiti
       crc^=*cp;
     if(crc==0)
     {
-      if(debug)
+      if(debug>0)
       {
 	ecrit_rapport("CRC Ok\n");
       }
@@ -72,7 +72,7 @@ int test_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_partiti
     {
       if(bsd_header->d_partitions[i].p_fstype>0)
       {
-	if(debug)
+	if(debug>0)
 	{
 	  /* UFS UFS2 SWAP */
 	  ecrit_rapport("BSD %c: ", 'a'+i);
@@ -135,10 +135,11 @@ int recover_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_part
 	  le32(bsd_header->d_partitions[i_max_p_offset].p_offset) - 1) * disk_car->sector_size - partition->part_offset;
     else
       partition->part_size=0;
-    partition->part_type=P_FREEBSD;
+    partition->part_type_i386=P_FREEBSD;
     set_part_name(partition,bsd_header->d_packname,16);
     if(check_volume_name(partition->name,16))
       partition->name[0]='\0';
+    partition->info[0]='\0';
     return 0;
   }
   if(test_BSD(disk_car,bsd_header,partition,debug,dump_ind,OPENBSD_MAXPARTITIONS)==0)
@@ -156,10 +157,11 @@ int recover_BSD(t_param_disk *disk_car, const struct disklabel*bsd_header,t_part
 	  le32(bsd_header->d_partitions[i_max_p_offset].p_offset) - 1) * disk_car->sector_size - partition->part_offset;
     else
       partition->part_size=0;
-    partition->part_type=P_OPENBSD;
+    partition->part_type_i386=P_OPENBSD;
     set_part_name(partition,bsd_header->d_packname,16);
     if(check_volume_name(partition->name,16))
       partition->name[0]='\0';
+    partition->info[0]='\0';
     return 0;
   }
   return 1;
