@@ -2,7 +2,7 @@
 
     File: chgtype.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,35 +38,45 @@ int change_part_type(const t_param_disk *disk_car,t_partition *partition)
   char response[100];
   int nbr=0;
   int i;
+  struct MenuItem menuType[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Proceed","Go set the partition type"},
+    { 0, NULL, NULL }
+  };
+  t_partition *new_partition=partition_new();
+  dup_t_partition(new_partition,partition);
   aff_buffer(BUFFER_RESET,"Q");
   aff_buffer(BUFFER_ADD,"List of partition type\n");
   for(i=0;i<=0xFF;i++)
   {
-    const unsigned char *part_name;
-    part_name=partition_type(i, ((partition->arch!=NULL)?partition->arch->parttype_name_table:disk_car->arch->parttype_name_table));
-    if(part_name!=NULL)
+    if(disk_car->arch->set_part_type(new_partition,i)==0)
     {
-      aff_buffer(BUFFER_ADD,"%02x %-20s ",i,part_name);
-      nbr++;
-      if(nbr==3)
+      const unsigned char *part_name;
+      part_name=disk_car->arch->get_partition_name(new_partition);
+      if(part_name!=NULL)
       {
-	nbr=0;
-	aff_buffer(BUFFER_ADD,"\n");
+	aff_buffer(BUFFER_ADD,"%02x %-20s ",i,part_name);
+	nbr++;
+	if(nbr==3)
+	{
+	  nbr=0;
+	  aff_buffer(BUFFER_ADD,"\n");
+	}
       }
     }
   }
+  FREE(new_partition);
   aff_copy(stdscr);
   wmove(stdscr,4,0);
   aff_part(stdscr,AFF_PART_ORDER,disk_car,partition);
-  aff_buffer(BUFFER_DISPLAY,"Q",stdscr);
+  screen_buffer_display(stdscr,"",menuType);
   wmove(stdscr,23,0);
-  wdoprintf(stdscr,"New partition type [current %02x] ?",partition->part_type);
+  wdoprintf(stdscr,"New partition type [current %02x] ? ",disk_car->arch->get_part_type(partition));
   if (get_string(response, sizeof(response), NULL) > 0) {
     int tmp_val = strtol(response, NULL, 16);
-    if(disk_car->arch->is_part_type_ok(tmp_val)!=0)
-    {
-      partition->part_type = tmp_val;
-    }
+    disk_car->arch->set_part_type(partition,tmp_val);
   }
   return 0;
 }
