@@ -20,6 +20,6 @@
 
  */
 
-void change_geometry(t_param_disk *disk_car);
+void change_geometry(t_param_disk *disk_car, char **current_cmd);
 int change_part_type(const t_param_disk *disk_car,t_partition *partition);
 
