@@ -2,7 +2,7 @@
 
     File: cramfs.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ static int test_cramfs(const t_param_disk *disk_car, const struct cramfs_super *
   }
   else
     return 1;
-  if(debug!=0)
+  if(debug>0)
     ecrit_rapport("\ncramfs Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   return 0;
 }
@@ -65,7 +65,7 @@ int recover_cramfs(t_param_disk *disk_car, const struct cramfs_super *sb,t_parti
 {
   if(test_cramfs(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
-  if(debug!=0 || dump_ind!=0)
+  if(debug>0 || dump_ind!=0)
   {
     ecrit_rapport("\nrecover_cramfs\n");
     if(dump_ind!=0)
@@ -74,14 +74,15 @@ int recover_cramfs(t_param_disk *disk_car, const struct cramfs_super *sb,t_parti
     }
   }
   partition->part_size = sb->size;
-  partition->part_type = (unsigned char)P_LINUX;
+  partition->part_type_i386 = P_LINUX;
+  partition->part_type_mac= PMAC_LINUX;
+  partition->part_type_sun= PSUN_LINUX;
   set_cramfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
 
 static int set_cramfs_info(const t_param_disk *disk_car,const struct cramfs_super *sb,t_partition *partition, const int debug, const int dump_ind)
 {
-  partition->info[0]='\0';
   set_part_name(partition,sb->name,16);
   switch(partition->upart_type)
   {
@@ -89,6 +90,7 @@ static int set_cramfs_info(const t_param_disk *disk_car,const struct cramfs_supe
       strncpy(partition->info,"cramfs",sizeof(partition->info));
       break;
     default:
+      partition->info[0]='\0';
       return 1;
   }
 
