@@ -2,7 +2,7 @@
 
     File: dir.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,7 +29,9 @@
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -42,8 +44,8 @@
 #include "intrf.h"
 #include "dir.h"
 
-static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time);
-static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time);
+static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time, char **current_cmd);
+static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time, char**current_cmd);
 
 static const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
@@ -164,6 +166,7 @@ int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, cons
     if(current_file->filestat.st_mtime)
     {
       tm_p = localtime(&current_file->filestat.st_mtime);
+
       snprintf(datestr, sizeof(datestr),"%2d-%s-%4d %02d:%02d",
 	  tm_p->tm_mday, monstr[tm_p->tm_mon],
 	  1900 + tm_p->tm_year, tm_p->tm_hour,
@@ -186,7 +189,7 @@ int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, cons
   return test_date;
 }
 
-static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time)
+static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const t_file_data*dir_list, const unsigned long int inode, const int first_time, char **current_cmd)
 {
   /* Return value
    * -1: quit
@@ -224,6 +227,7 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
 	    tm_p->tm_mday, monstr[tm_p->tm_mon],
 	    1900 + tm_p->tm_year, tm_p->tm_hour,
 	    tm_p->tm_min);
+      /* May have to use %d instead of %e */
       } else {
 	strncpy(datestr, "                 ",sizeof(datestr));
       }
@@ -244,12 +248,15 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
     if(dir_list==NULL)
     {
       wmove(dir_data->window,8,0);
-      wdoprintf(dir_data->window,"No file found, damaged partition ?");
+      wdoprintf(dir_data->window,"No file found, filesystem seems damaged.");
     }
     wrefresh(dir_data->window);
     /* Using gnome terminal under FC3, TERM=xterm, the screen is not always correct */
     wredrawln(dir_data->window,0,getmaxy(dir_data->window));	/* redrawwin def is boggus in pdcur24 */
-    car=wgetch(dir_data->window);
+    if(*current_cmd!=NULL)
+      car='q';
+    else
+      car=wgetch(dir_data->window);
     switch(car)
     {
       case key_ESC:
@@ -341,7 +348,6 @@ static long int dir_aff(t_param_disk *disk_car, const t_partition *partition, t_
 void delete_list_file(t_file_data *file_list)
 {
   t_file_data *current_file=file_list;
-  /* Libere la memoire */
   while(current_file!=NULL)
   {
     t_file_data *next=current_file->next;
@@ -350,28 +356,28 @@ void delete_list_file(t_file_data *file_list)
   }
 }
 
-int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode)
+int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, char **current_cmd)
 {
   if(dir_data==NULL)
     return -1;
-  return dir_partition_aux(disk_car,partition,dir_data,inode,1);
+  return dir_partition_aux(disk_car,partition,dir_data,inode,1,current_cmd);
 }
 
-static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time)
+static int dir_partition_aux(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, const int first_time, char**current_cmd)
 {
   t_file_data *dir_list;
   long int new_inode;
-  if(dir_data->debug!=0)
+  if(dir_data->debug>0)
     ecrit_rapport("\ndir_partition inode=%ld\n",inode);
   dir_list=dir_data->get_dir(disk_car,partition,dir_data,inode);
   dir_aff_log(disk_car, partition, dir_data, dir_list);
   do
   {
     unsigned int current_directory_namelength=strlen(dir_data->current_directory);
-    new_inode=dir_aff(disk_car,partition,dir_data,dir_list,inode,first_time);
+    new_inode=dir_aff(disk_car,partition,dir_data,dir_list,inode,first_time,current_cmd);
     if(new_inode==0 || new_inode>1)
     {
-      if(dir_partition_aux(disk_car, partition, dir_data, (unsigned long int)new_inode,0)<0)
+      if(dir_partition_aux(disk_car, partition, dir_data, (unsigned long int)new_inode,0,current_cmd)<0)
       { /* quit */
 	delete_list_file(dir_list);
 	return -1;
