@@ -21,7 +21,7 @@ struct file_data
 
 int dir_aff_log(const t_param_disk *disk_car, const t_partition *partition, const t_dir_data *dir_data, const t_file_data*dir_list);
 void delete_list_file(t_file_data *file_list);
-int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode);
+int dir_partition(t_param_disk *disk_car, const t_partition *partition, t_dir_data *dir_data, const unsigned long int inode, char **current_cmd);
 void mode_string (const unsigned int mode, char *str);
 
 #define LINUX_S_IFMT  00170000
