@@ -1,8 +1,8 @@
 /*
 
-    File: ext2.h
+    File: ext2_dir.h
 
-    Copyright (C) 1998-2003  Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005  Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,4 +20,5 @@
 
  */
 
-int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug);
+int dir_partition_ext2(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd);
+const char*td_ext2fs_version(void);
