@@ -24,14 +24,16 @@
 #include <config.h>
 #endif
  
-#include <ctype.h>	/* toupper */
+#include <ctype.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "fat.h"
@@ -283,7 +285,7 @@ int check_FAT(t_param_disk *disk_car,t_partition *partition,const int debug)
   }
   if(test_FAT(disk_car,(const struct fat_boot_sector *)buffer,partition,debug,0)!=0)
   {
-    if(debug!=0)
+    if(debug>0)
     {
       ecrit_rapport("\n\ntest_FAT()\n");
       aff_part_rapport(disk_car,partition);
@@ -299,7 +301,7 @@ int check_FAT(t_param_disk *disk_car,t_partition *partition,const int debug)
 static int set_FAT_info(t_param_disk *disk_car, const struct fat_boot_sector *fat_header, t_partition *partition,const int debug)
 {
   const char*buffer=(const char*)fat_header;
-  partition->name[0]=0;
+  partition->name[0]='\0';
   switch(partition->upart_type)
   {
     case UP_FAT12:
@@ -447,7 +449,7 @@ int set_next_cluster(t_param_disk *disk_car,const t_partition *partition, const
   }
   if(disk_car->write(disk_car,sizeof(buffer), &buffer, partition->part_offset+(uint64_t)(offset+offset_s)*disk_car->sector_size)!=0)
   {
-    ecrit_rapport("\nset_next_cluster write error");
+    display_message("Write error: set_next_cluster write error\n");
     return 1;
   }
   return 0;
@@ -801,7 +803,7 @@ int recover_FAT(t_param_disk *disk_car, const struct fat_boot_sector*fat_header,
 	  ecrit_rapport("\nFAT12 at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
 	}
 	partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
-	partition->part_type=P_12FAT;
+	partition->part_type_i386=P_12FAT;
 	break;
       case UP_FAT16:
 	if(debug||dump_ind)
@@ -811,13 +813,13 @@ int recover_FAT(t_param_disk *disk_car, const struct fat_boot_sector*fat_header,
 	if(sectors(fat_header)!=0)
 	{
 	  partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
-	  partition->part_type=P_16FAT;
+	  partition->part_type_i386=P_16FAT;
 	} else {
 	  partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
 	  if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
-	    partition->part_type=P_16FATBD;
+	    partition->part_type_i386=P_16FATBD;
 	  else
-	    partition->part_type=P_16FATBD_LBA;
+	    partition->part_type_i386=P_16FATBD_LBA;
 	}
 	break;
       case UP_FAT32:
@@ -830,9 +832,9 @@ int recover_FAT(t_param_disk *disk_car, const struct fat_boot_sector*fat_header,
 	else
 	  partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
 	if(offset2cylinder(disk_car,partition->part_offset+partition->part_size-1)<=1024)
-	  partition->part_type=P_32FAT;
+	  partition->part_type_i386=P_32FAT;
 	else
-	  partition->part_type=P_32FAT_LBA;
+	  partition->part_type_i386=P_32FAT_LBA;
 	if(backup)
 	{
 	  partition->boot_sector=6;
@@ -889,7 +891,28 @@ int fat32_set_part_name(t_param_disk *disk_car, t_partition *partition, const st
   return 0;
 }
 
-int recover_HPFS(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind)
+int check_HPFS(t_param_disk *disk_car,t_partition *partition,const int debug)
+{
+  unsigned char buffer[disk_car->sector_size];
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  {
+    aff_buffer(BUFFER_ADD,"check_HPFS: Read error\n");
+    ecrit_rapport("check_HPFS: Read error\n");
+    return 1;
+  }
+  if(test_HPFS(disk_car,(const struct fat_boot_sector *)buffer,partition,debug,0)!=0)
+  {
+    if(debug>0)
+    {
+      ecrit_rapport("\n\ntest_HPFS()\n");
+      aff_part_rapport(disk_car,partition);
+    }
+    return 1;
+  }
+  return 0;
+}
+
+int test_HPFS(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_partition *partition,const int debug, const int dump_ind)
 {
   const char*buffer=(const char*)fat_header;
   if(le16(fat_header->marker)==0xAA55)
@@ -908,7 +931,6 @@ int recover_HPFS(t_param_disk *disk_car,const struct fat_boot_sector*fat_header,
 	partition->part_size=(uint64_t)sectors(fat_header)*fat_sector_size(fat_header);
       else
 	partition->part_size=(uint64_t)le32(fat_header->total_sect)*fat_sector_size(fat_header);
-      partition->part_type=P_HPFS;
       partition->upart_type=UP_HPFS;
       return 0;
     }
@@ -916,7 +938,49 @@ int recover_HPFS(t_param_disk *disk_car,const struct fat_boot_sector*fat_header,
   return 1;
 }
 
+int recover_HPFS(t_param_disk *disk_car,const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind)
+{
+  if(test_HPFS(disk_car,fat_header,partition,debug,0)!=0)
+    return 1;
+  partition->part_type_i386=P_HPFS;
+  partition->name[0]='\0';
+  partition->info[0]='\0';
+  return 0;
+}
+
+int check_OS2MB(t_param_disk *disk_car,t_partition *partition,const int debug)
+{
+  unsigned char buffer[disk_car->sector_size];
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  {
+    aff_buffer(BUFFER_ADD,"check_OS2MB: Read error\n");
+    ecrit_rapport("check_OS2MB: Read error\n");
+    return 1;
+  }
+  if(test_OS2MB(disk_car,(const struct fat_boot_sector *)buffer,partition,debug,0)!=0)
+  {
+    if(debug>0)
+    {
+      ecrit_rapport("\n\ntest_OS2MB()\n");
+      aff_part_rapport(disk_car,partition);
+    }
+    return 1;
+  }
+  return 0;
+}
+
 int recover_OS2MB(t_param_disk *disk_car, const struct fat_boot_sector*fat_header, t_partition *partition, const int debug, const int dump_ind)
+{
+  if(test_OS2MB(disk_car, fat_header, partition, debug, dump_ind))
+    return 1;
+  partition->part_size=(uint64_t)(disk_car->CHS.head+1) * disk_car->CHS.sector*disk_car->sector_size; /* 1 cylinder */
+  partition->part_type_i386=P_OS2MB;
+  partition->name[0]='\0';
+  partition->info[0]='\0';
+  return 0;
+}
+
+int test_OS2MB(t_param_disk *disk_car,const struct fat_boot_sector *fat_header, t_partition *partition,const int debug, const int dump_ind)
 {
   const char*buffer=(const char*)fat_header;
   if(le16(fat_header->marker)==0xAA55)
@@ -929,8 +993,6 @@ int recover_OS2MB(t_param_disk *disk_car, const struct fat_boot_sector*fat_heade
       }
       if(dump_ind)
 	dump(stdscr,buffer,DEFAULT_SECTOR_SIZE);
-      partition->part_size=(uint64_t)(disk_car->CHS.head+1) * disk_car->CHS.sector*disk_car->sector_size; /* 1 cylinder */
-      partition->part_type=P_OS2MB;
       partition->upart_type=UP_OS2MB;
       return 0;
     }
@@ -938,25 +1000,27 @@ int recover_OS2MB(t_param_disk *disk_car, const struct fat_boot_sector*fat_heade
   return 1;
 }
 
-int is_fat(const int part_type)
+int is_fat(const t_partition *partition)
 {
-  return (is_fat12(part_type)||is_fat16(part_type)||is_fat32(part_type));
+  return (is_fat12(partition)||is_fat16(partition)||is_fat32(partition));
 }
 
-int is_fat12(const int part_type)
+int is_fat12(const t_partition *partition)
 {
-  switch(part_type)
+  switch(partition->part_type_i386)
   {
     case P_12FAT:
     case P_12FATH:
       return 1;
   }
+  if(partition->upart_type==UP_FAT12)
+    return 1;
   return 0;
 }
 
-int is_fat16(const int part_type)
+int is_fat16(const t_partition *partition)
 {
-  switch(part_type)
+  switch(partition->part_type_i386)
   {
     case P_16FAT:
     case P_16FATH:
@@ -966,12 +1030,14 @@ int is_fat16(const int part_type)
     case P_16FATBD_LBAH:
       return 1;
   }
+  if(partition->upart_type==UP_FAT16)
+    return 1;
   return 0;
 }
 
-int is_fat32(const int part_type)
+int is_fat32(const t_partition *partition)
 {
-  switch(part_type)
+  switch(partition->part_type_i386)
   {
     case P_32FAT:
     case P_32FAT_LBA:
@@ -979,6 +1045,8 @@ int is_fat32(const int part_type)
     case P_32FAT_LBAH:
       return 1;
   }
+  if(partition->upart_type==UP_FAT32)
+    return 1;
   return 0;
 }
 
