@@ -2,7 +2,7 @@
 
     File: fat_adv.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -24,14 +24,16 @@
 #include <config.h>
 #endif
  
-#include <ctype.h>	/* toupper */
+#include <ctype.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif
 #ifdef HAVE_SYS_STAT_H
 #include <sys/stat.h>
 #endif
@@ -73,13 +75,13 @@ struct info_cluster_offset
   unsigned int nbr;
   unsigned int first_sol;
 };
-static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved, unsigned int*fat_length, const t_partition *partition,const unsigned long int max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats);
-static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,const unsigned long int max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface,unsigned int *nbr_offset,t_info_offset *info_offset);
+static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved, unsigned int*fat_length, const t_partition *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats);
+static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface,unsigned int *nbr_offset,t_info_offset *info_offset);
 
 static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_fat12, const int p_fat16, const int p_fat32,unsigned long int*fat_offset, const unsigned int sector_size);
 
 static upart_type_t no_of_cluster2part_type(const unsigned long int no_of_cluster);
-static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats);
+static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd);
 static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_partition *partition,const unsigned int cluster_size, const unsigned long int no_of_cluster, const unsigned int reserved, const unsigned int fat_length, const int interface, const int debug, const unsigned int expert, const unsigned int first_free_cluster, const unsigned int fats);
 static int write_FAT_boot_code_aux(unsigned char *buffer);
 static int find_cluster_size(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface, unsigned int *cluster_size, unsigned long int *offset);
@@ -92,7 +94,7 @@ static int check_entree(const unsigned char *entree);
 static int fat32_create_rootdir(t_param_disk *disk_car,const t_partition *partition, const unsigned int reserved, const unsigned int fat_length, const unsigned int root_cluster, const unsigned int cluster_size, const int debug, t_file_data *rootdir_list, const unsigned int fats);
 
 static void fat_date_unix2dos(int unix_date,unsigned short *mstime, unsigned short *msdate);
-static upart_type_t select_fat_info(const t_info_offset *info_offset, const unsigned int nbr_offset,unsigned int*reserved, unsigned int*fat_length, const unsigned long int max_offset, unsigned int *fats);
+static upart_type_t select_fat_info(const t_info_offset *info_offset, const unsigned int nbr_offset,unsigned int*reserved, unsigned int*fat_length, const unsigned long int max_sector_offset, unsigned int *fats);
 static unsigned long int get_subdirectory(t_param_disk *disk_car,const uint64_t hd_offset, const unsigned long int i);
 
 /* The goal is to identify the begining of the two FAT
@@ -329,28 +331,15 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
     }
     for(root_cluster=2;(root_cluster<2+no_of_cluster)&&(ind_stop==0);root_cluster++)
     {
+      unsigned long int percent=root_cluster*100/(2+no_of_cluster);
       if(interface&&(root_cluster%171)==0)
       {
 	wmove(stdscr,9,0);
 	wclrtoeol(stdscr);
-	wdoprintf(stdscr,"Search root cluster %10lu/%lu",root_cluster,2+no_of_cluster);
+	wdoprintf(stdscr,"Search root cluster %10lu/%lu %lu%%",root_cluster,2+no_of_cluster,percent);
 	wrefresh(stdscr);
-	switch(wgetch_nodelay(stdscr))
-	{
-	  case KEY_ENTER:
-#ifdef PADENTER
-	  case PADENTER:
-#endif
-	  case '\n':
-	  case '\r':
-	  case 's':
-	  case 'S':
-	    ind_stop=1;
-	    break;
-	}
+	ind_stop|=check_enter_or_s(stdscr);
       }
-      /*
-       */
       if(disk_car->read(disk_car,cluster_size*disk_car->sector_size, &buffer, partition->part_offset+(uint64_t)(start_data+(root_cluster-2)*cluster_size)*disk_car->sector_size)==0)
       {
 	if(debug>1)
@@ -386,15 +375,14 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
 		found=0;
 		break;
 	    }
-	    if((buffer[i*0x20]!=DELETED_FLAG) && (buffer[i*0x20+0xB]==ATTR_DIR)) /* Test directory */
+	    if((buffer[i*0x20]!=DELETED_FLAG) && (buffer[i*0x20+0xB]!= ATTR_EXT && (buffer[i*0x20+0xB]&ATTR_DIR)!=0)) /* Test directory */
 	    {
 	      nb_subdir++;
-	      /*	  ecrit_rapport("directory %s\n",&buffer[i*0x20]); */
 	    }
 	  }
 	  for(i=0;found && (i<16*cluster_size);i++)
 	  {
-	    if((buffer[i*0x20]!=DELETED_FLAG) && (buffer[i*0x20+0xB]==ATTR_DIR)) /* Test directory */
+	    if((buffer[i*0x20]!=DELETED_FLAG) && (buffer[i*0x20+0xB]!= ATTR_EXT && (buffer[i*0x20+0xB]&ATTR_DIR)!=0)) /* Test directory */
 	    {
 	      unsigned long int cluster=(buffer[i*0x20+0x15]<<24)+(buffer[i*0x20+0x14]<<16)+
 		(buffer[i*0x20+0x1B]<<8)+buffer[i*0x20+0x1A];
@@ -452,6 +440,11 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
 	      } while(tmp && (++back<10));
 	      return new_root_cluster;
 	    }
+	    else
+	    {
+	      if(debug>1)
+		ecrit_rapport("cluster %lu, etat=%d, found=%d,nb_subdir=%d,nb_subdir_ok=%d\n",root_cluster,etat,found,nb_subdir,nb_subdir_ok);
+	    }
 	    {
 	      t_file_data *dir_list;
 	      dir_list=dir_fat_aux(buffer,cluster_size*disk_car->sector_size,cluster_size);
@@ -488,7 +481,7 @@ static unsigned int fat32_find_root_cluster(t_param_disk *disk_car,const t_parti
 	  unsigned long int cluster=(buffer[1*0x20+0x15]<<24)+(buffer[1*0x20+0x14]<<16)+
 	    (buffer[1*0x20+0x1B]<<8)+buffer[1*0x20+0x1A];
 	  if((memcmp(&buffer[0x20],"..         ",8+3)==0) &&
-	      (buffer[1*0x20+0xB]==ATTR_DIR) && (cluster==0)
+	      (buffer[1*0x20+0xB]!=ATTR_EXT && (buffer[1*0x20+0xB]&ATTR_DIR)!=0) && (cluster==0)
 	      && (buffer[0x40]!=0)) /* First-level directory */
 	  {
 	    t_file_data *dir_list;
@@ -624,7 +617,10 @@ static int fat32_create_rootdir(t_param_disk *disk_car,const t_partition *partit
     if(++current_entry==(disk_car->sector_size*cluster_size/sizeof(struct msdos_dir_entry)))
     {
       unsigned int next_cluster;
-      disk_car->write(disk_car,disk_car->sector_size*cluster_size, buffer, partition->part_offset+(uint64_t)(start_data+(cluster-2)*cluster_size)*disk_car->sector_size);
+      if(disk_car->write(disk_car,disk_car->sector_size*cluster_size, buffer, partition->part_offset+(uint64_t)(start_data+(cluster-2)*cluster_size)*disk_car->sector_size)!=0)
+      {
+	display_message("Write error: Can't create FAT32 root cluster.\n");
+      }
       current_entry=0;
       memset(buffer,0,disk_car->sector_size*cluster_size);
       /* FIXME need to write fat32_get_next_free_cluster */
@@ -634,7 +630,10 @@ static int fat32_create_rootdir(t_param_disk *disk_car,const t_partition *partit
       cluster=next_cluster;
     }
   }
-  disk_car->write(disk_car,disk_car->sector_size*cluster_size, buffer, partition->part_offset+(uint64_t)(start_data+(cluster-2)*cluster_size)*disk_car->sector_size);
+  if(disk_car->write(disk_car,disk_car->sector_size*cluster_size, buffer, partition->part_offset+(uint64_t)(start_data+(cluster-2)*cluster_size)*disk_car->sector_size)!=0)
+  {
+    display_message("Write error: Can't create FAT32 root cluster.\n");
+  }
   set_next_cluster(disk_car,partition,UP_FAT32,reserved,cluster,FAT32_EOC);
   set_next_cluster(disk_car,partition,UP_FAT32,reserved+fat_length,cluster,FAT32_EOC);
 #ifdef DEBUG
@@ -833,7 +832,7 @@ static int analyse_dir_entries2(t_param_disk *disk_car,const t_partition *partit
   return root_size_max;
 }
 
-static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats)
+static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *partition, const unsigned int reserved, const int debug, const unsigned int dir_entries, const unsigned long int root_cluster, const unsigned int cluster_size, const unsigned int fat_length,const int interface, const upart_type_t upart_type, const unsigned int fats, char **current_cmd)
 {
   unsigned char orgboot[3*disk_car->sector_size];
   unsigned char newboot[3*disk_car->sector_size];
@@ -1033,6 +1032,7 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
     int do_exit=0;
     do
     {
+      int command;
       do_exit=0;
       aff_copy(stdscr);
       wmove(stdscr,4,0);
@@ -1043,7 +1043,6 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
       wmove(stdscr,8,0);
       if(memcmp(newboot,orgboot,DEFAULT_SECTOR_SIZE))	/* Only compare the first sector */
       {
-	options="DLWQ";
 	dump_2fat_info(fat_header, org_fat_header, upart_type,disk_car->sector_size);
 	wdoprintf(stdscr,"Extrapolated boot sector and current boot sector are different.\n");
 	if(error)
@@ -1051,27 +1050,51 @@ static int create_fat_boot_sector(t_param_disk *disk_car, const t_partition *par
 	  wdoprintf(stdscr,"Warning: Extrapolated boot sector have incorrect values.\n");
 	  ecrit_rapport("Warning: Extrapolated boot sector have incorrect values.\n");
 	}
+	options="DLWQ";
       }
       else
       {
 	dump_fat_info(fat_header, upart_type,disk_car->sector_size);
 	wdoprintf(stdscr,"Extrapolated boot sector and current boot sector are identical.\n");
       }
-      switch(toupper(wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuSaveBoot,8,options,MENU_HORIZ|MENU_BUTTON, 1)))
+      if(*current_cmd!=NULL)
       {
+	command='Q';
+	while(*current_cmd[0]==',')
+	  (*current_cmd)++;
+	if(strncmp(*current_cmd,"list",9)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='L';
+	}
+	else if(strncmp(*current_cmd,"dump",4)==0)
+	{
+	  (*current_cmd)+=4;
+	  command='D';
+	}
+	screen_buffer_to_log();
+      }
+      else
+	command=wmenuSelect(stdscr,INTER_DUMP_Y, INTER_DUMP_X, menuSaveBoot,8,options,MENU_HORIZ|MENU_BUTTON, 1);
+      switch(command)
+      {
+	case 'w':
 	case 'W':
 	  if(strchr(options,'W')!=NULL)
 	    do_write=1;
 	  break;
+	case 'd':
 	case 'D':
 	  if(strchr(options,'D')!=NULL)
 	  {
 	    dump2(stdscr,newboot,orgboot, (unsigned int)(upart_type==UP_FAT32?3*disk_car->sector_size:DEFAULT_SECTOR_SIZE));
 	  }
 	  break;
+	case 'l':
 	case 'L':
-	  dir_partition_fat_aux(stdscr,disk_car, partition, debug, fat_header,upart_type);
+	  dir_partition_fat_aux(stdscr,disk_car, partition, debug, fat_header,upart_type,current_cmd);
 	  break;
+	case 'q':
 	case 'Q':
 	  do_exit=1;
 	  break;
@@ -1126,8 +1149,10 @@ static int calcul_cluster_size(const upart_type_t upart_type, const unsigned lon
 
 static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_fat12, const int p_fat16, const int p_fat32,unsigned long int*fat_offset, const unsigned int sector_size)
 {
-  t_info_offset info_offset[0x200];
+  /* TODO: Handle data_size for I/O optimization, handle limited size of info_offset */
+  t_info_offset info_offset[sector_size];
   unsigned int nbr_offset=0;
+  int have_fat_signature=0;
   if(p_fat12!=0)
   {
     unsigned int i;
@@ -1146,7 +1171,8 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
       if((cluster!=0) && ((cluster&0x0ff8)!=(unsigned)0x0ff8) && (((cluster-i-1)*3)%(2*sector_size)==0))
       {
 	unsigned int j;
-	for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=(cluster-i-1)*3/(2*sector_size))&&(info_offset[j].fat_type==12);j++);
+	for(j=0;(j<nbr_offset) &&
+	    (info_offset[j].offset!=(cluster-i-1)*3/(2*sector_size) || info_offset[j].fat_type!=12);j++);
 	if(j<nbr_offset)
 	  info_offset[j].nbr++;
 	else
@@ -1180,7 +1206,8 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
       if((cluster!=0) && ((cluster&0x0ff8)!=(unsigned)0x0ff8) && (((cluster-i-1)*3+1)%(2*sector_size)==0))
       {
 	unsigned int j;
-	for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=((cluster-i-1)*3+1)/(2*sector_size))&&(info_offset[j].fat_type==12);j++);
+	for(j=0;(j<nbr_offset) &&
+	  (info_offset[j].offset!=((cluster-i-1)*3+1)/(2*sector_size) || info_offset[j].fat_type!=12);j++);
 	if(j<nbr_offset)
 	  info_offset[j].nbr++;
 	else
@@ -1214,7 +1241,8 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
       if((cluster!=0) && ((cluster&0x0ff8)!=(unsigned)0x0ff8) && (((cluster-i-1)*3+2)%(2*sector_size)==0))
       {
 	unsigned int j;
-	for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=((cluster-i-1)*3+2)/(2*sector_size))&&(info_offset[j].fat_type==12);j++);
+	for(j=0;(j<nbr_offset) &&
+	    (info_offset[j].offset!=((cluster-i-1)*3+2)/(2*sector_size) || info_offset[j].fat_type!=12);j++);
 	if(j<nbr_offset)
 	  info_offset[j].nbr++;
 	else
@@ -1235,6 +1263,22 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
       high++;
       i++;
     }
+    if((buffer[0]==0xF0 || buffer[0]>=0xF8) && buffer[1]==0xFF)
+    {
+      unsigned int j;
+      for(j=0;(j<nbr_offset) &&
+	  (info_offset[j].offset!=0 || info_offset[j].fat_type!=12);j++);
+      if(j<nbr_offset)
+	info_offset[j].nbr++;
+      else
+      {
+	info_offset[nbr_offset].offset=0;
+	info_offset[nbr_offset].nbr=1;
+	info_offset[nbr_offset].fat_type=12;
+	nbr_offset++;
+      }
+      have_fat_signature=1;
+    }
   }
   if(p_fat16!=0)
   {
@@ -1266,7 +1310,8 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
 	unsigned long int cluster=le16(p16[i]);
 	if((cluster!=0) && ((cluster&0x0fff8)!=(unsigned)0x0fff8)&&((cluster-i-1)%(sector_size/2)==0))
 	{
-	  for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=(cluster-i-1)/(sector_size/2))&&(info_offset[j].fat_type==16);j++);
+	  for(j=0;(j<nbr_offset) &&
+	    (info_offset[j].offset!=(cluster-i-1)/(sector_size/2) || info_offset[j].fat_type!=16);j++);
 	  if(j<nbr_offset)
 	    info_offset[j].nbr++;
 	  else
@@ -1279,6 +1324,21 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
 	}
       }
     }
+    if((buffer[0]==0xF0 || buffer[0]>=0xF8) && buffer[1]==0xFF
+	&& buffer[2]==0xFF && ((buffer[3] & 0xF7)==0xF7))
+    {
+      for(j=0;(j<nbr_offset)&& (info_offset[j].offset!=0 || info_offset[j].fat_type!=16);j++);
+      if(j<nbr_offset)
+	info_offset[j].nbr++;
+      else
+      {
+	info_offset[nbr_offset].offset=0;
+	info_offset[nbr_offset].nbr=1;
+	info_offset[nbr_offset].fat_type=16;
+	nbr_offset++;
+      }
+      have_fat_signature=1;
+    }
   }
   if(p_fat32!=0)
   {
@@ -1310,7 +1370,8 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
 	unsigned long int cluster=le32(p32[i])&0x0FFFFFFF;
 	if((cluster!=0) && ((cluster&0x0ffffff8)!=(unsigned)0x0ffffff8)&&((cluster-i-1)%(sector_size/4)==0))
 	{
-	  for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=(cluster-i-1)/(sector_size/4))&&(info_offset[j].fat_type==32);j++);
+	  for(j=0;(j<nbr_offset) &&
+	      ((info_offset[j].offset!=(cluster-i-1)/(sector_size/4)) || (info_offset[j].fat_type!=32));j++);
 	  if(j<nbr_offset)
 	    info_offset[j].nbr++;
 	  else
@@ -1323,6 +1384,22 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
 	}
       }
     }
+    if((buffer[0]==0xF0 || buffer[0]>=0xF8) && buffer[1]==0xFF &&
+	buffer[2]==0xFF && ((buffer[3]==0x0F) ||(buffer[3]==0xFF)) &&
+	buffer[4]==0xFF && buffer[5]==0xFF && buffer[6]==0xFF)
+    {
+      for(j=0;(j<nbr_offset)&&(info_offset[j].offset!=0 || info_offset[j].fat_type!=32);j++);
+      if(j<nbr_offset)
+	info_offset[j].nbr++;
+      else
+      {
+	info_offset[nbr_offset].offset=0;
+	info_offset[nbr_offset].nbr=1;
+	info_offset[nbr_offset].fat_type=32;
+	nbr_offset++;
+      }
+      have_fat_signature=1;
+    }
   }
   if(nbr_offset>0)
   {
@@ -1333,7 +1410,7 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
       if(info_offset[j].nbr>info_offset[best_j].nbr)
 	best_j=j;
     }
-    if(info_offset[best_j].nbr>10)
+    if(info_offset[best_j].nbr>10 || have_fat_signature>0)
     {
       *fat_offset=info_offset[best_j].offset;
       return info_offset[best_j].fat_type;
@@ -1342,58 +1419,87 @@ static unsigned int fat_find_fat_start(const unsigned char *buffer,const int p_f
   return 0;
 }
 
-static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,const unsigned long int max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface,unsigned int *nbr_offset,t_info_offset *info_offset)
+static int fat_find_type(t_param_disk *disk_car,const t_partition *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface,unsigned int *nbr_offset,t_info_offset *info_offset)
 {
-  unsigned long int offset;
-  uint64_t hd_offset;
+  uint64_t offset;
+  unsigned long int old_percent=0;
+  unsigned char buffer[disk_car->sector_size];
+  int ind_stop=0;
   if(debug>0)
   {
-    ecrit_rapport("fat_find_type(partition,max_offset=%ld,p_fat12=%d,p_fat16=%d,p_fat32=%d,*f_fat12,*f_fat16,*f_fat32,debug=%d,dump_ind=%d)\n",max_offset,p_fat12,p_fat16,p_fat32,debug,dump_ind);
+    ecrit_rapport("fat_find_type(partition,max_offset=%lu,p_fat12=%d,p_fat16=%d,p_fat32=%d,*f_fat12,*f_fat16,*f_fat32,debug=%d,dump_ind=%d)\n",(long unsigned)(max_offset/disk_car->sector_size),p_fat12,p_fat16,p_fat32,debug,dump_ind);
   }
-  for(offset=1,hd_offset=partition->part_offset+disk_car->sector_size;(offset<max_offset)&&(*nbr_offset<0x200);offset++,hd_offset+=disk_car->sector_size)
+  if(interface)
   {
-    unsigned char buffer[disk_car->sector_size];
-    if(interface && ((offset&0x1FF)==0))
-    {
       wmove(stdscr,8,0);
       wdoprintf(stdscr,"FAT : %s%s%s?\n",p_fat12?"12 ":"", p_fat16?"16 ":"", p_fat32?"32 ":"");
+      wmove(stdscr,22,0);
+      wattrset(stdscr,A_STANDOUT);
+      waddstr(stdscr,"  Stop  ");
+      wattroff(stdscr,A_STANDOUT);
+  }
+  for(offset=disk_car->sector_size;(offset<max_offset)&&(*nbr_offset<0x200)&&!ind_stop;offset+=sizeof(buffer))
+  {
+    unsigned long int percent=offset*100/max_offset;
+    if(interface && (percent!=old_percent))
+    {
       wmove(stdscr,8,30);
       wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
-      wdoprintf(stdscr,"analyse %10lu/%lu",offset,max_offset);
+      wdoprintf(stdscr,"Searching for FAT table %lu%%",percent);
+      old_percent=percent;
       wrefresh(stdscr);
+      ind_stop|=check_enter_or_s(stdscr);
     }
-    if(disk_car->read(disk_car,sizeof(buffer), buffer, hd_offset)==0)
+    if(disk_car->read(disk_car,sizeof(buffer), buffer, partition->part_offset+offset)==0)
     {
       unsigned long int fat_offset=0;
       unsigned int fat_type;
       fat_type=fat_find_fat_start(buffer,p_fat12,p_fat16,p_fat32,&fat_offset,disk_car->sector_size);
-      if(fat_type!=0 && fat_offset<offset)
+      if(fat_type!=0 && fat_offset<=(offset/disk_car->sector_size))
       {
 	unsigned int j;
 	if(debug>1)
 	{
-	  ecrit_rapport("fat_find_fat_start FAT%u at %lu:%lu\n",fat_type,offset-fat_offset,offset);
+	  ecrit_rapport("fat_find_fat_start FAT%u at %lu:%lu\n", fat_type,
+	      (long unsigned)(offset/disk_car->sector_size-fat_offset),
+	      (long unsigned)(offset/disk_car->sector_size));
 	}
-	for(j=0;(j<*nbr_offset)&& !((info_offset[j].offset==offset-fat_offset)&&(info_offset[j].fat_type==fat_type));j++);
+	for(j=0;(j<*nbr_offset)&& !((info_offset[j].offset==offset/disk_car->sector_size-fat_offset)&&(info_offset[j].fat_type==fat_type));j++);
 	if(j<*nbr_offset)
 	  info_offset[j].nbr++;
 	else
 	{
+	  unsigned int new_info=0;
 	  if(*nbr_offset<0x200)
 	  {
-	    info_offset[*nbr_offset].offset=offset-fat_offset;
-	    info_offset[*nbr_offset].nbr=1;
-	    info_offset[*nbr_offset].fat_type=fat_type;
+	    new_info=*nbr_offset;
 	    (*nbr_offset)++;
 	  }
+	  else
+	  { /* Overwrite the last information field with the lower nbr */
+	    for(j=0;j<0x200;j++)
+	    {
+	      if(info_offset[j].nbr <= info_offset[new_info].nbr)
+		new_info=j;
+	    }
+	  }
+	  info_offset[new_info].offset=offset/disk_car->sector_size-fat_offset;
+	  info_offset[new_info].nbr=1;
+	  info_offset[new_info].fat_type=fat_type;
 	}
       }
     }
   }
+  if(interface)
+  {
+    wmove(stdscr,22,0);
+    wclrtoeol(stdscr);
+    wrefresh(stdscr);
+  }
   return 0;
 }
 
-static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved, unsigned int*fat_length, const t_partition *partition,const unsigned long int max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats)
+static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved, unsigned int*fat_length, const t_partition *partition,const uint64_t max_offset,const int p_fat12,const int p_fat16,const int p_fat32,const int debug,const int dump_ind,const int interface, const unsigned int expert, unsigned int *fats)
 {
   unsigned int nbr_offset=0;
   unsigned int i;
@@ -1503,16 +1609,23 @@ static upart_type_t fat_find_info(t_param_disk *disk_car,unsigned int*reserved,
   }
   if(expert>0 && (interface>0))
   {
-    return select_fat_info(info_offset,nbr_offset,reserved,fat_length,max_offset,fats);
+    return select_fat_info(info_offset,nbr_offset,reserved,fat_length,max_offset/disk_car->sector_size,fats);
   }
   return upart_type;
 }
 
-static upart_type_t select_fat_info(const t_info_offset *info_offset, const unsigned int nbr_offset,unsigned int*reserved, unsigned int*fat_length, const unsigned long int max_offset, unsigned int *fats)
+static upart_type_t select_fat_info(const t_info_offset *info_offset, const unsigned int nbr_offset,unsigned int*reserved, unsigned int*fat_length, const unsigned long int max_sector_offset, unsigned int *fats)
 {
   unsigned int i;
   int reserved_can_be_one=0;
   unsigned long int fat2_location=*reserved+*fat_length;
+  struct MenuItem menuSelectFAT[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Proceed","Set FAT table location"},
+    { 0, NULL, NULL }
+  };
   aff_buffer(BUFFER_RESET,"Q");
   aff_buffer(BUFFER_ADD,"Potential FAT location\n");
   aff_buffer(BUFFER_ADD,"FAT - sector - score\n");
@@ -1526,12 +1639,12 @@ static upart_type_t select_fat_info(const t_info_offset *info_offset, const unsi
   }
   aff_copy(stdscr);
   wmove(stdscr,4,0);
-  aff_buffer(BUFFER_DISPLAY,"Q",stdscr);
+  screen_buffer_display(stdscr,"",menuSelectFAT);
   wmove(stdscr,23,0);
-  *reserved=ask_number(*reserved,0,max_offset,"FAT1 location (Number of reserved sector) ");
+  *reserved=ask_number(*reserved,0,max_sector_offset,"FAT1 location (Number of reserved sector) ");
   if(*reserved>0)
   {
-    fat2_location=ask_number(fat2_location,0,max_offset,"FAT2 location ");
+    fat2_location=ask_number(fat2_location,0,max_sector_offset,"FAT2 location ");
     if(fat2_location>*reserved)
     {
       *fat_length=fat2_location-*reserved;
@@ -1609,19 +1722,7 @@ static int find_cluster_size(t_param_disk *disk_car, t_partition *partition, con
       wclrtoeol(stdscr);
       wdoprintf(stdscr,"Search subdirectory %10lu/%lu %u",(unsigned long)(offset/disk_car->sector_size),(unsigned long)(partition->part_size/disk_car->sector_size),nbr_subdir);
       wrefresh(stdscr);
-      switch(wgetch_nodelay(stdscr))
-      {
-	case KEY_ENTER:
-#ifdef PADENTER
-	case PADENTER:
-#endif
-	case '\n':
-	case '\r':
-	case 's':
-	case 'S':
-	  ind_stop=1;
-	  break;
-      }
+      ind_stop|=check_enter_or_s(stdscr);
     }
     if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset+offset)==0)
     {
@@ -1752,13 +1853,12 @@ static upart_type_t no_of_cluster2part_type(const unsigned long int no_of_cluste
   return UP_FAT32;
 }
 
-int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface,const unsigned int expert)
+int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int debug, const int dump_ind,const int interface,const unsigned int expert, char**current_cmd)
 {
   unsigned long int data_size;
-  unsigned long int fat_length_max;
   unsigned long int max_offset;
   unsigned int fat_length=0;
-  unsigned int cluster_size_min=1;
+  unsigned int cluster_size_min=disk_car->sector_size;
   unsigned int cluster_size=0;
   unsigned int reserved=0;
   unsigned int dir_entries=0;
@@ -1798,30 +1898,34 @@ int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int deb
     aff_part_rapport(disk_car,partition);
     ecrit_rapport("rebuild_FAT_BS p_fat12 %d, p_fat16 %d, p_fat32 %d\n", p_fat12,p_fat16,p_fat32);
   }
-  /* Set fat_length_max */
-  if(p_fat32)
-  {	/* Cluster 512 bytes */
-    fat_length_max=partition->part_size/disk_car->sector_size/cluster_size_min*4/disk_car->sector_size;
-  }
-  else
-    if(p_fat16)
-    {
-      while(partition->part_size/disk_car->sector_size/cluster_size_min>(1<<16))
-	cluster_size_min*=2;
-      fat_length_max=partition->part_size/disk_car->sector_size/cluster_size_min*2/disk_car->sector_size;
+  {
+    /* Set fat_length_max */
+    unsigned long int fat_length_max;
+    if(p_fat32)
+    {	/* Cluster 512 bytes */
+      fat_length_max=partition->part_size/cluster_size_min*4;
     }
     else
+      if(p_fat16)
+      {
+	while(partition->part_size/cluster_size_min > (1<<16))
+	  cluster_size_min*=2;
+	fat_length_max=partition->part_size/cluster_size_min*2;
+      }
+      else
+      {
+	while(partition->part_size/cluster_size_min > (1<<12))
+	  cluster_size_min*=2;
+	fat_length_max=partition->part_size/cluster_size_min*1.5;
+      }
+    fat_length_max=fat_length_max/disk_car->sector_size*disk_car->sector_size;
+    if(debug>1)
     {
-      while(partition->part_size/disk_car->sector_size/cluster_size_min>(1<<12))
-	cluster_size_min*=2;
-      fat_length_max=partition->part_size/disk_car->sector_size/cluster_size_min*1.5/disk_car->sector_size;
+      ecrit_rapport("cluster_size_min %u sectors\n",cluster_size_min/disk_car->sector_size);
+      ecrit_rapport("fat_length_max %ld sectors\n", fat_length_max/disk_car->sector_size);
     }
-  if(debug>1)
-  {
-    ecrit_rapport("cluster_size_min %u\n",cluster_size_min);
-    ecrit_rapport("fat_length_max %ld\n", fat_length_max);
+    max_offset=fat_length_max+64*disk_car->sector_size;
   }
-  max_offset=fat_length_max+64;
   /*
      if(debug>1)
      ecrit_rapport("search_fat16(partition,max_offset=%d,p_fat12=%d,p_fat16=%d,p_fat32=%d,debug=%d,dump_ind=%d)\n",max_offset,p_fat12,p_fat16,p_fat32,debug,dump_ind);
@@ -2007,7 +2111,7 @@ int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int deb
       }
       if(expert>0)
       {
-	cluster_size=reserved=ask_number(cluster_size,0,128,"cluster size ");
+	cluster_size=ask_number(cluster_size,0,128,"cluster size ");
 	switch(cluster_size)
 	{
 	  case 1:
@@ -2036,13 +2140,14 @@ int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int deb
       fat32_free_info(disk_car,partition,reserved,data_size/cluster_size,&first_free_cluster,&free_cluster_count);
       /* FAT32 : Find root cluster */
       root_cluster=fat32_find_root_cluster(disk_car,partition,cluster_size,data_size/cluster_size,reserved,fat_length,interface,debug,expert,first_free_cluster,fats);
-      /*
-      if(debug>1)
+      if(expert>0)
       {
-	ecrit_rapport("override root_cluster=2\n");
-	root_cluster=2;
+	root_cluster=ask_number(root_cluster,2,data_size/cluster_size+1,"root cluster ");
+	if(debug>1)
+	{
+	  ecrit_rapport("root_cluster=%lu (new)\n",root_cluster);
+	}
       }
-      */
     }
     if(interface)
     {
@@ -2050,7 +2155,7 @@ int rebuild_FAT_BS(t_param_disk *disk_car, t_partition *partition, const int deb
       wclrtoeol(stdscr);
       wrefresh(stdscr);
     }
-    create_fat_boot_sector(disk_car,partition, reserved, debug,dir_entries,root_cluster,cluster_size,fat_length,interface,upart_type,fats);
+    create_fat_boot_sector(disk_car,partition, reserved, debug,dir_entries,root_cluster,cluster_size,fat_length,interface,upart_type,fats,current_cmd);
     if(debug)
     {
       ecrit_rapport("\n");
