@@ -37,7 +37,7 @@ const t_file_hint file_hint_crw= {
   .description="Canon Raw picture",
   .min_header_distance=0,
   .min_filesize=0,
-  .max_filesize=0,
+  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .stream=0,
   .recover=1,
   .header_check=&header_check_crw,
