@@ -31,7 +31,7 @@
 
 static int header_check_dir(const unsigned char *buffer, const unsigned int buffer_size);
 const t_file_hint file_hint_dir= {
-  .extension="",
+  .extension=NULL,
   .description="FAT subdirectory",
   .min_header_distance=0,
   .min_filesize=0,
