@@ -35,7 +35,7 @@ const t_file_hint file_hint_mp3= {
   .description="MP3 audio (MPEG ADTS, layer III, v1), prone to false positive",
   .min_header_distance=0,
   .min_filesize=0,
-  .max_filesize=0,
+  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
   .stream=1,
   .recover=1,
   .header_check=&header_check_mp3,
