@@ -2,7 +2,7 @@
 
     File: fnctdsk.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -38,6 +38,7 @@
 #include "testdisk.h"
 #include "intrf.h"
 #include "analyse.h"
+
 static unsigned int get_geometry_from_list_part_aux(const t_param_disk *disk_car, const t_list_part *list_part, const int debug);
 
 unsigned long int CHS2LBA(const t_param_disk *disk_car,const t_CHS*CHS)
@@ -87,7 +88,9 @@ void dup_t_partition(t_partition *dest, const t_partition *src)
   dest->part_size=src->part_size;
   dest->boot_sector=src->boot_sector;
   dest->blocksize=src->blocksize;
-  dest->part_type=src->part_type;
+  dest->part_type_i386=src->part_type_i386;
+  dest->part_type_sun=src->part_type_sun;
+  dest->part_type_mac=src->part_type_mac;
   dest->upart_type=src->upart_type;
   dest->status=src->status;
   dest->order=src->order;
@@ -97,26 +100,6 @@ void dup_t_partition(t_partition *dest, const t_partition *src)
   dest->arch=src->arch;
 }
 
-int read_MBR(t_param_disk *disk_car,void *buffer)
-{
-  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
-  {
-    wdoprintf(stdscr,msg_PART_RD_ERR);
-    return 1;
-  }
-  return 0;
-}
-
-int write_MBR(t_param_disk *disk_car,void *buffer)
-{
-  if(disk_car->write(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
-  {
-    wdoprintf(stdscr,msg_PART_WR_ERR);
-    return 1;
-  }
-  return 0;
-}
-
 t_list_disk *insert_new_disk(t_list_disk *list_disk, t_param_disk *disk_car)
 {
   if(disk_car==NULL)
@@ -170,7 +153,9 @@ t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_e
       if(dont_free_part!=2 &&
 	(next!=NULL)&&(next->part->part_offset==new_element->part->part_offset) &&
 	(next->part->part_size==new_element->part->part_size) &&
-	(next->part->part_type==new_element->part->part_type) &&
+	(next->part->part_type_i386==new_element->part->part_type_i386) &&
+	(next->part->part_type_mac==new_element->part->part_type_mac) &&
+	(next->part->part_type_sun==new_element->part->part_type_sun) &&
 	(next->part->upart_type==new_element->part->upart_type || new_element->part->upart_type==UP_UNK))
       { /*CGR 2004/05/31*/
 	if(next->part->status==STATUS_DELETED)
@@ -200,6 +185,25 @@ t_list_part *insert_new_partition_aux(t_list_part *list_part, t_list_part *new_e
   }
 }
 
+int delete_list_disk(t_list_disk *list_disk)
+{
+  t_list_disk *element_disk;
+  int write_used=0;
+  for(element_disk=list_disk;element_disk!=NULL;)
+  {
+    t_list_disk *element_disk_next=element_disk->next;
+    write_used|=element_disk->disk->write_used;
+    if(element_disk->disk->clean!=NULL)
+      element_disk->disk->clean(element_disk->disk);
+    if(element_disk->disk->device!=NULL)
+      FREE(element_disk->disk->device);
+    FREE(element_disk->disk);
+    FREE(element_disk);
+    element_disk=element_disk_next;
+  }
+  return write_used;
+}
+
 int check_list_part(t_list_part *list_part)
 {
   t_list_part *prev=NULL;
@@ -270,7 +274,9 @@ void  partition_reset(t_partition *partition)
   partition->part_size=(uint64_t)0;
   partition->boot_sector=0;
   partition->blocksize=0;
-  partition->part_type=P_NO_OS;
+  partition->part_type_i386=P_NO_OS;
+  partition->part_type_sun=PSUN_UNK;
+  partition->part_type_mac=PMAC_UNK;
   partition->upart_type=UP_UNK;
   partition->status=STATUS_DELETED;
   partition->order=NO_ORDER;
@@ -306,10 +312,14 @@ static unsigned int get_geometry_from_list_part_aux(const t_param_disk *disk_car
     t_CHS end;
     offset2CHS(disk_car,element->part->part_offset,&start);
     offset2CHS(disk_car,element->part->part_offset+element->part->part_size-1,&end);
-    if(start.sector==1 && start.head<=1 && end.head==disk_car->CHS.head)
+    if(start.sector==1 && start.head<=1)
     {
-      /* Doesn't check if end.sector==disk_car->CHS.sector */
       nbr++;
+      if(end.head==disk_car->CHS.head)
+      {
+	nbr++;
+	/* Doesn't check if end.sector==disk_car->CHS.sector */
+      }
     }
   }
   if(nbr>0)
@@ -356,3 +366,4 @@ unsigned int get_geometry_from_list_part(const t_param_disk *disk_car, const t_l
   FREE(new_disk_car);
   return head_max;
 }
+
