@@ -23,7 +23,7 @@
 #include <config.h>
 #endif
  
-#include <ctype.h>      /* toupper, tolower */
+#include <ctype.h>      /* tolower */
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
 #endif
@@ -46,15 +46,16 @@
 #define RO 1
 #define RW 0
 #define MAX_SEARCH_LOCATION 1024
+extern const t_arch_fnct arch_mac;
 extern const t_arch_fnct arch_none;
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part,const int align);
 static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part);
 static t_list_part *reduce_structure(t_list_part *list_part);
-static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
-static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert);
+static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align,const int debug);
+static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd);
 static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part_bad);
-static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder);
-static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind);
+static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder, char **current_cmd);
+static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind, const int interface);
 
 static void align_structure(const t_param_disk *disk_car, t_list_part *list_part, const int align)
 {
@@ -208,14 +209,13 @@ static int interface_part_bad(t_param_disk *disk_car,t_list_part *list_part)
 	}
 	break;
       default:
-/*	ecrit_rapport("ask_structure car=%x\n",car); */
 	break;
     }
   } while(quit==0);
   return 0;
 }
 
-static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder)
+static void warning_geometry(t_param_disk *disk_car, const unsigned int recommanded_heads_per_cylinder, char**current_cmd)
 {
   int quit=0;
   aff_copy(stdscr);
@@ -223,14 +223,18 @@ static void warning_geometry(t_param_disk *disk_car, const unsigned int recomman
   wdoprintf(stdscr,"%s",disk_car->description(disk_car));
   wmove(stdscr,6,0);
   ecrit_rapport("Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",disk_car->CHS.head+1,recommanded_heads_per_cylinder);
-  wdoprintf(stdscr,"Warning: the current number of heads per cylinder is %u but the correct value may be %u.\n",disk_car->CHS.head+1,recommanded_heads_per_cylinder);
+  wdoprintf(stdscr,"Warning: the current number of heads per cylinder is %u",disk_car->CHS.head+1);
   wmove(stdscr,7,0);
-  wdoprintf(stdscr,"Use the Geometry menu to change this value.\n");
+  wdoprintf(stdscr,"but the correct value may be %u.",recommanded_heads_per_cylinder);
+  wmove(stdscr,8,0);
+  wdoprintf(stdscr,"Use the Geometry menu to change this value.");
   wmove(stdscr,22,0);
   wattrset(stdscr,A_STANDOUT);
   wdoprintf(stdscr,"[  Quit  ]");
   wattroff(stdscr,A_STANDOUT);
   wrefresh(stdscr);
+  if(*current_cmd!=NULL)
+    return;
   do
   {
     int car;
@@ -248,17 +252,34 @@ static void warning_geometry(t_param_disk *disk_car, const unsigned int recomman
 	quit=1;
 	break;
       default:
-/*	ecrit_rapport("ask_structure car=%x\n",car); */
 	break;
     }
   } while(quit==0);
 }
 
-static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind)
+static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_part, t_list_part *list_part_bad, const int debug, const int dump_ind, const int interface)
 {
   t_list_part *element;
-  /* Little trick to detect Linux software Raid quickly */
+  unsigned int test_current=0;
+  unsigned int test_total=0;
+  int ind_stop=0;
   for(element=list_part_bad;element!=NULL;element=element->next)
+  {
+    if(element->part->upart_type!=UP_RAID)
+      test_total+=6;
+  }
+  if(interface)
+  {
+    aff_copy(stdscr);
+    wmove(stdscr,4,0);
+    wdoprintf(stdscr,"%s",disk_car->description(disk_car));
+    wmove(stdscr,22,0);
+    wattrset(stdscr,A_STANDOUT);
+    waddstr(stdscr,"  Stop  ");
+    wattroff(stdscr,A_STANDOUT);
+  }
+  /* Little trick to detect Linux software Raid quickly */
+  for(element=list_part_bad;element!=NULL && ind_stop==0;element=element->next)
   {
     if(element->part->upart_type!=UP_RAID)
     {
@@ -266,12 +287,25 @@ static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_p
       struct mdp_superblock_s *sb=(struct mdp_superblock_s*)&buffer;
       unsigned int disk_factor;
       unsigned int help_factor;
-      for(disk_factor=1;disk_factor<=6;disk_factor++)
-      { /* disk_factor=1, detect Raid 1 */
+      if(debug>1)
+      {
+	ecrit_rapport("search_part_raid:\n");
+	aff_part_rapport(disk_car,element->part);
+      }
+      for(disk_factor=6;disk_factor>=1 && ind_stop==0;disk_factor--)
+      { /* disk_factor=1, detect Raid 0/1 */
         /* disk_factor>1, detect Raid 5 */
-	for(help_factor=0;help_factor<=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES+3;help_factor++)
+	if(interface)
 	{
-	  uint64_t offset=MD_NEW_SIZE_SECTORS((element->part->part_size/disk_factor+help_factor*MD_RESERVED_BYTES-1)/MD_RESERVED_BYTES*MD_RESERVED_BYTES/512)*512;
+	  wmove(stdscr,6,0);
+	  wdoprintf(stdscr,"Searching for software Raid partition: %02d%%",test_current*100/test_total);
+	  wrefresh(stdscr);
+	  ind_stop|=check_enter_or_s(stdscr);
+	}
+	test_current++;
+	for(help_factor=0;help_factor<=MD_MAX_CHUNK_SIZE/MD_RESERVED_BYTES+3 && ind_stop==0;help_factor++)
+	{
+	  uint64_t offset=(uint64_t)MD_NEW_SIZE_SECTORS((element->part->part_size/disk_factor+help_factor*MD_RESERVED_BYTES-1)/MD_RESERVED_BYTES*MD_RESERVED_BYTES/512)*512;
 	  if(debug>1)
 	  {
 	    ecrit_rapport("search_part_raid offset=%llu\n",(long long unsigned)offset);
@@ -283,7 +317,9 @@ static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_p
 	    if(test_MD(disk_car,sb,new_partition,debug,dump_ind)==0)
 	    {
 	      set_MD_info(disk_car,sb,new_partition,debug,dump_ind);
-	      new_partition->part_type=P_RAID;
+	      new_partition->part_type_i386=P_RAID;
+	      new_partition->part_type_sun=PSUN_RAID;
+	      new_partition->part_type_mac=PMAC_LINUX;
 	      new_partition->part_size=offset+MD_RESERVED_BYTES;
 	      aff_part_rapport(disk_car,new_partition);
 	      list_part=insert_new_partition(list_part,new_partition);
@@ -298,11 +334,12 @@ static t_list_part *search_part_raid(t_param_disk *disk_car, t_list_part *list_p
   return list_part;
 }
 
-t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int debug, const int dump_ind, const int fast_mode,const int interface)
+t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int debug, const int dump_ind, const int fast_mode, const int interface)
 {
   int ind_stop=0;
   uint64_t try_offset[MAX_SEARCH_LOCATION];
   unsigned int try_offset_nbr=0;
+  unsigned int old_permil=0;
   t_CHS start;
   t_list_part *list_part=NULL;
   t_list_part *list_part_bad=NULL;
@@ -322,10 +359,15 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
   {
     if(interface!=0)
     {
-      wmove(stdscr,ANALYSE_Y,ANALYSE_X);
-      wclrtoeol(stdscr);
-      wdoprintf(stdscr,"Analyse cylinder %5u/%u",start.cylinder,disk_car->CHS.cylinder);
-      wrefresh(stdscr);
+      unsigned int permil=start.cylinder*1000/(disk_car->CHS.cylinder>0?disk_car->CHS.cylinder:1);
+      if(permil>old_permil || disk_car->arch!=&arch_mac)
+      {
+	wmove(stdscr,ANALYSE_Y,ANALYSE_X);
+	wclrtoeol(stdscr);
+	wdoprintf(stdscr,"Analyse cylinder %5u/%u: %02u%%",start.cylinder,disk_car->CHS.cylinder,start.cylinder*100/(disk_car->CHS.cylinder>0?disk_car->CHS.cylinder:1));
+	wrefresh(stdscr);
+	old_permil=permil;
+      }
     }
     for(;(start.head<=disk_car->CHS.head)&&(ind_stop==0);start.head++)
     {
@@ -333,19 +375,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
       int part_offset;
       if(interface!=0)
       {
-	switch(wgetch_nodelay(stdscr))
-	{
-	  case KEY_ENTER:
-#ifdef PADENTER
-	  case PADENTER:
-#endif
-	  case '\n':
-	  case '\r':
-	  case 's':
-	  case 'S':
-	    ind_stop=1;
-	    break;
-	}
+	ind_stop|=check_enter_or_s(stdscr);
       }
       for(start.sector=1,sector_inc=1,part_offset=0;(start.sector<=disk_car->CHS.sector);start.sector+=sector_inc)
       {
@@ -357,9 +387,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	for(i=0;i<try_offset_nbr;i++)
 	{
 	  if(try_offset[i]==partition->part_offset)
-	  {
 	    search_now=1;
-	  }
 	}
 	if((start.sector==1) && ((start.head<=2)||(fast_mode==2)||(search_now>0)))
 	{
@@ -472,20 +500,24 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 	}
 	if(res>0)
 	{
+	  partition->status=STATUS_DELETED;
 	  aff_part_rapport(disk_car,partition);
-	  if(disk_car->arch->is_part_type_ok(partition->part_type)!=0 && partition->part_size>1 && (partition->part_offset>0 || disk_car->arch==&arch_none))
+	  aff_part_buffer(AFF_PART_ORDER,disk_car,partition);
+	  if(interface)
 	  {
-	    uint64_t pos_fin=partition->part_offset+partition->part_size-1;
+	    aff_buffer(BUFFER_SHOW,"Q");
+	  }
+	  if(disk_car->arch->is_part_known(partition)!=0 && partition->part_size>1 && (partition->part_offset>0 || disk_car->arch==&arch_none))
+	  {
+	    uint64_t pos_fin;
+	    pos_fin=partition->part_offset+partition->part_size-1;
 	    if(pos_fin<=disk_car->disk_size)
 	    {
 	      {
 		t_partition *new_partition=partition_new();
 		dup_t_partition(new_partition,partition);
-		new_partition->status=STATUS_DELETED;
-		aff_part_buffer(AFF_PART_ORDER,disk_car,new_partition);
-		/* aff_part_buffer must be before insert_new_partition because
-		   insert_new_partition can free the memory */
 		list_part=insert_new_partition(list_part,new_partition);
+		/* Never use new_partition after insert_new_partition, it may have been freed */
 	      }
 	      {
 		uint64_t next_part_offset=partition->part_offset+partition->part_size-1+1;
@@ -510,36 +542,29 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
 		  }
 		}
 	      }
-	      if(interface)
-	      {
-		aff_buffer(BUFFER_SHOW,"Q");
-	      }
 	      if((paranoid==0) && (partition->part_offset+partition->part_size-disk_car->sector_size> CHS2offset(disk_car,&start)))
 	      {
 		offset2CHS(disk_car,partition->part_offset+partition->part_size-1,&start);
 		part_offset=0;
 		sector_inc=1;
 	      }
-	    } else {
-	      if(debug>0)
-	      {
-		aff_part_rapport(disk_car,partition);
-	      }
-	      if(pos_fin>disk_car->disk_size)
+	    }
+	    else
+	    {
 	      {
 		t_partition *new_partition=partition_new();
 		dup_t_partition(new_partition,partition);
-		new_partition->status=STATUS_DELETED;
 		list_part_bad=insert_new_partition(list_part_bad,new_partition);
-		if(debug>0)
-		{
-		  ecrit_rapport("This partition ends after the disk limits. ");
-		}
+		/* Never use new_partition after insert_new_partition, it may have been freed */
 	      }
+	      ecrit_rapport("This partition ends after the disk limits. ");
 	      if(debug>0)
 	      {
 		ecrit_rapport("(start=%lu, size=%lu, end=%lu, disk end=%lu)\n",
-		    (unsigned long)(partition->part_offset/disk_car->sector_size),(unsigned long)(partition->part_size/disk_car->sector_size), (unsigned long)(pos_fin/disk_car->sector_size),(unsigned long)(disk_car->disk_size/disk_car->sector_size));
+		    (unsigned long)(partition->part_offset/disk_car->sector_size),
+		    (unsigned long)(partition->part_size/disk_car->sector_size),
+		    (unsigned long)(pos_fin/disk_car->sector_size),
+		    (unsigned long)(disk_car->disk_size/disk_car->sector_size));
 	      }
 	    }
 	  }
@@ -555,7 +580,7 @@ t_list_part *search_part(t_param_disk *disk_car, const int paranoid, const int d
     }
   }
   FREE(partition);
-  list_part=search_part_raid(disk_car,list_part,list_part_bad,debug,dump_ind);
+  list_part=search_part_raid(disk_car,list_part,list_part_bad,debug,dump_ind,interface);
   if(interface!=0 && list_part_bad!=NULL)
   {
     interface_part_bad(disk_car,list_part_bad);
@@ -611,7 +636,6 @@ static void ask_mbr_order(t_param_disk *disk_car,t_list_part *list_part)
     {
       ecrit_rapport("\nBUG part %d, nbr_prim %d\n", part, nbr_prim);
     }
-    /* */
     for(i=0;i<nbr_prim;i++)
     {
       wmove(stdscr,5+2+i,0);
@@ -704,7 +728,7 @@ static t_list_part *reduce_structure(t_list_part *list_part)
   return list_part;
 }
 
-static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align, const int debug)
+static t_list_part *add_ext_part_i386(t_param_disk *disk_car, t_list_part *list_part, const int max_ext, const int align, const int debug)
 {
   /* list_part need to be sorted! */
   /* All extended partitions of an P_EXTENDX are P_EXTENDED */
@@ -754,7 +778,7 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
   {
     if(debug>1)
     {
-      ecrit_rapport("add_ext_part: max\n");
+      ecrit_rapport("add_ext_part_i386: max\n");
     }
     if(deb->prev==NULL)
     {
@@ -820,7 +844,7 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
   {
     if(debug>1)
     {
-      ecrit_rapport("add_ext_part: min\n");
+      ecrit_rapport("add_ext_part_i386: min\n");
     }
     offset2CHS(disk_car,deb->part->part_offset-1,&start);
     start.sector=1;
@@ -848,14 +872,14 @@ static t_list_part *add_ext_part(t_param_disk *disk_car, t_list_part *list_part,
   }
   new_partition=partition_new();
   new_partition->order=order;
-  new_partition->part_type=(end.cylinder>1023?P_EXTENDX:P_EXTENDED);
+  new_partition->part_type_i386=(end.cylinder>1023?P_EXTENDX:P_EXTENDED);
   new_partition->status=STATUS_EXT;
   new_partition->part_offset=CHS2offset(disk_car,&start);
   new_partition->part_size=(uint64_t)CHS2offset(disk_car,&end)-new_partition->part_offset+disk_car->sector_size;
   return insert_new_partition(list_part, new_partition);
 }
 
-static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert)
+static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, const int debug,const int dump_ind, const unsigned int expert, char**current_cmd)
 {
   const t_list_part *element;
   if(debug>1)
@@ -869,10 +893,10 @@ static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, cons
       switch(element->part->upart_type)
       {
 	case UP_FAT32:
-	  fat32_boot_sector(disk_car, element->part, debug, dump_ind, expert);
+	  fat32_boot_sector(disk_car, element->part, debug, dump_ind, expert,current_cmd);
 	  break;
 	case UP_NTFS:
-	  ntfs_boot_sector(disk_car, element->part, debug, dump_ind);
+	  ntfs_boot_sector(disk_car, element->part, debug, dump_ind,current_cmd);
 	  break;
 	default:
 	  ecrit_rapport("Need to fix\n");
@@ -884,9 +908,10 @@ static int use_backup(t_param_disk *disk_car, const t_list_part *list_part, cons
   return 0;
 }
 
-int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align, const int ask_part_order, const unsigned int expert)
+int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const int dump_ind, int fast_mode, const int align, const int ask_part_order, const unsigned int expert, char **current_cmd)
 {
   int res_interface_write;
+  int menu=-1;
   do
   {
     t_list_part *list_part;
@@ -897,7 +922,7 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
     wdoprintf(stdscr,"%s",disk_car->description(disk_car));
     wmove(stdscr,5,0);
     res_interface_write=0;
-    if(paranoid==1 && fast_mode==2)
+    if(paranoid==1 && (fast_mode==2 || (fast_mode==1 &&expert==0)))
       can_search_deeper=0;
     list_part=search_part(disk_car,paranoid,debug,dump_ind,fast_mode,1);
     if(list_part!=NULL)
@@ -906,13 +931,11 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       head_max=get_geometry_from_list_part(disk_car, list_part, debug);
       if(disk_car->CHS.head!=head_max)
       {
-	warning_geometry(disk_car,head_max+1);
+	warning_geometry(disk_car,head_max+1,current_cmd);
       }
     }
     align_structure(disk_car,list_part,align);
-#ifdef DEBUG
-    check_list_part(list_part);
-#endif
+    list_part=search_part_raid(disk_car,list_part,list_part,debug,dump_ind,1);
     align_structure(disk_car,list_part,align);
 #ifdef DEBUG
     check_list_part(list_part);
@@ -929,7 +952,7 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
 	aff_part_rapport(disk_car,element->part);
     }
     wdoprintf(stdscr,msg_GOD_MODE);
-    list_part=ask_structure(disk_car,list_part,debug);
+    list_part=ask_structure(disk_car,list_part,debug,current_cmd);
 #ifdef DEBUG
     check_list_part(list_part);
 #endif
@@ -951,11 +974,12 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       check_list_part(list_part);
 #endif
       /* Creer la partition etendue globale, cherche  aligner la partition */
+      /* if(disk_car->arch==&arch_i386) */
       {
 	t_list_part *parts;
 	uint64_t partext_offset=0;
 	uint64_t partext_size=0;
-	list_part=add_ext_part(disk_car,list_part,!max_ext,2,debug);
+	list_part=add_ext_part_i386(disk_car,list_part,!max_ext,2,debug);
 	for(parts=list_part;parts!=NULL;parts=parts->next)
 	  if(parts->part->status==STATUS_EXT)
 	  {
@@ -967,7 +991,7 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
 #endif
 	if(partext_offset>0)
 	{
-	  list_part=add_ext_part(disk_car,list_part,max_ext,2,debug);
+	  list_part=add_ext_part_i386(disk_car,list_part,max_ext,2,debug);
 	  for(parts=list_part;parts!=NULL;parts=parts->next)
 	    if(parts->part->status==STATUS_EXT)
 	    {
@@ -989,22 +1013,35 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
       do
       {
 	do_again=0;
-	res_interface_write=interface_write(disk_car,list_part,can_search_deeper,can_ask_minmax_ext);
+	res_interface_write=interface_write(disk_car,list_part,can_search_deeper,can_ask_minmax_ext,current_cmd,&menu);
 	switch(res_interface_write)
 	{
 	  case 'W':
-	    if(ask_confirmation("Write partition table, confirm ? (Y/N)"))
+	    if(disk_car->arch->write_part!=NULL)
 	    {
-	      ecrit_rapport("write!\n");
-	      disk_car->arch->write_part(disk_car,list_part,RW,debug,align);
-	      use_backup(disk_car,list_part,debug,dump_ind,expert);
+	      if(ask_confirmation("Write partition table, confirm ? (Y/N)"))
+	      {
+		ecrit_rapport("write!\n");
+		if(disk_car->arch->write_part(disk_car,list_part,RW,debug,align))
+		{
+		  display_message(msg_PART_WR_ERR);
+		}
+		else
+		{
+		  use_backup(disk_car,list_part,debug,dump_ind,expert,current_cmd);
+		  display_message("You will have to reboot for the change to take effect.\n");
+		}
+	      }
+	      else
+		ecrit_rapport("Don't write, no confirmation\n");
 	    }
-	    else
-	      ecrit_rapport("Don't write, no confirmation\n");
 	    break;
 	  case 0:
-	    ecrit_rapport("simulate write!\n");
-	    disk_car->arch->write_part(disk_car,list_part,RO,debug,align);
+	    if(disk_car->arch->write_part!=NULL)
+	    {
+	      ecrit_rapport("simulate write!\n");
+	      disk_car->arch->write_part(disk_car,list_part,RO,debug,align);
+	    }
 	    break;
 	  case 'S':
 	    paranoid=1;
@@ -1013,7 +1050,7 @@ int interface_recovery(t_param_disk *disk_car,int paranoid,const int debug,const
 	    break;
 	  case 'E':
 	    max_ext=!max_ext;
-	    list_part=add_ext_part(disk_car,list_part,max_ext,2,debug);
+	    list_part=add_ext_part_i386(disk_car,list_part,max_ext,2,debug);
 	    do_again=1;
 	    break;
 	}
