@@ -65,6 +65,9 @@
 #ifdef HAVE_SYS_DKIO_H
 #include <sys/dkio.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #ifdef HAVE_LINUX_FS_H
 #include <linux/fs.h>
 #endif
@@ -78,11 +81,14 @@
 #ifdef HAVE_WINIOCTL_H
 #include <winioctl.h>
 #endif
+#ifdef HAVE_FNCTL_H
+#include <fnctl.h>
+#endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>     /* atexit, posix_memalign */
+#endif
 
 #ifdef DJGPP
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>     /* atexit */
-#endif
 #include <go32.h>       /* dosmemget/put */
 #include <dpmi.h>
 #include <bios.h>       /* bios_k* */
@@ -90,18 +96,21 @@
 #include <io.h>
 #include <windows.h>
 #include <winnt.h>
-#include <winioctl.h>
 #endif
 #include "fnctdsk.h"
 #ifndef O_BINARY
 #define O_BINARY 0
 #endif
+extern const t_arch_fnct arch_mac;
 
-struct info_file
+struct info_file_struct
 {
   int handle;
   uint64_t offset;
-  char name[DISKNAME_MAX];
+  char file_name[DISKNAME_MAX];
+  int mode;
+  void *buffer;
+  unsigned int buffer_size;
 };
 
 static void autoset_geometry(t_param_disk * disk_car, const unsigned char *buffer, const int debug, const t_arch_fnct *arch);
@@ -110,10 +119,13 @@ static int file_clean(t_param_disk *disk_car);
 static int file_read(t_param_disk *disk_car, const unsigned int count, void *nom_buffer, const uint64_t offset);
 static int file_write(t_param_disk *disk_car, const unsigned int count, const void *nom_buffer, const uint64_t offset);
 static int file_nowrite(t_param_disk *disk_car, const unsigned int count, const void *nom_buffer, const uint64_t offset);
+#ifndef DJGPP
 static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const int debug);
+#endif
 #ifdef DEBUG
 static void bug(void);
 #endif
+static void compute_device_size(t_param_disk *disk_car);
 
 #ifdef DJGPP
 #define HD_RW_BUF_SIZ 0x10
@@ -131,11 +143,12 @@ static int hd_report_error(t_param_disk *disk_car, const uint64_t hd_offset, con
 static const char *disk_description(t_param_disk *disk_car);
 static int disk_read(t_param_disk *disk_car, const unsigned int count, void *nom_buffer, const uint64_t hd_offset);
 static int disk_write(t_param_disk *disk_car, const unsigned int count, const void *nom_buffer, const uint64_t hd_offset);
-struct info_disk
+static int disk_clean(t_param_disk *disk_car);
+struct info_disk_struct
 {
   unsigned int disk;
   t_CHS CHSR;	/* CHS low level */
-  int mode;
+  int mode_enh;
   unsigned int nbr_err;
   int bad_geometry;
 };
@@ -158,7 +171,9 @@ static int alloc_dos_buffer(void)
     dos_segment = 0;
     return 1;
   }
+#ifdef HAVE_ATEXIT
   atexit(free_dos_buffer);
+#endif
   return 0;
 }
 
@@ -167,9 +182,9 @@ static int hd_super(int cmd, t_param_disk *disk_car, unsigned long int hd_offset
   __dpmi_regs r;
   unsigned char buf[HD_RW_BUF_SIZ];
   int dos_segment2, dos_selector2, xfer2=nsects*DEFAULT_SECTOR_SIZE;
-  struct info_disk*data=disk_car->data;
+  struct info_disk_struct*data=disk_car->data;
 
-  if((data->mode==0)||(cmd==0))
+  if((data->mode_enh==0)||(cmd==0))
   { /* Limite CHS = 1023,255,63 = 8,064Mo ~= 7.8 Go */
     int head, track, sector;
     if(data->CHSR.sector==0)
@@ -268,7 +283,7 @@ static int hd_identify_enh_bios(t_param_disk *disk_car,const int debug)
   int compute_LBA=0;
   __dpmi_regs r;
   unsigned char buf[0x200];	/* Don't change it! */
-  struct info_disk*data=disk_car->data;
+  struct info_disk_struct*data=disk_car->data;
   buf[0]=HDPARM_BUF_SIZ;
   buf[1]=0;
   if(dos_segment==0)
@@ -371,13 +386,16 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
   if(disk>(unsigned int)(0x80+buf[2]-1))
     return NULL;
   {
-    struct info_disk*data=(struct info_disk*)MALLOC(sizeof(*data));
+    char device[100];
+    struct info_disk_struct*data=(struct info_disk_struct*)MALLOC(sizeof(*data));
     t_param_disk *disk_car=(t_param_disk *)MALLOC(sizeof(*disk_car));
     data->disk=disk;
     data->bad_geometry=0;
-    data->mode=0;
+    data->mode_enh=0;
     data->nbr_err=0;
     disk_car->arch=arch;
+    snprintf(device,sizeof(device),"/dev/sda%u",disk);
+    disk_car->device=strdup(device);
     disk_car->halt_on_errors=0;
     disk_car->write_used=0;
     disk_car->autodetect=0;
@@ -385,7 +403,7 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
     disk_car->description=disk_description;
     disk_car->read=disk_read;
     disk_car->write=disk_write;
-    disk_car->clean=file_clean;
+    disk_car->clean=disk_clean;
     disk_car->data=data;
     disk_car->CHS.cylinder=((buf[0] & 0x0C0)<<2)|buf[1];
     disk_car->CHS.head=buf[3];
@@ -416,7 +434,7 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
       param_disk_enh->write_used=0;
       param_disk_enh->sector_size=DEFAULT_SECTOR_SIZE;
       param_disk_enh->data=data;
-      data->mode=1;
+      data->mode_enh=1;
       if(!hd_identify_enh_bios(param_disk_enh,debug))
       {
 	/* standard geometry H,S, compute C from LBA */
@@ -424,7 +442,7 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
 	disk_car->CHS.cylinder=(disk_car->disk_size/(disk_car->CHS.head+1))/disk_car->CHS.sector/DEFAULT_SECTOR_SIZE-1;
       }
       else
-	data->mode=0;
+	data->mode_enh=0;
       FREE(param_disk_enh);
     }
     return disk_car;
@@ -433,9 +451,11 @@ static t_param_disk *hd_identify(const int debug, const unsigned int disk, const
 
 static const char *disk_description(t_param_disk *disk_car)
 {
-  struct info_disk*data=disk_car->data;
-  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %2x - CHS %u %u %u - %u MB %s",
-      data->disk, disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector,(unsigned int)(disk_car->disk_size/1024/1024),data->bad_geometry!=0?" (Buggy BIOS)":"");
+  struct info_disk_struct*data=disk_car->data;
+  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %2x - %u MB - CHS %u %u %u %s",
+      data->disk, (unsigned int)(disk_car->disk_size/1024/1024),
+      disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector,
+      data->bad_geometry!=0?" (Buggy BIOS)":"");
   return disk_car->description_txt;
 }
 
@@ -459,7 +479,7 @@ static int disk_read(t_param_disk *disk_car,const unsigned int count, void *nom_
   }
   else
   {
-    struct info_disk*data=disk_car->data;
+    struct info_disk_struct*data=disk_car->data;
     if(hd_offset==0)
     {
       /* CHS=(0,0,1) */
@@ -513,7 +533,7 @@ static int disk_write(t_param_disk *disk_car,const unsigned int count, const voi
   }
   else
   {
-    struct info_disk*data=disk_car->data;
+    struct info_disk_struct*data=disk_car->data;
     disk_car->write_used=1;
     int i,rc=4; /* sector not found/read error */
     for(i=0;(rc==4)&&(i<MAX_IO_NBR);i++)
@@ -529,9 +549,20 @@ static int disk_write(t_param_disk *disk_car,const unsigned int count, const voi
   }
 }
 
+static int disk_clean(t_param_disk *disk_car)
+{
+  if(disk_car->data!=NULL)
+  {
+    struct info_disk_struct *data=disk_car->data;
+    FREE(disk_car->data);
+    disk_car->data=NULL;
+  }
+  return 0;
+}
+
 static int hd_report_error(t_param_disk *disk_car, const uint64_t hd_offset, const unsigned int nbr_sector, const int rc)
 {
-  struct info_disk*data=disk_car->data;
+  struct info_disk_struct*data=disk_car->data;
   data->nbr_err++;
   ecrit_rapport(" lba=%lu(%u/%u/%u) nbr_sector=%u, rc=%d\n",(long unsigned int)(hd_offset/disk_car->sector_size),
       offset2cylinder(disk_car,hd_offset),offset2head(disk_car,hd_offset),offset2sector(disk_car,hd_offset),
@@ -579,7 +610,7 @@ static int hd_report_error(t_param_disk *disk_car, const uint64_t hd_offset, con
 }
 #endif
 
-t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct *arch)
+t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct *arch, const int use_direct_io)
 {
   int i;
 #ifdef DJGPP
@@ -599,7 +630,23 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
     for(i=0;i<8;i++)
     {
       device_scsi[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch,use_direct_io));
+    }
+  }
+#elif defined(__APPLE__)
+  {
+    char device_scsi[]="/dev/disk0";
+    char device_raw[]="/dev/rdisk0";
+    /* Disk */
+    for(i=0;i<10;i++)
+    {
+      device_scsi[strlen(device_scsi)-1]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch,use_direct_io));
+    }
+    for(i=0;i<10;i++)
+    {
+      device_raw[strlen(device_raw)-1]='0'+i;
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_raw,debug,arch,use_direct_io));
     }
   }
 #elif defined(__MINGW32__)
@@ -608,9 +655,9 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
     for(i=0;i<8;i++)
     {
       device_hd[strlen(device_hd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_hd,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_hd,debug,arch,use_direct_io));
     }
-#elif defined(LINUX)
+#elif defined(TARGET_LINUX)
   {
     int j;
     char device[100];
@@ -624,13 +671,13 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
     for(i=0;i<8;i++)
     {
       device_ide[strlen(device_ide)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,debug,arch,use_direct_io));
     }
     /* Disk SCSI */
     for(i=0;i<26;i++)
     {
       device_scsi[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch,use_direct_io));
     }
     /* Device RAID Compaq */
     for(j=0;j<8;j++)
@@ -639,40 +686,40 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
       for(i=0;i<8;i++)
       {
 	device_ida[strlen(device_ida)-1]='0'+i;
-	list_disk=insert_new_disk(list_disk,file_test_availability(device_ida,debug,arch));
+	list_disk=insert_new_disk(list_disk,file_test_availability(device_ida,debug,arch,use_direct_io));
       }
     }
     for(i=0;i<8;i++)
     {
       device_cciss[strlen(device_cciss)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_cciss,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_cciss,debug,arch,use_direct_io));
     }
     /* Device RAID */
     for(i=0;i<10;i++)
     {
       snprintf(device,sizeof(device),"/dev/rd/c0d%u",i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device,debug,arch,use_direct_io));
     }
     /* Device RAID IDE */
     for(i=0;i<15;i++)
     {
       snprintf(device,sizeof(device),"/dev/ataraid/d%u",i);
-      list_disk=insert_new_disk(list_disk,file_test_availability(device,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device,debug,arch,use_direct_io));
     }
     /* Parallel port IDE disk */
     for(i=0;i<4;i++)
     {
       device_p_ide[strlen(device_p_ide)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_p_ide,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_p_ide,debug,arch,use_direct_io));
     }
     /* I2O hard disk */
     for(i=0;i<26;i++)
     {
       device_i2o_hd[strlen(device_i2o_hd)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_i2o_hd,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_i2o_hd,debug,arch,use_direct_io));
     }
   }
-#elif defined(SOLARIS)
+#elif defined(TARGET_SOLARIS)
   {
     char rdsk[]="/dev/rdsk/c0t0d0s2";
     for(i=0;i<15;i++)
@@ -680,7 +727,7 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
       if(i!=7)
       {
 	rdsk[13]='0'+i;
-	list_disk=insert_new_disk(list_disk,file_test_availability(rdsk,debug,arch));
+	list_disk=insert_new_disk(list_disk,file_test_availability(rdsk,debug,arch,use_direct_io));
       }
     }
   }
@@ -701,57 +748,56 @@ t_list_disk *hd_parse(t_list_disk *list_disk,const int debug, const t_arch_fnct
     for(i=0;i<8;i++)
     {
       device_ide[strlen(device_ide)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_ide2[strlen(device_ide2)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide2,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide2,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_ide3[strlen(device_ide3)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide3,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide3,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_ide4[strlen(device_ide4)-2]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide4,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide4,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_ide_hd[strlen(device_ide_hd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide_hd,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_ide_hd,debug,arch,use_direct_io));
     }
     /* Disk SCSI */
     for(i=0;i<8;i++)
     {
       device_scsi[strlen(device_scsi)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_scsi2[strlen(device_scsi2)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi2,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi2,debug,arch,use_direct_io));
     }
     for(i=0;i<8;i++)
     {
       device_scsi_hd[strlen(device_scsi_hd)-1]='0'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi_hd,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_scsi_hd,debug,arch,use_direct_io));
     }
     /* optical disks */
     for(i=0;i<8;i++)
     {
       device_optdisk[strlen(device_scsi)-1]='a'+i;
-      list_disk=insert_new_disk(list_disk,file_test_availability(device_optdisk,debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(device_optdisk,debug,arch,use_direct_io));
     } 
   }
 #endif
-  /* hdimage */
-  list_disk=insert_new_disk(list_disk,file_test_availability("hdimage",debug,arch));
   return list_disk;
 }
 
+#ifndef DJGPP
 static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const int debug)
 {
   t_param_disk *disk_car=NULL;
@@ -854,12 +900,12 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
       {
 	ecrit_rapport("disk_get_geometry BLKGETSIZE64 %s size %llu\n", device, (long long unsigned)longsectors64);
       }
+      if(sector_size<=0)
+      {
+	sector_size=DEFAULT_SECTOR_SIZE;
+      }
       if(disk_car->CHS.cylinder!= longsectors64 / (disk_car->CHS.head+1) / disk_car->CHS.sector/sector_size-1)
       {
-	if(sector_size<=0)
-	{
-	  sector_size=DEFAULT_SECTOR_SIZE;
-	}
 	ecrit_rapport("disk_get_geometry BLKGETSIZE64 %s number of cylinders %u !=  %lu (calculated)\n", device,disk_car->CHS.cylinder+1,(long unsigned)(longsectors64 / (disk_car->CHS.head+1) / (disk_car->CHS.sector)/sector_size));
 	disk_car->CHS.cylinder= longsectors64 / (disk_car->CHS.head+1) / disk_car->CHS.sector/sector_size-1;
       }
@@ -1039,12 +1085,14 @@ static t_param_disk *disk_get_geometry(const int hd_h, const char *device, const
   }
   return disk_car;
 }
+#endif
 
 static const char *file_description(t_param_disk *disk_car)
 {
-  struct info_file *data=disk_car->data;
-  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - CHS %u %u %u - %u MB",
-      data->name, disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector,(unsigned)(disk_car->disk_size/1024/1024));
+  struct info_file_struct *data=disk_car->data;
+  snprintf(disk_car->description_txt, sizeof(disk_car->description_txt),"Disk %s - %u MB - CHS %u %u %u",
+      data->file_name, (unsigned)(disk_car->disk_size/1024/1024),
+      disk_car->CHS.cylinder+1, disk_car->CHS.head+1, disk_car->CHS.sector);
   return disk_car->description_txt;
 }
 
@@ -1052,7 +1100,7 @@ static int file_clean(t_param_disk *disk_car)
 {
   if(disk_car->data!=NULL)
   {
-    struct info_file *data=disk_car->data;
+    struct info_file_struct *data=disk_car->data;
     /*
 #ifdef BLKRRPART
     if (ioctl(data->handle, BLKRRPART, NULL)) {
@@ -1063,6 +1111,11 @@ static int file_clean(t_param_disk *disk_car)
 #endif
     */
     close(data->handle);
+    if(data->buffer!=NULL)
+    {
+      FREE(data->buffer);
+      data->buffer=NULL;
+    }
     FREE(disk_car->data);
     disk_car->data=NULL;
   }
@@ -1071,10 +1124,35 @@ static int file_clean(t_param_disk *disk_car)
 
 static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_buffer, const uint64_t offset)
 {
-  if(count%disk_car->sector_size!=0 || offset%disk_car->sector_size!=0)
+  struct info_file_struct *data=disk_car->data;
+  if(count%disk_car->sector_size!=0 || offset%disk_car->sector_size!=0
+#ifdef O_DIRECT
+      || ((data->mode&O_DIRECT)!=0 && nom_buffer!=data->buffer)
+#endif
+    )
   {
-    char*buffer=(char*)MALLOC((offset%disk_car->sector_size+count+disk_car->sector_size-1)
-	/ disk_car->sector_size * disk_car->sector_size);
+    if(data->buffer==NULL)
+    {
+      data->buffer_size=128*512;
+    }
+    while(data->buffer_size<((offset%disk_car->sector_size+count+disk_car->sector_size-1)
+	  / disk_car->sector_size * disk_car->sector_size))
+    {
+      if(data->buffer!=NULL)
+      {
+	FREE(data->buffer);
+	data->buffer=NULL;
+      }
+      data->buffer_size*=2;
+    }
+    if(data->buffer==NULL)
+    {
+#ifdef HAVE_POSIX_MEMALIGN
+      posix_memalign(&data->buffer,disk_car->sector_size,data->buffer_size);
+#else
+      data->buffer=(char*)MALLOC(data->buffer_size);
+#endif
+    }
     /*
     ecrit_rapport("file_read handle unaligned data\n");
      */
@@ -1082,24 +1160,24 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
     bug();
 #endif
     if(file_read(disk_car,(offset%disk_car->sector_size+count+disk_car->sector_size-1)
-      / disk_car->sector_size * disk_car->sector_size, buffer,
+      / disk_car->sector_size * disk_car->sector_size, data->buffer,
       offset/disk_car->sector_size*disk_car->sector_size))
     {
-      FREE(buffer);
       return -1;
     }
-    memcpy(nom_buffer,buffer+(offset%disk_car->sector_size),count);
-    FREE(buffer);
+    memcpy(nom_buffer,(char*)data->buffer+(offset%disk_car->sector_size),count);
+    return 0;
   }
-  else
   {
-    struct info_file *data=disk_car->data;
     long int ret;
 #ifdef DEBUG
       ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u))\n", data->handle,
 	  (unsigned)(count/disk_car->sector_size), (long unsigned int)(offset/disk_car->sector_size),
 	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
 #endif
+#ifdef HAVE_PREAD
+      ret=pread(data->handle,nom_buffer,count,offset+data->offset);
+#else
 #ifdef __MINGW32__
     if(_lseeki64(data->handle,offset+data->offset,SEEK_SET)==-1)
     {
@@ -1131,27 +1209,25 @@ static int file_read(t_param_disk *disk_car,const unsigned int count, void *nom_
     }
 #else
     ret=read(data->handle, nom_buffer, count);
+#endif
 #endif
     if(ret!=count)
     {
-      ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", data->handle,
-	  (unsigned)(count/disk_car->sector_size), (long unsigned)(offset/disk_car->sector_size),
-	  offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
-      if(ret<0)
+      if(ret>0 || offset<disk_car->disk_size)
       {
-	ecrit_rapport("%s\n", strerror(errno));
+	ecrit_rapport("file_read(%d,%u,buffer,%lu(%u/%u/%u)) read err: ", data->handle,
+	    (unsigned)(count/disk_car->sector_size), (long unsigned)(offset/disk_car->sector_size),
+	    offset2cylinder(disk_car,offset), offset2head(disk_car,offset), offset2sector(disk_car,offset));
+	if(ret<0)
+	  ecrit_rapport("%s\n", strerror(errno));
+	else if(ret==0)
+	  ecrit_rapport("read after end of file\n");
+	else
+	  ecrit_rapport("Partial read\n");
+      }
+      if(ret<=0)
 	return -1;
-      }
-      else if(ret==0)
-      {
-	ecrit_rapport("read after end of file\n");
-	return -1;
-      }
-      else
-      {
-	ecrit_rapport("Partial read\n");
-	memset((char*)nom_buffer+ret,0,count-ret);
-      }
+      memset((char*)nom_buffer+ret,0,count-ret);
     }
   }
   return 0;
@@ -1179,8 +1255,11 @@ static int file_write(t_param_disk *disk_car,const unsigned int count, const voi
   else
   {
     long int ret;
-    struct info_file *data=disk_car->data;
+    struct info_file_struct *data=disk_car->data;
 /*    ecrit_rapport("file_write offset=%lu data->offset=%lu\n",(long unsigned)offset,(long unsigned)data->offset); */
+#ifdef HAVE_PWRITE
+      ret=pwrite(data->handle,nom_buffer,count,offset+data->offset);
+#else
 #ifdef __MINGW32__
     if(_lseeki64(data->handle,offset+data->offset,SEEK_SET)==-1)
     {
@@ -1196,8 +1275,9 @@ static int file_write(t_param_disk *disk_car,const unsigned int count, const voi
       return -1;
     }
 #endif
-    disk_car->write_used=1;
     ret=write(data->handle, nom_buffer, count);
+#endif
+    disk_car->write_used=1;
     if(ret!=count)
     {
       ecrit_rapport("file_write(%d,%u,buffer,%lu(%u/%u/%u)) write err %s\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
@@ -1210,7 +1290,7 @@ static int file_write(t_param_disk *disk_car,const unsigned int count, const voi
 
 static int file_nowrite(t_param_disk *disk_car,const unsigned int count, const void *nom_buffer, const uint64_t offset)
 {
-  struct info_file *data=disk_car->data;
+  struct info_file_struct *data=disk_car->data;
   ecrit_rapport("write_file(%d,%u,buffer,%lu(%u/%u/%u)) write refused\n", data->handle,(unsigned)(count/disk_car->sector_size),(long unsigned)(offset/disk_car->sector_size),
       offset2cylinder(disk_car,offset),offset2head(disk_car,offset),offset2sector(disk_car,offset));
   return -1;
@@ -1232,42 +1312,95 @@ static void autoset_geometry(t_param_disk * disk_car, const unsigned char *buffe
     disk_car->CHS.sector=63;
   }
   /* Round up because file is often truncated. */
-  disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size+disk_car->CHS.sector*(disk_car->CHS.head+1)-1)/disk_car->CHS.sector/(disk_car->CHS.head+1)-1;
+  disk_car->CHS.cylinder=(disk_car->disk_size/disk_car->sector_size+(uint64_t)disk_car->CHS.sector*(disk_car->CHS.head+1)-1)/disk_car->CHS.sector/(disk_car->CHS.head+1)-1;
 }
 
-t_param_disk *file_test_availability(const char *device, const int debug, const t_arch_fnct *arch)
+static void compute_device_size(t_param_disk *disk_car)
+{
+  /* This function can failed if there are bad sectors */
+  uint64_t min_offset, max_offset;
+  char *buffer=MALLOC(disk_car->sector_size);
+  int read_error=1;
+  min_offset=0;
+  max_offset=disk_car->sector_size;
+  /* Search the maximum device size */
+  while(disk_car->read(disk_car,1,buffer,max_offset)==0)
+  {
+    min_offset=max_offset;
+    max_offset*=2;
+  }
+  /* Search the device size by dichotomy */
+  while(min_offset<=max_offset)
+  {
+    uint64_t cur_offset;
+    cur_offset=(min_offset+max_offset)/2/disk_car->sector_size*disk_car->sector_size;
+    if(disk_car->read(disk_car,1,buffer,cur_offset)==0)
+      min_offset=cur_offset+disk_car->sector_size;
+    else
+    {
+      if(cur_offset>=disk_car->sector_size)
+	max_offset=cur_offset-disk_car->sector_size;
+      else
+	break;
+    }
+  }
+  if(disk_car->read(disk_car,1,buffer,min_offset)==0)
+    min_offset+=disk_car->sector_size;
+  disk_car->disk_size=min_offset;
+  FREE(buffer);
+}
+
+t_param_disk *file_test_availability(const char *device, const int debug, const t_arch_fnct *arch, const int use_direct_io)
 {
   unsigned int offset=0;
   t_param_disk *disk_car=NULL;
   int hd_h;
   int no_write=0;
+  int mode;
+  mode=O_RDWR|O_BINARY;
 #ifdef O_LARGEFILE
-  hd_h = open(device, O_RDWR|O_LARGEFILE|O_BINARY);
-#else
-  hd_h = open(device, O_RDWR|O_BINARY);
+  mode|=O_LARGEFILE;
 #endif
+#ifdef O_DIRECT
+  if(use_direct_io)
+    mode|=O_DIRECT;
+#endif
+  hd_h = open(device, mode);
   if(hd_h<0)
   {
     if(debug>1)
     {
       ecrit_rapport("file_test_availability %s:%s\n", device,strerror(errno));
     }
-/*   if(errno==EROFS||errno==EACCES|errno==EPERM) */
-    if(errno!=ENXIO && errno!=ENOENT)
+    switch(errno)
     {
-      no_write=1;
-#ifdef O_LARGEFILE
-      hd_h = open(device, O_RDONLY|O_LARGEFILE|O_BINARY);
-#else
-      hd_h = open(device, O_RDONLY|O_BINARY);
+      case ENXIO:
+      case ENOENT:
+#ifdef ENOMEDIUM
+      case ENOMEDIUM:
 #endif
-      if(hd_h<0)
-      {
-	if(debug>1)
+	break;
+      default:
 	{
-	  ecrit_rapport("file_test_availability %s:%s\n", device,strerror(errno));
+	  no_write=1;
+	  mode=O_RDONLY|O_BINARY;
+#ifdef O_LARGEFILE
+	  mode|=O_LARGEFILE;
+#endif
+#ifdef O_DIRECT
+	  if(use_direct_io)
+	    mode|=O_DIRECT;
+#endif
+	  hd_h = open(device, mode);
+	  if(hd_h<0)
+	  {
+	    if(debug>1)
+	    {
+	      ecrit_rapport("file_test_availability %s:%s\n", device,strerror(errno));
+	    }
+	  }
 	}
-      }
+	break;
     }
   }
   if(hd_h>=0)
@@ -1309,6 +1442,7 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
     if(disk_car==NULL || disk_car->disk_size==0)
     {
       unsigned char buffer[DEFAULT_SECTOR_SIZE];
+      /* FIXME use_direct_io */
       if(read(hd_h,buffer,DEFAULT_SECTOR_SIZE)<0)
       {
 	close(hd_h);
@@ -1363,19 +1497,15 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
     }
     if(disk_car!=NULL)
     {
-      struct info_file *data;
-      if(disk_car->disk_size==0)
-      {
-	ecrit_rapport("Warning: can't get size for %s\n",device);
-	FREE(disk_car);
-	close(hd_h);
-	return NULL;
-      }
+      struct info_file_struct *data;
       data=MALLOC(sizeof(*data));
       data->offset=offset;
-      strncpy(data->name,device,sizeof(data->name));
-      data->name[sizeof(data->name)-1]='\0';
+      strncpy(data->file_name,device,sizeof(data->file_name));
+      data->file_name[sizeof(data->file_name)-1]='\0';
       data->handle=hd_h;
+      data->mode=mode;
+      data->buffer=NULL;
+      disk_car->device=strdup(device);
       disk_car->halt_on_errors=0;
       disk_car->write_used=0;
       disk_car->description_txt[0]='\0';
@@ -1385,6 +1515,23 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
       disk_car->clean=file_clean;
       disk_car->data=data;
       disk_car->arch=arch;
+      if(disk_car->disk_size==0)
+      {
+	/* Handle Mac */
+	compute_device_size(disk_car);
+	if(debug>1)
+	{
+	  ecrit_rapport("file_test_availability compute_device_size %s size %llu\n", device, (long long unsigned)disk_car->disk_size);
+	}
+      }
+      if(disk_car->disk_size==0)
+      {
+	ecrit_rapport("Warning: can't get size for %s\n",device);
+	FREE(data);
+	FREE(disk_car);
+	close(hd_h);
+	return NULL;
+      }
       return disk_car;
     }
     close(hd_h);
@@ -1392,19 +1539,27 @@ t_param_disk *file_test_availability(const char *device, const int debug, const
   return NULL;
 }
 
-void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cylinder, const t_arch_fnct *arch)
+void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cylinder, const t_arch_fnct *arch, const int debug)
 {
   const t_list_disk *element_disk;
+  if(debug>1)
+  {
+    ecrit_rapport("hd_parse_bis\n");
+  }
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
   {
     t_param_disk *disk_car=element_disk->disk;
     unsigned char buffer[disk_car->sector_size];
     uint64_t pos;
     t_CHS pos_CHS;
-    if(disk_car->autodetect!=0)
+    if(disk_car->autodetect!=0 || disk_car->arch==&arch_mac)
     {
       if(disk_car->read(element_disk->disk,disk_car->sector_size, &buffer, 0)==0)
       {
+	if(debug>1)
+	{
+	  ecrit_rapport("autoset_geometry\n");
+	}
 	autoset_geometry(disk_car,buffer,1,arch);
       }
     }
@@ -1418,14 +1573,14 @@ void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cy
 #ifdef DJGPP
     if(disk_car->description==disk_description)
     {
-      struct info_disk*data=disk_car->data;
+      struct info_disk_struct*data=disk_car->data;
       data->CHSR.cylinder=0;
     }
 #endif
     if(disk_car->read(element_disk->disk,disk_car->sector_size, &buffer, pos)==0)
     {
       disk_car->CHS.cylinder++;
-      if(disk_car->disk_size < (disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size)
+      if(disk_car->disk_size < (uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size)
       {
 	disk_car->disk_size=(uint64_t)(disk_car->CHS.cylinder+1)*(disk_car->CHS.head+1)*disk_car->CHS.sector*disk_car->sector_size;
 	ecrit_rapport("Computes LBA from CHS for %s\n",disk_car->description(disk_car));
@@ -1434,11 +1589,10 @@ void hd_parse_bis(const t_list_disk * list_disk, const int allow_partial_last_cy
 #ifdef DJGPP
     if(disk_car->description==disk_description)
     {
-      struct info_disk*data=disk_car->data;
+      struct info_disk_struct*data=disk_car->data;
       data->CHSR.cylinder=disk_car->CHS.cylinder;
     }
 #endif
   }
 }
 
-
