@@ -60,8 +60,9 @@ int recover_HFSP(t_param_disk *disk_car, const struct hfsp_vh *vh,t_partition *p
   }
   partition->part_size=part_size;
   set_HFSP_info(partition,vh);
-  partition->part_type=(unsigned char)P_HFSP;
-  if(debug!=0)
+  partition->part_type_i386=P_HFSP;
+  partition->part_type_mac=PMAC_HFS;
+  if(debug>0)
   {
     ecrit_rapport("part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
   }
