@@ -47,10 +47,6 @@
 /* Miscellenous constants */
 #define MENU_SPACING 2
 #define MENU_MAX_ITEMS 256 /* for simpleMenu function */
-#define MENU_UP 1
-#define MENU_DOWN 2
-#define MENU_RIGHT 3
-#define MENU_LEFT 4
 #define key_CR '\015'
 #define key_ESC '\033'
 #define key_DEL '\177'
@@ -59,7 +55,7 @@
 #define key_REDRAWKEY '\014'
 
 
-enum buffer_cmd {BUFFER_RESET, BUFFER_ADD, BUFFER_DISPLAY, BUFFER_WRITE,BUFFER_SHOW};
+enum buffer_cmd {BUFFER_RESET, BUFFER_ADD, BUFFER_WRITE,BUFFER_SHOW};
 typedef enum buffer_cmd buffer_cmd_t;
 /* int aff_buffer(const int cmd, const char *_format, ...) __attribute__ ((format (printf, 2, 3))); */
 int aff_buffer(const buffer_cmd_t cmd, const char *_format, ...);
@@ -71,18 +67,20 @@ struct MenuItem
     const char *desc; /* Item description to be printed when item is selected */
 };
 int wdoprintf(WINDOW *window, const char *_format, ...) __attribute__ ((format (printf, 2, 3)));
-int wmenuSelect(WINDOW *window, int y, int x, struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int menuDefault);
+int wmenuSelect(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int menuDefault);
+int wmenuSelect_ext(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, int menuType, unsigned int *current, int *real_key);
 void aff_LBA2CHS(const t_param_disk *disk_car, const unsigned long int pos_LBA);
 void aff_LBA2CHS_rapport(const t_param_disk *disk_car, const unsigned long int pos_LBA);
 void aff_CHS_rapport(const t_CHS * CHS);
 void aff_CHS(const t_CHS * CHS);
 void aff_CHS_buffer(const t_CHS * CHS);
+const char *aff_part_aux(const aff_part_type_t newline, const t_param_disk *disk_car, const t_partition *partition);
 void aff_part_buffer(const aff_part_type_t newline,const t_param_disk *disk_car,const t_partition *partition);
 void aff_part(WINDOW *window,const aff_part_type_t newline,const t_param_disk *disk_car,const t_partition *partition);
 void aff_part_rapport(const t_param_disk *disk_car,const t_partition *partition);
 
 int ask_confirmation(const char*_format, ...) __attribute__ ((format (printf, 1, 2)));
-unsigned int ask_number(const unsigned int val_cur, const unsigned int val_min, const unsigned int val_max, const char * _format, ...) __attribute__ ((format (printf, 4, 5)));;
+unsigned long int ask_number(const unsigned long int val_cur, const unsigned long int val_min, const unsigned long int val_max, const char * _format, ...) __attribute__ ((format (printf, 4, 5)));;
 int display_message_aux(const char*_format,...) __attribute__ ((format (printf, 1, 2)));
 int display_message(const char*msg);
 int get_string(char *str, int len, char *def);
@@ -91,11 +89,16 @@ void dump(WINDOW *window,const void *nom_dump,unsigned int lng);
 void dump2_log(const void *dump_1, const void *dump_2,const unsigned int lng);
 void dump2(WINDOW *window,const void *dump_1, const void *dump_2,const unsigned int lng);
 int wgetch_nodelay(WINDOW *window);
-int menuUpdate( int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, const int menuType, unsigned int current );
 int wmenuUpdate(WINDOW *window, int y, int x, const struct MenuItem *menuItems, const unsigned int itemLength, const char *available, const int menuType, unsigned int current);
-int wmenuSimple(WINDOW *window,struct MenuItem *menuItems, int menuDefault);
+int wmenuSimple(WINDOW *window, const struct MenuItem *menuItems, int menuDefault);
 void aff_copy(WINDOW *window);
 int ask_YN(WINDOW *window);
 void not_implemented(const char *msg);
+int start_ncurses(const char *prog_name);
+int end_ncurses(void);
+int screen_buffer_display(WINDOW *window, const char *options_org, const struct MenuItem *menuItems);
+int screen_buffer_display_ext(WINDOW *window, const char *options_org, const struct MenuItem *menuItems, int *menu);
+void screen_buffer_to_log(void);
+int check_enter_or_s(WINDOW *window);
 
 
