@@ -37,6 +37,9 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#ifdef HAVE_SYS_STAT_H
+#include <sys/stat.h>
+#endif
 #include "types.h"
 #include "common.h"
 #include "lang.h"
@@ -45,22 +48,24 @@
 #include "godmode.h"
 #include "fnctdsk.h"
 #include "testdisk.h"
-#include "analyse.h"
-#include "fat.h"
-#include "ext2_dir.h"
-#include "rfs.h"
-#include "ntfs.h"
 #include "adv.h"
-#include "savehdr.h"
+#include "analyse.h"
 #include "chgtype.h"
-#include "add_part.h"
 #include "edit.h"
+#include "savehdr.h"
+#include "dir.h"
+#include "ext2_dir.h"
+#include "fat.h"
+#include "fat_dir.h"
+#include "ntfs_dir.h"
+#include "rfs_dir.h"
 
 extern const t_arch_fnct arch_i386;
 extern const t_arch_fnct arch_sun;
 extern const t_arch_fnct arch_none;
+extern const t_arch_fnct arch_mac;
 
-static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align,int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch);
+static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align, int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch, char**current_cmd);
 static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part, const int debug);
 static t_list_part *backup_merge(t_list_part *list_part,t_list_part *backup_part, const int debug);
 static int write_MBR_code(t_param_disk *disk_car);
@@ -71,6 +76,7 @@ void interface_list(t_param_disk *disk_car, const int debug,const int test_recov
   ecrit_rapport("\nAnalyse ");
   ecrit_rapport("%s\n",disk_car->description(disk_car));
   printf("%s\n",disk_car->description(disk_car));
+  printf(msg_PART_HEADER_LONG);
   list_part=disk_car->arch->read_part(disk_car,debug,saveheader);
   aff_buffer(BUFFER_WRITE,"Q");
   if(backup>0)
@@ -83,11 +89,12 @@ void interface_list(t_param_disk *disk_car, const int debug,const int test_recov
     ecrit_rapport("rebuild_FAT_BS()\n");
     for(element=list_part;element!=NULL;element=element->next)
     {
-      if(is_fat(element->part->part_type))		/* do not use upart_type */
+      if(is_fat(element->part))
       {
 	aff_part_rapport(disk_car,element->part);
-	rebuild_FAT_BS(disk_car,element->part,debug,0,0,0);	/* dump_ind */
+	rebuild_FAT_BS(disk_car,element->part,debug,0,0,0,NULL);	/* dump_ind */
       }
+      /* TODO ntfs */
     }
   }
   delete_list_part(list_part);
@@ -95,113 +102,88 @@ void interface_list(t_param_disk *disk_car, const int debug,const int test_recov
 
 static int write_MBR_code(t_param_disk *disk_car)
 {
-  wdoprintf(stdscr,msg_WRITE_MBR_CODE);
-  if(ask_YN(stdscr)!=0 && ask_confirmation("Write a new copy of i386 MBR code, confirm ? (Y/N)")!=0)
+  if(disk_car->arch->write_MBR_code==NULL)
   {
-    unsigned char buffer[DEFAULT_SECTOR_SIZE];
-    if(read_MBR(disk_car,&buffer)!=0)
+    display_message("Function to write a new MBR code not implemented for this partition type.\n");
+    return 1;
+  }
+  wdoprintf(stdscr,msg_WRITE_MBR_CODE);
+  if(ask_YN(stdscr)!=0 && ask_confirmation("Write a new copy of MBR code, confirm ? (Y/N)")!=0)
+  {
+    if(disk_car->arch->write_MBR_code(disk_car))
     {
-      memset(buffer,0,sizeof(buffer));
+      display_message("Write error: Can't write new MBR code.\n");
+      return 2;
     }
-    disk_car->arch->write_MBR_code(buffer);
-    return write_MBR(disk_car,&buffer);
+    else
+      display_message("You have to reboot for the change to take effect.\n");
   }
   return 0;
 }
 
 int write_clean_table(t_param_disk *disk_car)
 {
+  if(disk_car->arch->erase_list_part==NULL)
+  {
+    display_message("Clear partition table not implemented for this partition type.\n");
+    return 1;
+  }
   wdoprintf(stdscr,msg_WRITE_CLEAN_TABLE);
   if(ask_YN(stdscr)!=0 && ask_confirmation("Clear partition table, confirm ? (Y/N)")!=0)
   {
-    unsigned char buffer[DEFAULT_SECTOR_SIZE];
-    if(read_MBR(disk_car,&buffer)!=0)
+    if(disk_car->arch->erase_list_part(disk_car))
     {
-      memset(buffer,0,sizeof(buffer));
+      display_message("Write error: Can't clear partition table.\n");
+      return 2;
     }
-    disk_car->arch->erase_list_part(buffer);
-    return write_MBR(disk_car,&buffer);
+    else
+      display_message("You have to reboot for the change to take effect.\n");
   }
   return 0;
 }
 
-int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const t_arch_fnct **arch)
+int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, const int saveheader, const t_arch_fnct **arch, const char *cmd_device, char *cmd_run)
 {
-  int command;
-  int done=0;
+  int align=2;
   int allow_partial_last_cylinder=0;
   int ask_part_order=0;
-  int halt_on_errors=1;
-  unsigned int expert=0;
-  int align=2;
+  int command;
+  int done=0;
   int fast_mode=0;
-  int paranoid=0;
+  int halt_on_errors=1;
   int menu=0;
+  int paranoid=0;
+  int real_key;
+  unsigned int expert=0;
   const t_list_disk *element_disk;
-  const t_list_disk *current_disk=list_disk;
+  const t_list_disk *current_disk=NULL;
   static struct MenuItem menuMain[]=
   {
 	{'A',"Analyse","Analyse current partition structure and search for lost partitions"},
-	{'D',"Delete","Delete all data in the partition table"},
-	{'C',"MBR Code","Write the Classic MBR code to first sector"},
+	{'T',"Advanced","Advanced boot recovery"},
 	{'G',"Geometry", "Change disk geometry (expert only)" },
 	{'O',"Options","Modify options"},
-	{'T',"Advanced","Advanced boot recovery"},
+	{'D',"Delete","Delete all data in the partition table"},
+	{'C',"MBR Code","Write the Classic MBR code to first sector"},
 	{'Q',"Quit","Quit program"},
 	{'E',"Editor","Basic disk editor"},
 	{0,NULL,NULL}
   };
-  if(list_disk==NULL)
-    return 0;
-/* use_env(TRUE); */
-#ifdef HAVE_SETENV
-#ifdef DJGPP
-#elif defined(BSD)
-    setenv("TERM","cons25",0);
-#elif defined(LINUX)
-    setenv("TERM","linux",0);
-#elif defined(__CYGWIN__)
-    setenv("TERM","cygwin",0);
-#endif
-#endif
-#if defined(DJGPP) || defined(__MINGW32__)
-  if(initscr()==NULL)
+  char *current_cmd=cmd_run;
+  if(cmd_device!=NULL)
   {
-    printf("initscr() has failed. Exiting\n");
-  }
-#else
-  if(newterm(NULL,stdout,stdin)==NULL)
-  {
-#ifdef HAVE_SETENV
-    setenv("TERMINFO",".",1);
-    if(newterm(NULL,stdout,stdin)==NULL)
-#endif
+    for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
     {
-      ecrit_rapport("Terminfo file is missing.\n");
-      printf("Terminfo file is missing.\nCheck if you have not forgot to extract the subdirectories.\n");
-      return 1;
+      if(strcmp(element_disk->disk->device,cmd_device)==0)
+	current_disk=element_disk;
     }
   }
-#endif
-  if(LINES<25)
-  {
-    ecrit_rapport("Terminal has only %u lines\n",LINES);
-    endwin();
-    printf("TestDisk need 25 lines to work.\nPlease enlarge the terminal and restart TestDisk.\n");
+  else
+    current_disk=list_disk;
+  if(current_disk==NULL)
+    return 0;
+  if(start_ncurses("TestDisk"))
     return 1;
-  }
-  noecho();
-#ifndef DJGPP
-  nonl(); /*don't use for Dos version but enter will work with it... dilema */
-#endif
-  /*  intrflush(stdscr, FALSE); */
-  cbreak();
-  /* Should solve a problem with users who redefined the colors */
-  if(has_colors())
-  {
-    start_color();
-  }
-  curs_set(0);
   /* ncurses interface */
   while(done==0)
   {
@@ -219,60 +201,94 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
       }
     }
     current_disk->disk->halt_on_errors=halt_on_errors;
-    command = toupper(wmenuSelect(stdscr,INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
-	  "AGDCOTQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, menu));
+    if(current_cmd!=NULL)
+    {
+      while(current_cmd[0]==',')
+	current_cmd++;
+      if(strncmp(current_cmd,"analyze",7)==0 || strncmp(current_cmd,"analyse",7)==0)
+      {
+	current_cmd+=7;
+	command='A';
+      }
+      else if(strncmp(current_cmd,"geometry",8)==0)
+      {
+	current_cmd+=8;
+	command='G';
+      }
+      else if(strncmp(current_cmd,"advanced",8)==0)
+      {
+	current_cmd+=8;
+	command='T';
+      }
+      else if(strncmp(current_cmd,"options",7)==0)
+      {
+	current_cmd+=7;
+	command='O';
+      }
+      else
+      {
+	if(current_cmd[0]!='\0')
+	  ecrit_rapport("error >%s<\n",current_cmd);
+	command='Q';
+      }
+    }
+    else
+      command = wmenuSelect_ext(stdscr,INTER_MAIN_Y, INTER_MAIN_X, menuMain, 8,
+	    "AGDCOTQ", MENU_HORIZ | MENU_BUTTON | MENU_ACCEPT_OTHERS, &menu,&real_key);
     /* e for editor will be added when the editor will be better */
     switch(command)
     {
+      case 'a':
       case 'A':
-	interface_analyse(current_disk->disk,debug,saveheader);
-	interface_recovery(current_disk->disk,paranoid,debug, dump_ind, fast_mode,align,ask_part_order,expert);
-	menu=0;
+	interface_analyse(current_disk->disk,debug,saveheader,&current_cmd);
+	interface_recovery(current_disk->disk,paranoid,debug, dump_ind, fast_mode,align,ask_part_order,expert,&current_cmd);
 	break;
+      case 'd':
       case 'D':
 	wmove(stdscr,INTER_MAIN_Y,INTER_MAIN_X);
 	write_clean_table(current_disk->disk);
-	menu=1;
 	break;
+      case 'c':
       case 'C':
 	wmove(stdscr,INTER_MAIN_Y,INTER_MAIN_X);
 	write_MBR_code(current_disk->disk);
-	menu=2;
 	break;
+      case 'g':
       case 'G':
-	change_geometry(current_disk->disk);
-	menu=3;
+	change_geometry(current_disk->disk,&current_cmd);
 	break;
+      case 'o':
       case 'O':
 	{
 	  const t_arch_fnct *old_arch=*arch;
 	  int old_allow_partial_last_cylinder=allow_partial_last_cylinder;
-	  interface_options(&paranoid,&dump_ind,&fast_mode,&align,&allow_partial_last_cylinder,&ask_part_order,&halt_on_errors,&expert,arch);
+	  interface_options(&paranoid,&dump_ind,&fast_mode,&align,&allow_partial_last_cylinder,&ask_part_order,&halt_on_errors,&expert,arch,&current_cmd);
 	  for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
 	  {
 	    element_disk->disk->arch=*arch;
 	  }
 	  if((old_allow_partial_last_cylinder!=allow_partial_last_cylinder) ||
 	      (old_arch!=*arch))
-	    hd_parse_bis(list_disk,allow_partial_last_cylinder,*arch);
-	  menu=4;
+	    hd_parse_bis(list_disk,allow_partial_last_cylinder,*arch,debug);
 	}
 	break;
+      case 't':
       case 'T':
-	interface_adv(current_disk->disk, debug, dump_ind, expert);
-	menu=5;
+	interface_adv(current_disk->disk, debug, dump_ind, expert,&current_cmd);
 	break;
+      case 'q':
       case 'Q':
 	done = 1;
 	break;
+      case 'e':
       case 'E':
 	interface_editor(current_disk->disk);
 	break;
-      case MENU_UP:
+      case KEY_UP:
 	if(current_disk->prev!=NULL)
 	  current_disk=current_disk->prev;
 	break;
-      case MENU_DOWN:
+      case KEY_DOWN:
 	if(current_disk->next!=NULL)
 	  current_disk=current_disk->next;
 	break;
@@ -285,32 +301,55 @@ int do_curses_testdisk(int debug,int dump_ind, const t_list_disk *list_disk, con
 #endif
     }
   }
-  wclear(stdscr);
-  wrefresh(stdscr);
-  nl();
-  endwin();
+  end_ncurses();
   ecrit_rapport("\n");
   return 0;
 }
 
-void interface_analyse(t_param_disk *disk_car, const int debug, const int saveheader)
+void interface_analyse(t_param_disk *disk_car, const int debug, const int saveheader, char**current_cmd)
 {
   t_list_part *list_part;
+  int command;
   /* log */
+  struct MenuItem menuAnalyse[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q',"Proceed","Try to locate partition"},
+    { 'T', "Save","Save partition structure"},
+    { 0, NULL, NULL }
+  };
   ecrit_rapport("\nAnalyse ");
   ecrit_rapport("%s\n",disk_car->description(disk_car));
   aff_buffer(BUFFER_RESET,"Q");
   /* ncurses interface */
   aff_copy(stdscr);
-  mvwaddstr(stdscr,21,0,msg_PART_TYPE);
+  if(disk_car->arch->msg_part_type!=NULL)
+    mvwaddstr(stdscr,21,0,disk_car->arch->msg_part_type);
   wmove(stdscr,4,0);
   wdoprintf(stdscr,"%s\n",disk_car->description(disk_car));
   mvwaddstr(stdscr,5,0,"Check current partition structure");
   wmove(stdscr,6,0);
   wrefresh(stdscr);
+  wdoprintf(stdscr,msg_PART_HEADER_LONG);
   list_part=disk_car->arch->read_part(disk_car,debug,saveheader);
   ecrit_rapport("Current partitions:\n");
-  if(aff_buffer(BUFFER_DISPLAY,(list_part!=NULL?"QT":"Q"),stdscr)=='T')
+  if(*current_cmd!=NULL)
+  {
+    command='Q';
+    while(*current_cmd[0]==',')
+      (*current_cmd)++;
+    if(strncmp(*current_cmd,"backup",6)==0)
+    {
+      (*current_cmd)+=6;
+      if(list_part!=NULL)
+	command='T';
+    }
+    screen_buffer_to_log();
+  }
+  else
+    command=screen_buffer_display(stdscr,(list_part!=NULL?"QT":"Q"),menuAnalyse);
+  if(command=='T')
   {
     ecrit_rapport("Backup partition structure\n");
     partition_save(disk_car,list_part,debug);
@@ -318,9 +357,20 @@ void interface_analyse(t_param_disk *disk_car, const int debug, const int savehe
   delete_list_part(list_part);
 }
 
-int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext)
+int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext, char **current_cmd, int *menu)
 {
   t_list_part *parts;
+  struct MenuItem menuWrite[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q', "Quit","Return to main menu"},
+    { 'S', "Search!","Search deeper, try to find more partitions"},
+    { 'W', "Write","Write partition structure to disk"},
+    { 'E', "Extd Part","Maximize/Minimize extended partition"},
+    { 0, NULL, NULL }
+  };
+  int command;
   ecrit_rapport("\ninterface_write()\n");
   aff_buffer(BUFFER_RESET,"Q");
   aff_copy(stdscr);
@@ -337,124 +387,225 @@ int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_
   if(list_part==NULL)
   {
     aff_buffer(BUFFER_ADD," \nNo partition found or selected for recovery");
-    return aff_buffer(BUFFER_DISPLAY,can_search_deeper?"S":"",stdscr);
+    if(*current_cmd!=NULL)
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"search",6)==0)
+      {
+	(*current_cmd)+=6;
+	command='S';
+      }
+      else
+	command='Q';
+      screen_buffer_to_log();
+    }
+    else
+      command=screen_buffer_display_ext(stdscr,(can_search_deeper?"S":""),menuWrite,menu);
   }
   else
-    return aff_buffer(BUFFER_DISPLAY,(can_search_deeper?(can_ask_minmax_ext?"SWE":"SW"):(can_ask_minmax_ext?"WE":"W")),stdscr);
+  {
+    if(*current_cmd!=NULL)
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      command='Q';
+      if(strncmp(*current_cmd,"search",6)==0)
+      {
+	(*current_cmd)+=6;
+	if(can_search_deeper)
+	  command='S';
+      }
+      else if(strncmp(*current_cmd,"write",5)==0)
+      {
+	(*current_cmd)+=5;
+	if(disk_car->arch->write_part!=NULL)
+	  command='W';
+      }
+      screen_buffer_to_log();
+    }
+    else
+    {
+      char options[10];
+      options[0]=0;
+      if(can_search_deeper)
+	strcat(options,"S");
+      if(disk_car->arch->write_part!=NULL)
+	strcat(options,"W");
+      if(can_ask_minmax_ext)
+	strcat(options,"E");
+      command=screen_buffer_display_ext(stdscr,options,menuWrite,menu);
+    }
+  }
+  return command;
 }
 
-static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align, int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch)
+static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *align, int *allow_partial_last_cylinder,int *ask_part_order,int *halt_on_errors, unsigned int *expert, const t_arch_fnct **arch, char**current_cmd)
 {
   /* paranoid: search in partition
    * dump_ind dump sector Y/N
    * fast_mode Partitions are aligned on cylinder boundary Y/N
    * */
-  int done = FALSE;
-  int menu = 9;
-  /* ncurses interface */
-  while (done==FALSE)
+
+  if(*current_cmd!=NULL)
   {
-    int car;
-    struct MenuItem menuOptions[]=
+    int keep_asking=1;
+    do
     {
-      { 'P', NULL, "Search all partitions" },
-      { 'D',NULL,"Dump essential sectors" },
-      { 'C',NULL,"Partitions are aligned on cylinder/head boundary" },
-      { 'F',NULL,"Don't check every hard disk head" },
-      { 'A',NULL,""},
-      { 'O',NULL,""},
-      { 'H',NULL,"Halt on Disk I/O errors" },
-      { 'E',NULL,"Expert mode"},
-      { 'T',NULL,"Partition table type"},
-      { 'Q',"Quit","Return to main menu"},
-      { 0, NULL, NULL }
-    };
-    menuOptions[0].name=*paranoid?"Paranoid : Yes":"Paranoid : No";
-    menuOptions[1].name=*dump_ind?"Dump : Yes":"Dump : No";
-    switch(*align)
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"partition_i386",14)==0)
+      {
+	(*current_cmd)+=14;
+	*arch=&arch_i386;
+      }
+      else if(strncmp(*current_cmd,"partition_none",14)==0)
+      {
+	(*current_cmd)+=14;
+	*arch=&arch_none;
+      }
+      else if(strncmp(*current_cmd,"partition_sun",13)==0)
+      {
+	(*current_cmd)+=13;
+	*arch=&arch_sun;
+      }
+      else if(strncmp(*current_cmd,"partition_mac",13)==0)
+      {
+	(*current_cmd)+=13;
+	*arch=&arch_mac;
+      }
+      else
+	keep_asking=0;
+    } while(keep_asking>0);
+  }
+  else
+  {
+    int done = FALSE;
+    int menu = 9;
+    /* ncurses interface */
+    while (done==FALSE)
     {
-      case 0:
-	menuOptions[2].name="Cylinder boundary : No";
-	break;
-      case 1:
-	menuOptions[2].name="Cylinder boundary : Head boundary only";
-	break;
-      case 2:
-	menuOptions[2].name="Cylinder boundary : Yes";
-	break;
-    }
-    switch(*fast_mode)
-    {
-      case 0:
-	menuOptions[3].name="Mode : Fast";
-	break;
-      case 1:
-	menuOptions[3].name="Mode : Medium";
-	break;
-      default:
-	menuOptions[3].name="Mode : Slow";
-	break;
-    }
-    menuOptions[4].name=*allow_partial_last_cylinder?"Allow partial last cylinder : Yes":"Allow partial last cylinder : No";
-    menuOptions[5].name=*ask_part_order?"Ask partition order : Yes":"Ask partition order : No";
-    menuOptions[6].name=*halt_on_errors?"Halt on errors : Yes":"Halt on errors : No";
-    menuOptions[7].name=*expert?"Expert mode : Yes":"Expert mode : No";
-    menuOptions[8].name=(*arch)->part_name;
-    aff_copy(stdscr);
-    car=toupper( wmenuSelect(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "DCAOHMETQ", MENU_VERT, menu));
-    switch(car)
-    {
-      case 'P':
-	*paranoid=!*paranoid;
-	menu=0;
-	break;
-      case 'D':
-	*dump_ind=!*dump_ind;
-	menu=1;
-	break;
-      case 'C':
-	if(*align<2)
-	  (*align)++;
-	else
-	  *align=0;
-	menu=2;
-	break;
-      case 'F':
-	if(*fast_mode<2)
-	  (*fast_mode)++;
-	else
-	  *fast_mode=0;
-	menu=3;
-	break;
-      case 'A':
-	*allow_partial_last_cylinder=!*allow_partial_last_cylinder;
-	menu=4;
-	break;
-      case 'O':
-	*ask_part_order=!*ask_part_order;
-	menu=5;
-	break;
-      case 'H':
-	*halt_on_errors=!*halt_on_errors;
-	menu=6;
-	break;
-      case 'E':
-	*expert=!*expert;
-	menu=7;
-	break;
-      case 'T':
-       if(*arch==&arch_i386)
-	 *arch=&arch_sun;
-       else if(*arch==&arch_sun)
-	 *arch=&arch_none;
-       else
-	 *arch=&arch_i386;
-	menu=8;
-	break;
-      case key_ESC:
-      case 'Q':
-	done = TRUE;
-	break;
+      int car;
+      int real_key;
+      struct MenuItem menuOptions[]=
+      {
+	{ 'P', NULL, "Search all partitions" },
+	{ 'D',NULL,"Dump essential sectors" },
+	{ 'C',NULL,"Partitions are aligned on cylinder/head boundary" },
+	{ 'F',NULL,"Don't check every hard disk head" },
+	{ 'A',NULL,""},
+	{ 'O',NULL,""},
+	{ 'H',NULL,"Halt on Disk I/O errors" },
+	{ 'E',NULL,"Expert mode"},
+	{ 'T',NULL,"Partition table type"},
+	{ 'Q',"[Ok]","Return to main menu"},
+	{ 0, NULL, NULL }
+      };
+      menuOptions[0].name=*paranoid?"Paranoid : Yes":"Paranoid : No";
+      menuOptions[1].name=*dump_ind?"Dump : Yes":"Dump : No";
+      switch(*align)
+      {
+	case 0:
+	  menuOptions[2].name="Cylinder boundary : No";
+	  break;
+	case 1:
+	  menuOptions[2].name="Cylinder boundary : Head boundary only";
+	  break;
+	case 2:
+	  menuOptions[2].name="Cylinder boundary : Yes";
+	  break;
+      }
+      switch(*fast_mode)
+      {
+	case 0:
+	  menuOptions[3].name="Mode : Fast";
+	  break;
+	case 1:
+	  menuOptions[3].name="Mode : Medium";
+	  break;
+	default:
+	  menuOptions[3].name="Mode : Slow";
+	  break;
+      }
+      menuOptions[4].name=*allow_partial_last_cylinder?"Allow partial last cylinder : Yes":"Allow partial last cylinder : No";
+      menuOptions[5].name=*ask_part_order?"Ask partition order : Yes":"Ask partition order : No";
+      menuOptions[6].name=*halt_on_errors?"Halt on errors : Yes":"Halt on errors : No";
+      menuOptions[7].name=*expert?"Expert mode : Yes":"Expert mode : No";
+      menuOptions[8].name=(*arch)->part_name;
+      aff_copy(stdscr);
+
+      car=wmenuSelect_ext(stdscr,INTER_OPTION_Y, INTER_OPTION_X, menuOptions, 0, "DCAOHMETQ", MENU_VERT, &menu,&real_key);
+      switch(car)
+      {
+	case 'p':
+	case 'P':
+	  *paranoid=!*paranoid;
+	  break;
+	case 'd':
+	case 'D':
+	  *dump_ind=!*dump_ind;
+	  break;
+	case 'c':
+	case 'C':
+	  if(*align<2)
+	    (*align)++;
+	  else
+	    *align=0;
+	  break;
+	case 'f':
+	case 'F':
+	  if(*fast_mode<2)
+	    (*fast_mode)++;
+	  else
+	    *fast_mode=0;
+	  break;
+	case 'a':
+	case 'A':
+	  *allow_partial_last_cylinder=!*allow_partial_last_cylinder;
+	  break;
+	case 'o':
+	case 'O':
+	  *ask_part_order=!*ask_part_order;
+	  break;
+	case 'h':
+	case 'H':
+	  *halt_on_errors=!*halt_on_errors;
+	  break;
+	case 'e':
+	case 'E':
+	  *expert=!*expert;
+	  break;
+	case 't':
+	case 'T':
+	  if(real_key==KEY_LEFT)
+	  {
+	    if(*arch==&arch_i386)
+	      *arch=&arch_none;
+	    else if(*arch==&arch_sun)
+	      *arch=&arch_i386;
+	    else if(*arch==&arch_mac)
+	      *arch=&arch_sun;
+	    else
+	      *arch=&arch_mac;
+	  }
+	  else
+	  {
+	    if(*arch==&arch_i386)
+	      *arch=&arch_sun;
+	    else if(*arch==&arch_sun)
+	      *arch=&arch_mac;
+	    else if(*arch==&arch_mac)
+	      *arch=&arch_none;
+	    else
+	      *arch=&arch_i386;
+	  }
+	  break;
+	case key_ESC:
+	case 'q':
+	case 'Q':
+	  done = TRUE;
+	  break;
+      }
     }
   }
   /* write new options to log file */
@@ -486,15 +637,15 @@ static void interface_options(int *paranoid,int *dump_ind,int *fast_mode,int *al
       ecrit_rapport("Mode : Slow");
       break;
   }
-  ecrit_rapport("\n Allow partial last cylinder : %s\n Ask partition order : %s\n Halt on errors : %s\n Expert mode : %s\n Partition table type : %s\n",
+  ecrit_rapport("\n Allow partial last cylinder : %s\n Ask partition order : %s\n Halt on errors : %s\n Expert mode : %s\n %s\n",
       *allow_partial_last_cylinder?"Yes":"No",
       *ask_part_order?"Yes":"No",
       *halt_on_errors?"Yes":"No",
       *expert?"Yes":"No",
-      (*arch==&arch_i386?"Intel":(*arch==&arch_sun?"Sun":"None")));
+      (*arch)->part_name);
 }
 
-t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug)
+t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd)
 {
   int quit=0;
   int offset=0;
@@ -504,8 +655,9 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
   do
   {
     int i;
-    int car;
+    int command;
     t_list_part *parts;
+    int structure_status;
     if(rewrite)
     {
       aff_copy(stdscr);
@@ -514,6 +666,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       mvwaddstr(stdscr,5,0,msg_PART_HEADER);
       rewrite=0;
     }
+    structure_status=disk_car->arch->test_structure(list_part);
     for(i=0,parts=list_part;(parts!=NULL) && (i<offset);parts=parts->next,i++);
     for(i=offset;(parts!=NULL) &&((i-offset)<INTER_STRUCTURE);i++,parts=parts->next)
     {
@@ -522,7 +675,15 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       if(parts==pos)
       {
 	wattrset(stdscr,A_STANDOUT);
-	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
+      }
+      /* TODO status letter in green unless D(eleted) or structure_status!=0 */
+      if(structure_status==0 && parts->part->status!=STATUS_DELETED && has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(2));
+      aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
+      if(structure_status==0 && parts->part->status!=STATUS_DELETED && has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(0));
+      if(parts==pos)
+      {
 	wattroff(stdscr,A_STANDOUT);
 	wmove(stdscr,24,0);
 	wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
@@ -531,19 +692,22 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	  wdoprintf(stdscr,"%s, ",parts->part->info);
 	}
 	wdoprintf(stdscr,"%lu MB",(long unsigned)(parts->part->part_size/(1024*1024)));
-      } else
-      {
-	aff_part(stdscr,AFF_PART_NONL,disk_car,parts->part);
       }
     }
-    if(disk_car->arch->test_structure(list_part)!=0)
-      mvwaddstr(stdscr,20,0,msg_STRUCT_BAD);
-    else
+    if(structure_status==0)
       mvwaddstr(stdscr,20,0,msg_STRUCT_OK);
-    if(list_part!=NULL)
+    else
+    {
+      if(has_colors())
+	wbkgdset(stdscr,' ' | COLOR_PAIR(1));
+      mvwaddstr(stdscr,20,0,msg_STRUCT_BAD);
+      if(has_colors())
+	bkgdset(' ' | COLOR_PAIR(0));
+    }
+    if(list_part!=NULL && disk_car->arch->msg_part_type!=NULL)
     {
       mvwaddstr(stdscr,20,16,"Use arrow keys to change partition characteristics:");
-      mvwaddstr(stdscr,21,0,msg_PART_TYPE);
+      mvwaddstr(stdscr,21,0,disk_car->arch->msg_part_type);
     }
     wmove(stdscr,22,0);
     wclrtoeol(stdscr);	/* before addstr for BSD compatibility */
@@ -577,8 +741,21 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
       mvwaddstr(stdscr,23,5, "ENTER: to continue");
     }
     wrefresh(stdscr);
-    car=wgetch(stdscr);
-    switch(car)
+    if(*current_cmd!=NULL)
+    {
+      while(*current_cmd[0]==',')
+	(*current_cmd)++;
+      if(strncmp(*current_cmd,"list",4)==0)
+      {
+	(*current_cmd)+=4;
+	command='p';
+      }
+      else
+	command='q';
+    }
+    else
+      command=wgetch(stdscr);
+    switch(command)
     {
       case 'q':
       case '\r':
@@ -692,19 +869,19 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	    case UP_FAT12:
 	    case UP_FAT16:
 	    case UP_FAT32:
-	      dir_partition_fat(window,disk_car,pos->part,debug);
+	      dir_partition_fat(window,disk_car,pos->part,debug,current_cmd);
 	      break;
 	    case UP_EXT2:
 	    case UP_EXT3:
-	      dir_partition_ext2(window,disk_car,pos->part,debug);
+	      dir_partition_ext2(window,disk_car,pos->part,debug,current_cmd);
 	      break;
 	    case UP_RFS:
 	    case UP_RFS2:
 	    case UP_RFS3:
-	      dir_partition_reiser(window,disk_car,pos->part,debug);
+	      dir_partition_reiser(window,disk_car,pos->part,debug,current_cmd);
 	      break;
 	    case UP_NTFS:
-	      dir_partition_ntfs(window,disk_car,pos->part,debug);
+	      dir_partition_ntfs(window,disk_car,pos->part,debug,current_cmd);
 	      break;
 	    default:
 	      break;
@@ -725,7 +902,7 @@ t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const
 	pos=list_part;
         break;
       default:
-/*	ecrit_rapport("ask_structure car=%x\n",car); */
+/*	ecrit_rapport("ask_structure command=%x\n",command); */
 	break;
     }
   } while(quit==0);
@@ -769,8 +946,8 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
     {
       { 'P', "Previous",""},
       { 'N', "Next","" },
-      { 'L',"Load","Load partition structure from backup"},
-      { 'Q',"Quit","Quit dump section"},
+      { 'L',"Load","Load partition structure from backup and try to location partition"},
+      { 'Q',"Cancel","Don't use backup and try to locate partition"},
       { 0, NULL, NULL }
     };
     char options[20];
@@ -836,7 +1013,7 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
 	case 'L':
 	  quit=2;
 	  break;
-	case MENU_UP:
+	case KEY_UP:
 	  if(list_backup!=NULL)
 	  {
 	    if(pos->prev!=NULL)
@@ -848,7 +1025,7 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
 	      offset--;
 	  }
 	  break;
-	case MENU_DOWN:
+	case KEY_DOWN:
 	  if(list_backup!=NULL)
 	  {
 	    if(pos->next!=NULL)
@@ -914,10 +1091,17 @@ static t_list_part *interface_load(t_param_disk *disk_car,t_list_part *list_part
   return list_part;
 }
 
-int interface_superblock(t_param_disk *disk_car,t_list_part *list_part)
+int interface_superblock(t_param_disk *disk_car,t_list_part *list_part, char**current_cmd)
 {
   t_list_part *parts;
   t_partition *old_part=NULL;
+  struct MenuItem menuSuperblock[]=
+  {
+    { 'P', "Previous",""},
+    { 'N', "Next","" },
+    { 'Q',"Quit","Return to Advanced menu"},
+    { 0, NULL, NULL }
+  };
   aff_buffer(BUFFER_RESET,"Q");
   aff_copy(stdscr);
   wmove(stdscr,4,0);
@@ -928,7 +1112,9 @@ int interface_superblock(t_param_disk *disk_car,t_list_part *list_part)
   {
     if(old_part==NULL || (old_part->part_offset!=parts->part->part_offset) ||
       old_part->part_size!=parts->part->part_size ||
-      old_part->part_type!=parts->part->part_type ||
+      old_part->part_type_i386!=parts->part->part_type_i386 ||
+      old_part->part_type_sun!=parts->part->part_type_sun ||
+      old_part->part_type_mac!=parts->part->part_type_mac ||
       old_part->upart_type!=parts->part->upart_type)
     {
       aff_part_buffer(AFF_PART_SHORT,disk_car,parts->part);
@@ -936,5 +1122,10 @@ int interface_superblock(t_param_disk *disk_car,t_list_part *list_part)
     }
     aff_buffer(BUFFER_ADD,"superblock %u, blocksize=%u\n",parts->part->boot_sector,parts->part->blocksize);
   }
-  return aff_buffer(BUFFER_DISPLAY,"",stdscr);
+  if(*current_cmd!=NULL)
+  {
+    screen_buffer_to_log();
+    return 0;
+  }
+  return screen_buffer_display(stdscr,"",menuSuperblock);
 }
