@@ -25,11 +25,11 @@
 #define INTER_STRUCTURE	14
 #define INTER_BAD_PART	12
 
-int do_curses_testdisk(int debug, int dump_ind, const t_list_disk *list_disk, const int saveheader, const t_arch_fnct **arch);
-int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext);
+int do_curses_testdisk(int debug, int dump_ind, const t_list_disk *list_disk, const int saveheader, const t_arch_fnct **arch, const char *cmd_device, char *cmd_run);
+int interface_write(t_param_disk *disk_car,t_list_part *list_part,const int can_search_deeper, const int can_ask_minmax_ext, char **current_cmd, int *menu);
 
-t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug);
-void interface_analyse(t_param_disk *disk_car, const int debug, const int saveheader);
+t_list_part *ask_structure(t_param_disk *disk_car,t_list_part *list_part, const int debug, char **current_cmd);
+void interface_analyse(t_param_disk *disk_car, const int debug, const int saveheader, char **current_cmd);
 void interface_list(t_param_disk *disk_car, const int debug,const int test_recovery, const int saveheader, const int backup);
-int interface_superblock(t_param_disk *disk_car,t_list_part *list_part);
+int interface_superblock(t_param_disk *disk_car,t_list_part *list_part,char**current_cmd);
 
