@@ -2,7 +2,7 @@
 
     File: jfs.c
 
-    Copyright (C) 2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -48,13 +48,12 @@ int check_JFS(t_param_disk *disk_car,t_partition *partition,const int debug)
 
 int set_JFS_info(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partition *partition,const int debug, const int dump_ind)
 {
-  partition->info[0]='\0';
+  snprintf(partition->info,sizeof(partition->info),"JFS %u",(unsigned int)sb->s_version);
   partition->name[0]='\0';
   if(sb->s_version==1)
   {
     set_part_name(partition,sb->s_fpack,11);
   }
-  snprintf(partition->info,sizeof(partition->info),"JFS %u",(unsigned int)sb->s_version);
   return 0;
 }
 
@@ -66,12 +65,14 @@ int recover_JFS(t_param_disk *disk_car, const struct jfs_superblock *sb,t_partit
   if(test_JFS(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
   set_JFS_info(disk_car,sb,partition,debug,dump_ind);
-  partition->part_type=(unsigned char)P_LINUX;
+  partition->part_type_i386=P_LINUX;
+  partition->part_type_sun=PSUN_LINUX;
+  partition->part_type_mac=PMAC_LINUX;
   partition->part_size=(uint64_t)sb->s_pbsize*le64(sb->s_size)+
     sb->s_bsize*(le24(sb->s_fsckpxd.len)+le24(sb->s_logpxd.len));
   partition->boot_sector=0;
   partition->blocksize=le32(sb->s_bsize);
-  if(debug!=0)
+  if(debug>0)
   {
     ecrit_rapport("\n");
     ecrit_rapport("recover_JFS: s_blocksize=%lu\n",partition->blocksize);
