@@ -2,7 +2,7 @@
 
     File: netware.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,15 +29,35 @@
 #include "types.h"
 #include "common.h"
 #include "netware.h"
+static int test_netware(t_param_disk *disk_car, const struct disk_netware *netware_block,t_partition *partition,const int debug, const int dump_ind);
 
-int recover_netware(t_param_disk *disk_car, const struct disk_netware *netware_block,t_partition *partition)
+static int test_netware(t_param_disk *disk_car, const struct disk_netware *netware_block,t_partition *partition,const int debug, const int dump_ind)
 {
   if(memcmp(netware_block->magic,"Nw_PaRtItIoN",12)==0)
   {
-    partition->part_size=(uint64_t)netware_block->nbr_sectors*disk_car->sector_size;
-    partition->part_type=(unsigned char)P_NETWARE;
     partition->upart_type=UP_NETWARE;
     return 0;
   }
   return 1;
 }
+
+int check_netware(t_param_disk *disk_car,t_partition *partition,const int debug)
+{
+  unsigned char buffer[8*DEFAULT_SECTOR_SIZE];
+  if(disk_car->read(disk_car,sizeof(buffer), &buffer, partition->part_offset)!=0)
+  { return 1; }
+  if(test_netware(disk_car,(const struct disk_netware *)&buffer,partition,debug,0)!=0)
+    return 1;
+  return 0;
+}
+
+int recover_netware(t_param_disk *disk_car, const struct disk_netware *netware_block,t_partition *partition)
+{
+  if(test_netware(disk_car,netware_block,partition,0,0)!=0)
+    return 1;
+  partition->part_type_i386=P_NETWARE;
+  partition->part_size=(uint64_t)netware_block->nbr_sectors*disk_car->sector_size;
+  partition->name[0]='\0';
+  partition->info[0]='\0';
+  return 0;
+}
