@@ -30,10 +30,9 @@ struct ntfs_boot_sector {
 	} __attribute__ ((__packed__));
 int check_NTFS(t_param_disk *disk_car,t_partition *partition,const int debug,const int dump_ind);
 int dir_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition);
-int dir_partition_ntfs(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug);
 int dump_2ntfs_rapport(const struct ntfs_boot_sector *nh1, const struct ntfs_boot_sector *nh2);
 int dump_ntfs_rapport(const struct ntfs_boot_sector *ntfs_header);
-int is_ntfs(const int part_type);
+int is_ntfs(const t_partition *partition);
 int ntfs_get_attr(const char *mft_record, const int my_type, t_partition *partition, const char *end, const int debug, const int dump_ind, const char*file_name_to_find);
 int recover_NTFS(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind, const int backup);
 int set_NTFS_info(t_param_disk *disk_car, const struct ntfs_boot_sector*ntfs_header,t_partition *partition,const int debug, const int dump_ind);
