@@ -2,7 +2,7 @@
 
     File: parti386.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#include <ctype.h>      /* toupper, tolower */
+#include <ctype.h>      /* tolower */
 #include "types.h"
 #include "common.h"
 #include "testdisk.h"
@@ -101,114 +101,120 @@ static int diff(const t_param_disk *disk_car,const unsigned char buffer[DEFAULT_
 static t_list_part *read_part_i386(t_param_disk *disk_car, const int debug, const int saveheader);
 static int write_part_i386(t_param_disk *disk_car, const t_list_part *list_part, const int ro , const int debug, const int align);
 static t_list_part *init_part_order_i386(const t_param_disk *disk_car, t_list_part *list_part);
-static int write_MBR_code_i386(unsigned char *buffer);
+static int write_MBR_code_i386(t_param_disk *disk_car);
+static int write_MBR_code_i386_aux(unsigned char *buffer);
 static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_part, const int debug);
 static void set_prev_status_i386(const t_param_disk *disk_car, t_partition *partition);
 static void set_next_status_i386(const t_param_disk *disk_car, t_partition *partition);
-static int is_part_type_ok_i386(unsigned int part_type);
+static int set_part_type_i386(t_partition *partition, unsigned int part_type);
+static int is_part_known_i386(const t_partition *partition);
 static void init_structure_i386(const t_param_disk *disk_car,t_list_part *list_part, const int debug);
-static void erase_list_part_i386(unsigned char *buffer);
+static int erase_list_part_i386(t_param_disk *disk_car);
 static int check_part_i386(t_param_disk *disk_car, const int debug,t_partition *partition,const int saveheader);
 
 static void partition2_i386_entry(const t_param_disk *disk_car,const uint64_t pos,const t_partition *partition, struct partition_dos *p);
 static int i386_entry2partition(t_param_disk *disk_car, const uint64_t offset, t_partition *partition, const struct partition_dos *p, const status_type_t status,const unsigned int order,const int debug, const int saveheader);
 static const char* errmsg_i386_entry2partition(const errcode_type_t errcode);
+static const unsigned char *get_partition_name_i386(const t_partition *partition);
+static const unsigned char *get_partition_name_i386_aux(const unsigned int part_type_i386);
+static unsigned int get_part_type_i386(const t_partition *partition);
+
 static const struct systypes i386_sys_types[] = {
-  {"\x00" "empty"},
-  {"\x01" "FAT12"},
-  {"\x02" "XENIX root"},
-  {"\x03" "XENIX /usr"},
-  {"\x04" "FAT16 <32M"},
-  {"\x05" "extended"},
-  {"\x06" "FAT16 >32M"},
-  {"\x07" "HPFS - NTFS"},
-  {"\x09" "AIX data"},
-  {"\x0a" "OS/2 Boot Manager"},
-  {"\x0b" "FAT32"},
-  {"\x0c" "FAT32 LBA"},
-  {"\x0E" "FAT16 LBA"},
-  {"\x0F" "extended LBA"},
-  {"\x10" "OPUS"},
-  {"\x11" "hid. FAT12"},
-  {"\x12" "Compaq Diagnostics"},
-  {"\x14" "hid. FAT16 <32M"},
-  {"\x16" "hid. FAT16 >32M"},
-  {"\x17" "hid. HPFS/NTFS"},
-  {"\x18" "AST swap"},
-  {"\x19" "Willowtech Photon"},
-  {"\x1b" "hid. FAT32"},
-  {"\x1c" "hid. FAT32 LBA"},
-  {"\x1e" "hid. FAT16 LBA"},
-  {"\x20" "Willowsoft OFS1"},
-  {"\x24" "NEC MS-DOS 3.x"},
-  {"\x38" "Theos"},
-  {"\x3c" "PMagic recovery"},
-  {"\x40" "VENIX 80286"},
-  {"\x41" "PPC PReP Boot"},
-  {"\x42" "W2K Dynamic/SFS"},
-  {"\x50" "OnTrack DM RO"},
-  {"\x51" "OnTrack DM RW-NOVEL"},
-  {"\x52" "CP/M-Microport V/386"},
-  {"\x53" "OnTrack DM WO ???"},
-  {"\x54" "OnTrack DM DDO"},
-  {"\x55" "EZ-Drive"},
-  {"\x56" "GoldenBow VFeature"},
-  {"\x61" "SpeedStor"},
-  {"\x63" "Unixware, HURD, SCO"},
-  {"\x64" "NetWare 286"},
-  {"\x65" "NetWare 3.11+"},
-  {"\x67" "Novell"},
-  {"\x68" "Novell"},
-  {"\x69" "Novell"},
-  {"\x70" "DiskSecure MB"},
-  {"\x75" "PC/IX"},
-  {"\x80" "Minix v1.1-1.4a"},
-  {"\x81" "Minix / old Linux"},
-  {"\x82" "Linux Swap"},
-  {"\x83" "Linux"},
-  {"\x85" "Linux extended"},
-  {"\x86" "NT FAT16 V/S set"},
-  {"\x87" "HPFS FT mirror-V/S set"},
-  {"\x8e" "Linux LVM"},
-  {"\x93" "Amoeba"},
-  {"\x94" "Amoeba bad block"},
-  {"\xa0" "NoteBIOS save2disk"},
-  {"\xa5" "FreeBSD"},
-  {"\xa6" "OpenBSD"},
-  {"\xa8" "Darwin UFS"},
-  {"\xa9" "NetBSD"},
-  {"\xab" "Darwin boot"},
-  {"\xaf" "HFS"},
-  {"\xb7" "BSDI"},
-  {"\xb8" "BSDI swap"},
-  {"\xbe" "Solaris boot"},
-  {"\xbf" "Solaris"},
-  {"\xc1" "secured FAT12"},
-  {"\xc4" "secured FAT16"},
-  {"\xc6" "sec. Huge-bad FAT16"},
-  {"\xc7" "Syrinx Boot-bad NTFS"},
-  {"\xd8" "CP/M-86"},
-  {"\xdb" "CP/M"},
-  {"\xde" "Dell Utility"},
-  {"\xe1" "SpeedStor FAT12 ext"},
-  {"\xe3" "DOS RO"},
-  {"\xe4" "SpeedStor FAT16 ext"},
-  {"\xeb" "BeFS"},
-  {"\xee" "EFI GPT"},          /* Intel EFI GUID Partition Table */
-  {"\xef" "EFI (FAT-12/16/32)"},/* Intel EFI System Partition */
-  {"\xf0" "Linux/PA-RISC boot"},/* Linux/PA-RISC boot loader */
-  {"\xf1" "Storage Dimensions"},
-  {"\xf2" "DOS secondary"},
-  {"\xf4" "SpeedStor"},
-  {"\xfd" "Linux RAID"},
-  {"\xfe" "LANstep"},
-  {"\xff" "Xenix bad block"},
-  { NULL }
+  {P_NO_OS,		"empty"},
+  {P_12FAT,		"FAT12"},
+  {0x02,		"XENIX root"},
+  {0x03,		"XENIX /usr"},
+  {P_16FAT,		"FAT16 <32M"},
+  {P_EXTENDED,		"extended"},
+  {P_16FATBD,		"FAT16 >32M"},
+  {P_NTFS,		"HPFS - NTFS"},
+  {0x09,		"AIX data"},
+  {P_OS2MB,		"OS/2 Boot Manager"},
+  {P_32FAT,		"FAT32"},
+  {P_32FAT_LBA,		"FAT32 LBA"},
+  {P_16FATBD_LBA,	"FAT16 LBA"},
+  {P_EXTENDX,		"extended LBA"},
+  {0x10,		"OPUS"},
+  {P_12FATH,		"hid. FAT12"},
+  {0x12,		"Compaq Diagnostics"},
+  {P_16FATH,		"hid. FAT16 <32M"},
+  {P_16FATBDH,		"hid. FAT16 >32M"},
+  {P_NTFSH,		"hid. HPFS/NTFS"},
+  {0x18,		"AST swap"},
+  {0x19,		"Willowtech Photon"},
+  {P_32FATH,		"hid. FAT32"},
+  {P_32FAT_LBAH,	"hid. FAT32 LBA"},
+  {P_16FATBD_LBAH,	"hid. FAT16 LBA"},
+  {0x20,		"Willowsoft OFS1"},
+  {0x24,		"NEC MS-DOS 3.x"},
+  {0x38,		"Theos"},
+  {0x3c,		"PMagic recovery"},
+  {0x40,		"VENIX 80286"},
+  {0x41,		"PPC PReP Boot"},
+  {0x42,		"W2K Dynamic/SFS"},
+  {0x50,		"OnTrack DM RO"},
+  {0x51,		"OnTrack DM RW-NOVEL"},
+  {0x52,		"CP/M-Microport V/386"},
+  {0x53,		"OnTrack DM WO ???"},
+  {0x54,		"OnTrack DM DDO"},
+  {0x55,		"EZ-Drive"},
+  {0x56,		"GoldenBow VFeature"},
+  {0x61,		"SpeedStor"},
+  {P_SYSV,		"Unixware, HURD, SCO"},
+  {0x64,		"NetWare 286"},
+  {P_NETWARE,		"NetWare 3.11+"},
+  {0x67,		"Novell"},
+  {0x68,		"Novell"},
+  {0x69,		"Novell"},
+  {0x70,		"DiskSecure MB"},
+  {0x75,		"PC/IX"},
+  {0x80,		"Minix v1.1-1.4a"},
+  {P_OLDLINUX,		"Minix / old Linux"},
+  {P_LINSWAP,		"Linux Swap"},
+  {P_LINUX,		"Linux"},
+  {P_LINUXEXTENDX,	"Linux extended"},
+  {0x86,		"NT FAT16 V/S set"},
+  {0x87,		"HPFS FT mirror-V/S set"},
+  {P_LVM,		"Linux LVM"},
+  {0x93,		"Amoeba"},
+  {0x94,		"Amoeba bad block"},
+  {0xa0,		"NoteBIOS save2disk"},
+  {P_FREEBSD,		"FreeBSD"},
+  {P_OPENBSD,		"OpenBSD"},
+  {0xa8,		"Darwin UFS"},
+  {P_NETBSD,		"NetBSD"},
+  {0xab,		"Darwin boot"},
+  {P_HFS,		"HFS"},
+  {0xb7,		"BSDI"},
+  {0xb8,		"BSDI swap"},
+  {0xbe,		"Solaris boot"},
+  {P_SUN,		"Solaris"},
+  {0xc1,		"secured FAT12"},
+  {0xc4,		"secured FAT16"},
+  {0xc6,		"sec. Huge-bad FAT16"},
+  {0xc7,		"Syrinx Boot-bad NTFS"},
+  {0xd8,		"CP/M-86"},
+  {0xdb,		"CP/M"},
+  {0xde,		"Dell Utility"},
+  {0xe1,		"SpeedStor FAT12 ext"},
+  {0xe3,		"DOS RO"},
+  {0xe4,		"SpeedStor FAT16 ext"},
+  {P_BEOS,		"BeFS"},
+  {0xee,		"EFI GPT"},          /* Intel EFI GUID Partition Table */
+  {0xef,		"EFI (FAT-12/16/32)"},/* Intel EFI System Partition */
+  {0xf0,		"Linux/PA-RISC boot"},/* Linux/PA-RISC boot loader */
+  {0xf1,		"Storage Dimensions"},
+  {0xf2,		"DOS secondary"},
+  {0xf4,		"SpeedStor"},
+  {P_RAID,		"Linux RAID"},
+  {0xfe,		"LANstep"},
+  {0xff,		"Xenix bad block"},
+  {P_NO_OS,		NULL }
 };
 
 t_arch_fnct arch_i386= {
   .part_name="Partition table type : Intel",
-  .parttype_name_table=i386_sys_types,
+  .msg_part_type="*=Primary bootable  P=Primary  L=Logical  E=Extended  D=Deleted",
   .read_part=read_part_i386,
   .write_part=write_part_i386,
   .init_part_order=init_part_order_i386,
@@ -219,11 +225,19 @@ t_arch_fnct arch_i386= {
   .set_prev_status=set_prev_status_i386,
   .set_next_status=set_next_status_i386,
   .test_structure=test_structure_i386,
-  .is_part_type_ok=is_part_type_ok_i386,
+  .set_part_type=set_part_type_i386,
+  .is_part_known=is_part_known_i386,
   .init_structure=init_structure_i386,
-  .erase_list_part=erase_list_part_i386
+  .erase_list_part=erase_list_part_i386,
+  .get_partition_name=get_partition_name_i386,
+  .get_part_type=get_part_type_i386
 };
 
+static unsigned int get_part_type_i386(const t_partition *partition)
+{
+  return partition->part_type_i386;
+}
+
 static void store4_little_endian(unsigned char *cp, unsigned int val)
 {
   cp[0] = (val & 0xff);
@@ -326,8 +340,11 @@ static t_list_part *read_part_i386(t_param_disk *disk_car, const int debug, cons
   int res=0;
   t_CHS geometry;
   aff_buffer(BUFFER_RESET,"Q");
-  if(read_MBR(disk_car,&buffer)!=0)
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    wdoprintf(stdscr,msg_PART_RD_ERR);
     return NULL;
+  }
   if(get_geometry_from_i386mbr(buffer,debug,&geometry)!=0)
   {
     aff_buffer(BUFFER_ADD,msg_TBL_NMARK);
@@ -377,7 +394,6 @@ static void test_MBR_data(t_list_part *list_part)
 {
   t_list_part *element;
   unsigned int nb_dos=0, nb_hidden=0, nb_mb=0, nb_ext=0, nb_boot=0;
-  wdoprintf(stdscr,msg_PART_HEADER_LONG);
   for(element=list_part;element!=NULL;element=element->next)
   {
     const t_partition *partition=element->part;
@@ -386,7 +402,7 @@ static void test_MBR_data(t_list_part *list_part)
       case STATUS_PRIM_BOOT:
 	nb_boot++;
       case STATUS_PRIM:
-	switch(partition->part_type)
+	switch(partition->part_type_i386)
 	{
 	  case P_12FAT:
 	  case P_16FAT:
@@ -589,8 +605,10 @@ static int test_MBR_over(t_param_disk *disk_car,t_list_part *list_part)
 
 int write_part_i386(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug, const int align)
 {
-  write_mbr_i386(disk_car,list_part,ro,debug);
-  return write_all_log_i386(disk_car,list_part,ro,debug,align);
+  int res=0;
+  res+=write_mbr_i386(disk_car,list_part,ro,debug);
+  res+=write_all_log_i386(disk_car,list_part,ro,debug,align);
+  return res;
 }
 
 static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part, const int ro, const int debug)
@@ -602,8 +620,9 @@ static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part,
   {
     ecrit_rapport("\nwrite_mbr_i386: starting...\n");
   }
-  if(read_MBR(disk_car,&buffer_org))
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer_org, (uint64_t)0))
   {
+    wdoprintf(stdscr,msg_PART_RD_ERR);
     memset((void *)&buffer_org,0,DEFAULT_SECTOR_SIZE);
   }
   memset((void *)&buffer,0,DEFAULT_SECTOR_SIZE);
@@ -613,7 +632,9 @@ static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part,
     buffer[0x1FE]=0x55;
     buffer[0x1FF]=0xAA;
   } else {
-    write_MBR_code_i386(buffer);
+    if(debug>0)
+      ecrit_rapport("Store new MBR code\n");
+    write_MBR_code_i386_aux(buffer);
   }
   for(element=list_part;element!=NULL;element=element->next)
   {
@@ -646,7 +667,12 @@ static int write_mbr_i386(t_param_disk *disk_car, const t_list_part *list_part,
     diff(disk_car,buffer,buffer_org);
   }
   if(ro==0)
-    return write_MBR(disk_car,&buffer);
+  {
+    if(disk_car->write(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+    {
+      return 1;
+    }
+  }
   return 0;
 }
 
@@ -656,13 +682,14 @@ static int write_all_log_i386(t_param_disk *disk_car, const t_list_part *list_pa
   const t_list_part *pos_ext=NULL;
   uint64_t current_pos;
   t_partition *bloc_nextext;
-  if(debug)
+  int res=0;
+  if(debug>0)
     ecrit_rapport("write_all_log_i386: starting...\n");
   for(element=list_part;element!=NULL;element=element->next)
   {
     if(element->part->status==STATUS_EXT)
     {
-      if(is_extended(element->part->part_type))
+      if(is_extended(element->part->part_type_i386))
       {
 	if(pos_ext!=NULL)
 	  ecrit_rapport("write_all_log_i386: pos_ext already defined\n");
@@ -676,12 +703,12 @@ static int write_all_log_i386(t_param_disk *disk_car, const t_list_part *list_pa
   }
   if(pos_ext==NULL)
   {
-    wdoprintf(stdscr,msg_NO_EXT_PART);
-    return 1;
+    ecrit_rapport(msg_NO_EXT_PART);
+    return 0;
   }
   current_pos=pos_ext->part->part_offset;
   bloc_nextext=(t_partition *)MALLOC(sizeof(*bloc_nextext));
-  bloc_nextext->part_type=P_EXTENDED;       /* Never P_EXTENDX */
+  bloc_nextext->part_type_i386=P_EXTENDED;       /* Never P_EXTENDX */
   if(pos_ext->next==NULL || (pos_ext->next->part->status!=STATUS_LOG))
   {
     unsigned char buffer[DEFAULT_SECTOR_SIZE];
@@ -707,8 +734,7 @@ static int write_all_log_i386(t_param_disk *disk_car, const t_list_part *list_pa
     {
       if(disk_car->write(disk_car,sizeof(buffer), &buffer, current_pos))
       {
-	ecrit_rapport(msg_PART_WR_ERR);
-	display_message(msg_PART_WR_ERR);
+	res=1;
       }
     }
   }
@@ -791,15 +817,14 @@ static int write_all_log_i386(t_param_disk *disk_car, const t_list_part *list_pa
       {
 	if(disk_car->write(disk_car,sizeof(buffer), &buffer, current_pos))
 	{
-	  ecrit_rapport(msg_PART_WR_ERR);
-	  display_message(msg_PART_WR_ERR);
+	  res=1;
 	}
       }
       current_pos=bloc_nextext->part_offset;
     }
   }
   FREE(bloc_nextext);
-  return 0;
+  return res;
 }
 
 static int diff(const t_param_disk *disk_car, const unsigned char buffer[DEFAULT_SECTOR_SIZE], const unsigned char buffer_org[DEFAULT_SECTOR_SIZE])
@@ -830,7 +855,23 @@ static int diff(const t_param_disk *disk_car, const unsigned char buffer[DEFAULT
   return 0;
 }
 
-static int write_MBR_code_i386(unsigned char *buffer)
+static int write_MBR_code_i386(t_param_disk *disk_car)
+{
+  unsigned char buffer[DEFAULT_SECTOR_SIZE];
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    wdoprintf(stdscr,msg_PART_RD_ERR);
+    memset(buffer,0,sizeof(buffer));
+  }
+  write_MBR_code_i386_aux(buffer);
+  if(disk_car->write(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    return 1;
+  }
+  return 0;
+}
+
+static int write_MBR_code_i386_aux(unsigned char *buffer)
 {
   /* od -t x1 -v testdisk.b
      Thanks to Neil Turton for writing it */
@@ -883,7 +924,7 @@ static void partition2_i386_entry(const t_param_disk *disk_car, const uint64_t p
     p->boot_ind=0x80;
   else
     p->boot_ind=0;             /* Non bootable */
-  p->sys_ind=partition->part_type;
+  p->sys_ind=partition->part_type_i386;
   set_start_sect(p,(partition->part_offset-pos)/disk_car->sector_size);
   if(start.cylinder>1023)
   { /* Partition Magic 5 uses CHS=(1023,0,1) if extended or last logical *
@@ -918,7 +959,7 @@ static int i386_entry2partition(t_param_disk *disk_car, const uint64_t offset, t
   t_CHS start,end;
   t_CHS start_calculated,end_calculated;
   partition_reset(partition);
-  partition->part_type=p->sys_ind;
+  partition->part_type_i386=p->sys_ind;
   partition->part_offset=offset+(uint64_t)get_start_sect(p)*disk_car->sector_size;
   partition->order=order;
   partition->part_size=(uint64_t)get_nr_sects(p)*disk_car->sector_size;
@@ -936,7 +977,7 @@ static int i386_entry2partition(t_param_disk *disk_car, const uint64_t offset, t
   switch(status)
   {
     case STATUS_PRIM:
-      if(is_extended(partition->part_type))
+      if(is_extended(partition->part_type_i386))
       {
 	partition->status=STATUS_EXT;
 	partition->upart_type=UP_EXTENDED;
@@ -1021,8 +1062,8 @@ static const char* errmsg_i386_entry2partition(const errcode_type_t errcode)
 
 static void aff_entry_rapport(const t_param_disk *disk_car, const struct partition_dos *entree)
 {
-  if(partition_type(entree->sys_ind,disk_car->arch->parttype_name_table)!=NULL)
-    ecrit_rapport(" %-20s ",partition_type(entree->sys_ind,disk_car->arch->parttype_name_table));
+  if(get_partition_name_i386_aux(entree->sys_ind)!=NULL)
+    ecrit_rapport(" %-20s ", get_partition_name_i386_aux(entree->sys_ind));
   else
     ecrit_rapport(" Sys=%02X               ", entree->sys_ind);
   ecrit_rapport("%4u %3u %2u"
@@ -1060,7 +1101,7 @@ static int test_structure_i386(t_list_part *list_part)
 	  first_log=element;
 	  nbr_log_block++;
 	}
-	if(is_extended(element->part->part_type))
+	if(is_extended(element->part->part_type_i386))
 	{
           return 1;
 	}
@@ -1126,12 +1167,12 @@ static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_
       int command;
       static struct MenuItem menuGeometry[]=
       {
-	{ 'c', "Cylinders", 	"Change starting cylinder" },
-	{ 'h', "Heads", 	"Change starting head" },
-	{ 's', "Sectors", 	"Change starting sector" },
-	{ 'C', "Cylinders", 	"Change ending cylinder" },
-	{ 'H', "Heads", 	"Change ending head" },
-	{ 'S', "Sectors", 	"Change ending sector" },
+	{ 'c', "Cylinder", 	"Change starting cylinder" },
+	{ 'h', "Head", 		"Change starting head" },
+	{ 's', "Sector", 	"Change starting sector" },
+	{ 'C', "Cylinder", 	"Change ending cylinder" },
+	{ 'H', "Head", 		"Change ending head" },
+	{ 'S', "Sector", 	"Change ending sector" },
 	{ 'T' ,"Type",		"Change partition type"},
 	{ 'd', "Done", "" },
 	{ 0, NULL, NULL }
@@ -1188,7 +1229,7 @@ static t_list_part *add_partition_i386(t_param_disk *disk_car,t_list_part *list_
       }
     }
   }
-  if((CHS2offset(disk_car,&end)>new_partition->part_offset)&&(new_partition->part_offset>0)&& new_partition->part_type>0)
+  if((CHS2offset(disk_car,&end)>new_partition->part_offset)&&(new_partition->part_offset>0)&& new_partition->part_type_i386!=P_NO_OS)
   {
     t_list_part *new_list_part=insert_new_partition(list_part, new_partition);
     if(test_structure_i386(list_part)==0)
@@ -1233,9 +1274,19 @@ static void set_prev_status_i386(const t_param_disk *disk_car, t_partition *part
     partition->status--;
 }
 
-static int is_part_type_ok_i386(unsigned int part_type)
+static int set_part_type_i386(t_partition *partition, unsigned int part_type)
 {
-  return (part_type>0 && part_type <= 255 && is_extended(part_type)==0);
+  if(part_type!=P_NO_OS && part_type <= 255 && is_extended(part_type)==0)
+  {
+    partition->part_type_i386=part_type;
+    return 0;
+  }
+  return 1;
+}
+
+static int is_part_known_i386(const t_partition *partition)
+{
+  return (partition->part_type_i386!=P_NO_OS && partition->part_type_i386!=P_UNK);
 }
 
 static void init_structure_i386(const t_param_disk *disk_car,t_list_part *list_part, const int debug)
@@ -1357,15 +1408,26 @@ static void init_structure_i386(const t_param_disk *disk_car,t_list_part *list_p
 #endif
 }
 
-static void erase_list_part_i386(unsigned char *buffer)
+static int erase_list_part_i386(t_param_disk *disk_car)
 {
+  unsigned char buffer[DEFAULT_SECTOR_SIZE];
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    wdoprintf(stdscr,msg_PART_RD_ERR);
+    memset(buffer,0,sizeof(buffer));
+  }
   memset(buffer+TAB_PART,0,0x40);
+  if(disk_car->write(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    return 1;
+  }
+  return 0;
 }
 
 static int check_part_i386(t_param_disk *disk_car,const int debug,t_partition *partition, const int saveheader)
 {
   int ret=0;
-  switch(partition->part_type)
+  switch(partition->part_type_i386)
   {
     case P_BEOS:
       ret=check_BeFS(disk_car,partition,debug);
@@ -1453,7 +1515,7 @@ static int check_part_i386(t_param_disk *disk_car,const int debug,t_partition *p
     default:
       if(debug>0)
       {
-	ecrit_rapport("check_part_i386 %u type %02X: no test\n",partition->order,partition->part_type);
+	ecrit_rapport("check_part_i386 %u type %02X: no test\n",partition->order,partition->part_type_i386);
       }
       if(saveheader>0)
       {
@@ -1463,7 +1525,7 @@ static int check_part_i386(t_param_disk *disk_car,const int debug,t_partition *p
   }
   if(ret!=0)
   {
-    ecrit_rapport("check_part_i386 failed for partition type %02X\n", partition->part_type);
+    ecrit_rapport("check_part_i386 failed for partition type %02X\n", partition->part_type_i386);
     aff_part_buffer(AFF_PART_ORDER,disk_car,partition);
     if(saveheader>0)
     {
@@ -1473,3 +1535,16 @@ static int check_part_i386(t_param_disk *disk_car,const int debug,t_partition *p
   return ret;
 }
 
+static const unsigned char *get_partition_name_i386_aux(const unsigned int part_type_i386)
+{
+  int i;
+  for (i=0; i386_sys_types[i].name!=NULL; i++)
+    if (i386_sys_types[i].part_type == part_type_i386)
+      return i386_sys_types[i].name;
+  return NULL;
+}
+
+static const unsigned char *get_partition_name_i386(const t_partition *partition)
+{
+  return get_partition_name_i386_aux(partition->part_type_i386);
+}
