@@ -2,7 +2,7 @@
 
     File: partsun.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,7 +31,7 @@
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
-#include <ctype.h>      /* toupper, tolower */
+#include <ctype.h>      /* tolower */
 #include "types.h"
 #include "common.h"
 #include "testdisk.h"
@@ -61,53 +61,62 @@ static int get_geometry_from_sunmbr(const unsigned char *buffer, const int debug
 static t_list_part *read_part_sun(t_param_disk *disk_car, const int debug, const int saveheader);
 static int write_part_sun(t_param_disk *disk_car, const t_list_part *list_part, const int ro , const int debug, const int align);
 static t_list_part *init_part_order_sun(const t_param_disk *disk_car, t_list_part *list_part);
-static int write_MBR_code_sun(unsigned char *buffer);
 static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug);
 static void set_next_status_sun(const t_param_disk *disk_car, t_partition *partition);
 static int test_structure_sun(t_list_part *list_part);
-static int is_part_type_ok_sun(unsigned int part_type);
+static int set_part_type_sun(t_partition *partition, unsigned int part_type_sun);
+static int is_part_known_sun(const t_partition *partition);
 static void init_structure_sun(const t_param_disk *disk_car,t_list_part *list_part, const int debug);
-static void erase_list_part_sun(unsigned char *buffer);
+static const unsigned char *get_partition_name_sun(const t_partition *partition);
+static const unsigned char *get_partition_name_sun_aux(const unsigned int part_type_sun);
+static unsigned int get_part_type_sun(const t_partition *partition);
 
 static const struct systypes sun_sys_types[] = {
-  /* 0            */  {"\x00" "Empty"        },
-  /* 1            */  {"\x01" "Boot"         },
-  /* 2            */  {"\x02" "SunOS root"   },
-  /* SUNOS_SWAP   */  {"\x03" "SunOS swap"   },
-  /* 4            */  {"\x04" "SunOS usr"    },
-  /* WHOLE_DISK   */  {"\x05" "Whole disk"   },
-  /* 6            */  {"\x06" "SunOS stand"  },
-  /* 7            */  {"\x07" "SunOS var"    },
-  /* 8            */  {"\x08" "SunOS home"   },
-  /* 9            */  {"\x09" "SunOS alt."   },
-  /* 10           */  {"\x0A" "SunOS cachefs"},
-  /* LINUX_SWAP   */  {"\x82" "Linux swap"   },
-  /* LINUX_NATIVE */  {"\x83" "Linux native" },
-  /* 0x8e         */  {"\x8e" "Linux LVM"    },
-  /* New (2.2.x) raid partition with autodetect using persistent superblock */
-  /* 0xfd         */  {"\xfd" "Linux raid autodetect" },
-                      { NULL }
+  {0x00,	 "Empty"        },
+  {PSUN_BOOT,	 "Boot"         },
+  {PSUN_ROOT,	 "SunOS root"   },
+  {PSUN_SWAP,	 "SunOS swap"   },
+  {PSUN_USR,	 "SunOS usr"    },
+  {PSUN_WHOLE_DISK,	 "Whole disk"   },
+  {PSUN_STAND,	 "SunOS stand"  },
+  {PSUN_VAR,	 "SunOS var"    },
+  {PSUN_HOME,	 "SunOS home"   },
+  {PSUN_ALT,	 "SunOS alt."   },
+  {PSUN_CACHEFS, "SunOS cachefs"},
+  {PSUN_LINSWAP, "Linux swap"   },
+  {PSUN_LINUX,	 "Linux native" },
+  {PSUN_LVM,	 "Linux LVM"    },
+  {PSUN_RAID,	 "Linux raid autodetect" },
+  {0, NULL }
 };
 
 t_arch_fnct arch_sun=
 {
   .part_name="Partition table type : Sun",
-  .parttype_name_table=sun_sys_types,
+  .msg_part_type="                P=Primary  D=Deleted",
   .read_part=read_part_sun,
   .write_part=write_part_sun,
   .init_part_order=init_part_order_sun,
   .get_geometry_from_mbr=get_geometry_from_sunmbr,
   .check_part=check_part_sun,
-  .write_MBR_code=write_MBR_code_sun,
+  .write_MBR_code=NULL,
   .add_partition=add_partition_sun,
   .set_prev_status=set_next_status_sun,
   .set_next_status=set_next_status_sun,
   .test_structure=test_structure_sun,
-  .is_part_type_ok=is_part_type_ok_sun,
+  .set_part_type=set_part_type_sun,
+  .is_part_known=is_part_known_sun,
   .init_structure=init_structure_sun,
-  .erase_list_part=erase_list_part_sun
+  .erase_list_part=NULL,
+  .get_partition_name=get_partition_name_sun,
+  .get_part_type=get_part_type_sun
 };
 
+static unsigned int get_part_type_sun(const t_partition *partition)
+{
+  return partition->part_type_sun;
+}
+
 int get_geometry_from_sunmbr(const unsigned char *buffer, const int debug, t_CHS *geometry)
 {
   const sun_partition *sunlabel=(const sun_partition*)buffer;
@@ -132,8 +141,11 @@ t_list_part *read_part_sun(t_param_disk *disk_car, const int debug, const int sa
   t_list_part *new_list_part=NULL;
   unsigned int i;
   aff_buffer(BUFFER_RESET,"Q");
-  if(read_MBR(disk_car,&buffer)!=0)
+  if(disk_car->read(disk_car,DEFAULT_SECTOR_SIZE, buffer, (uint64_t)0))
+  {
+    wdoprintf(stdscr,msg_PART_RD_ERR);
     return NULL;
+  }
   if (be16(sunlabel->magic) != SUN_LABEL_MAGIC)
   {
     aff_buffer(BUFFER_ADD,"Bad SUN partition\n");
@@ -143,12 +155,12 @@ t_list_part *read_part_sun(t_param_disk *disk_car, const int debug, const int sa
   {
      if (sunlabel->partitions[i].num_sectors > 0
 	 && sunlabel->infos[i].id > 0
-	 && sunlabel->infos[i].id != WHOLE_DISK)
+	 && sunlabel->infos[i].id != PSUN_WHOLE_DISK)
      {
        t_partition *new_partition=partition_new();
        partition_reset(new_partition);
        new_partition->order=i;
-       new_partition->part_type=sunlabel->infos[i].id;
+       new_partition->part_type_sun=sunlabel->infos[i].id;
        new_partition->part_offset=be32(sunlabel->partitions[i].start_cylinder)*be16(sunlabel->ntrks)*be16(sunlabel->nsect)*disk_car->sector_size;
        new_partition->part_size=(uint64_t)be32(sunlabel->partitions[i].num_sectors)*disk_car->sector_size;
        new_partition->status=STATUS_PRIM;
@@ -193,17 +205,11 @@ static t_list_part *init_part_order_sun(const t_param_disk *disk_car, t_list_par
   new_partition->part_offset=0;
   new_partition->part_size=disk_car->disk_size;
   new_partition->status=STATUS_PRIM;
-  new_partition->part_type=WHOLE_DISK;
+  new_partition->part_type_sun=PSUN_WHOLE_DISK;
   new_partition->order=2;
   return insert_new_partition(list_part, new_partition);
 }
 
-static int write_MBR_code_sun(unsigned char *buffer)
-{ /* TODO: Implement it */
-  not_implemented("write_MBR_code_sun");
-  return 0;
-}
-
 static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_part, const int debug)
 {
   t_CHS start,end;
@@ -263,7 +269,7 @@ static t_list_part *add_partition_sun(t_param_disk *disk_car,t_list_part *list_p
       }
     }
   }
-  if((CHS2offset(disk_car,&end)>new_partition->part_offset)&& new_partition->part_type>0)
+  if((CHS2offset(disk_car,&end)>new_partition->part_offset)&& new_partition->part_type_sun>0)
   {
     t_list_part *new_list_part=insert_new_partition(list_part, new_partition);
     new_partition->status=STATUS_PRIM;
@@ -314,9 +320,19 @@ static int test_structure_sun(t_list_part *list_part)
   return res;
 }
 
-static int is_part_type_ok_sun(unsigned int part_type)
+static int set_part_type_sun(t_partition *partition, unsigned int part_type_sun)
 {
-  return (part_type>0 && part_type <= 255);
+  if(part_type_sun>0 && part_type_sun <= 255)
+  {
+    partition->part_type_sun=part_type_sun;
+    return 0;
+  }
+  return 1;
+}
+
+static int is_part_known_sun(const t_partition *partition)
+{
+  return (partition->part_type_sun!=PSUN_UNK);
 }
 
 static void init_structure_sun(const t_param_disk *disk_car,t_list_part *list_part, const int debug)
@@ -365,15 +381,10 @@ static void init_structure_sun(const t_param_disk *disk_car,t_list_part *list_pa
 #endif
 }
 
-static void erase_list_part_sun(unsigned char *buffer)
-{ /* TODO: Implement it */
-  not_implemented("erase_list_part_sun");
-}
-
 static int check_part_sun(t_param_disk *disk_car,const int debug,t_partition *partition, const int saveheader)
 {
   int ret=0;
-  switch(partition->part_type)
+  switch(partition->part_type_sun)
   {
     case PSUN_BOOT:
     case PSUN_ROOT:
@@ -412,13 +423,13 @@ static int check_part_sun(t_param_disk *disk_car,const int debug,t_partition *pa
     default:
       if(debug>0)
       {
-	ecrit_rapport("check_part_sun %u type %02X: no test\n",partition->order,partition->part_type);
+	ecrit_rapport("check_part_sun %u type %02X: no test\n",partition->order,partition->part_type_sun);
       }
       break;
   }
   if(ret!=0)
   {
-    ecrit_rapport("check_part_sun failed for partition type %02X\n", partition->part_type);
+    ecrit_rapport("check_part_sun failed for partition type %02X\n", partition->part_type_sun);
     aff_part_buffer(AFF_PART_ORDER,disk_car,partition);
     if(saveheader>0)
     {
@@ -428,3 +439,16 @@ static int check_part_sun(t_param_disk *disk_car,const int debug,t_partition *pa
   return ret;
 }
 
+static const unsigned char *get_partition_name_sun_aux(const unsigned int part_type_sun)
+{
+  int i;
+  for (i=0; sun_sys_types[i].name!=NULL; i++)
+    if (sun_sys_types[i].part_type == part_type_sun)
+      return sun_sys_types[i].name;
+  return NULL;
+}
+
+static const unsigned char *get_partition_name_sun(const t_partition *partition)
+{
+  return get_partition_name_sun_aux(partition->part_type_sun);
+}
