@@ -19,12 +19,15 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
  */
-typedef struct file_hint t_file_hint;
-typedef struct file_recovery t_file_recovery;
-typedef struct list_file_hint t_list_file_hint;
-typedef struct struct_file_stat t_file_stat;
 
-struct struct_file_stat
+/* 2^31-1 */
+#define PHOTOREC_MAX_FILE_SIZE ((1>>(31-1))-1)
+typedef struct file_hint_struct t_file_hint;
+typedef struct file_recovery_struct t_file_recovery;
+typedef struct list_file_hint_struct t_list_file_hint;
+typedef struct file_stat_struct t_file_stat;
+
+struct file_stat_struct
 {
   unsigned int enable;
   unsigned int not_recovered;
@@ -32,17 +35,18 @@ struct struct_file_stat
   const t_file_hint *file_hint;
 };
 
-struct file_recovery
+struct file_recovery_struct
 {
   t_file_stat *file_stat;
   FILE *handle;
   char filename[2048];
   uint64_t header_distance;
   uint64_t file_size;
+  uint64_t file_size_on_disk;
   uint64_t start;
 };
 
-struct file_hint
+struct file_hint_struct
 {
   const char *extension;
   const char *description;
