@@ -2,7 +2,7 @@
 
     File: rfs.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -64,7 +64,7 @@ static int test_rfs(const t_param_disk *disk_car, const struct reiserfs_super_bl
     }
     else
       return 1;
-  if(debug!=0)
+  if(debug>0)
     ecrit_rapport("\nReiserFS Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   /*
    * sanity checks.
@@ -96,7 +96,7 @@ int recover_rfs(t_param_disk *disk_car, const struct reiserfs_super_block *sb,t_
 {
   if(test_rfs(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
-  if(debug!=0 || dump_ind!=0)
+  if(debug>0 || dump_ind!=0)
   {
     ecrit_rapport("\nrecover_rfs\n");
     ecrit_rapport("block_count=%u\n",(unsigned int)le32(sb->s_block_count));
@@ -108,13 +108,16 @@ int recover_rfs(t_param_disk *disk_car, const struct reiserfs_super_block *sb,t_
   }
 
   partition->part_size = (uint64_t)le32(sb->s_block_count) * le16(sb->s_blocksize);
-  partition->part_type = (unsigned char)P_LINUX;
+  partition->part_type_i386 = P_LINUX;
+  partition->part_type_mac= PMAC_LINUX;
+  partition->part_type_sun= PSUN_LINUX;
   set_rfs_info(disk_car,sb,partition,debug,dump_ind);
   return 0;
 }
 
 static int set_rfs_info(const t_param_disk *disk_car,const struct reiserfs_super_block *sb,t_partition *partition, const int debug, const int dump_ind)
 {
+  partition->name[0]='\0';
   partition->info[0]='\0';
   switch(partition->upart_type)
   {
@@ -128,8 +131,7 @@ static int set_rfs_info(const t_param_disk *disk_car,const struct reiserfs_super
     case UP_RFS3:
       if(le16(sb->sb_version)==1)
 	strncpy(partition->info,"ReiserFS 3.5 with non standard journal",sizeof(partition->info));
-      else 
-      if(le16(sb->sb_version)==2)
+      else if(le16(sb->sb_version)==2)
 	strncpy(partition->info,"ReiserFS 3.6 with non standard journal",sizeof(partition->info));
       else
 	strncpy(partition->info,"ReiserFS ? with non standard journal",sizeof(partition->info));
