@@ -2,7 +2,7 @@
 
     File: rfs_dir.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
     Some code from Yury Umanets <torque@ukrpost.net>
   
     This software is free software; you can redistribute it and/or modify
@@ -44,6 +44,7 @@
 #include "intrf.h"
 #include "rfs.h"
 #include "dir.h"
+#include "rfs_dir.h"
 
 struct list_dir_struct {
 	t_file_data *dir_list;
@@ -438,7 +439,7 @@ t_file_data *reiser_dir(t_param_disk *disk_car, const t_partition *partition, t_
 }
 
 
-int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
+int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
 {
   dal_t *dal;
   reiserfs_fs_t *fs;
@@ -489,7 +490,7 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *part
     dir_data.private_dir_data=&ls;
     dir_data.get_dir=reiser_dir;
     strncpy(dir_data.current_directory,"/",sizeof(dir_data.current_directory));
-    dir_partition(disk_car,partition,&dir_data,2);
+    dir_partition(disk_car,partition,&dir_data,2, current_cmd);
   }
   reiserfs_fs_close(fs);
   file_close(dal);
@@ -497,24 +498,33 @@ int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *part
 error_free_dal:
   file_close(dal);
 dir_partition_reiser_error:
-  aff_buffer(BUFFER_DISPLAY,"Q",window);
+  screen_buffer_display(window,"",NULL);
   return 0xff;
 }
 
+const char*td_reiserfs_version(void)
+{
+  return libreiserfs_get_version();
+}
 #else
 #include "types.h"
 #include "common.h"
 #include "intrf.h"
 #include "rfs.h"
 
-int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug)
+int dir_partition_reiser(WINDOW *window,t_param_disk *disk_car,t_partition *partition, const int debug, char **current_cmd)
 {
   aff_buffer(BUFFER_RESET,"Q");
   wmove(window,4,0);
   aff_part(window,AFF_PART_ORDER,disk_car,partition);
   aff_part_rapport(disk_car,partition);
   aff_buffer(BUFFER_ADD,"Recompile with progsreiserfs library");
-  aff_buffer(BUFFER_DISPLAY,"Q",window);
+  screen_buffer_display(window,"",NULL);
   return 0;
 }
+
+const char*td_reiserfs_version(void)
+{
+  return "none";
+}
 #endif
