@@ -2,7 +2,7 @@
 
     File: savehdr.c
 
-    Copyright (C) 2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -73,7 +73,7 @@ int save_header(t_param_disk *disk_car,t_partition *partition, const int debug)
       case STATUS_DELETED:        status='D'; break;
     }
     snprintf((char*)buffer,256*DEFAULT_SECTOR_SIZE,"%s\n%2u %c Sys=%02X %5u %3u %2u %5u %3u %2u %10lu\n",
-	disk_car->description(disk_car), partition->order,status,partition->part_type,
+	disk_car->description(disk_car), partition->order,status,disk_car->arch->get_part_type(partition),
 	offset2cylinder(disk_car,partition->part_offset), offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset),
 	offset2cylinder(disk_car,partition->part_offset+partition->part_size-disk_car->sector_size), offset2head(disk_car,partition->part_offset+partition->part_size-disk_car->sector_size),offset2sector(disk_car,partition->part_offset+partition->part_size-disk_car->sector_size),
 	(unsigned long)(partition->part_size/disk_car->sector_size));
@@ -177,7 +177,7 @@ t_backup_disk *partition_load(const t_param_disk *disk_car, const int debug)
       {
 	new_partition->part_offset=(uint64_t)part_offset*disk_car->sector_size;
 	new_partition->part_size=(uint64_t)part_size*disk_car->sector_size;
-	new_partition->part_type=part_type;
+	disk_car->arch->set_part_type(new_partition,part_type);
 	switch(status)
 	{
 	  case 'P':	new_partition->status=STATUS_PRIM; break;
@@ -250,7 +250,8 @@ int partition_save(t_param_disk *disk_car, t_list_part *list_part, const int deb
 	  }
 	  fprintf(f_backup,"%2d : start=%9lu, size=%9lu, Id=%02X, %c\n",
 	      parts->part->order, (unsigned long)(parts->part->part_offset/disk_car->sector_size),
-	      (unsigned long)(parts->part->part_size/disk_car->sector_size), parts->part->part_type, status);
+	      (unsigned long)(parts->part->part_size/disk_car->sector_size),
+	      disk_car->arch->get_part_type(parts->part), status);
 	}
 	break;
       default:
