@@ -2,7 +2,7 @@
 
     File: swap.c
 
-    Copyright (C) 1998-2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 1998-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@ int check_Linux_SWAP(t_param_disk *disk_car,t_partition *partition,const int deb
 
 static int set_Linux_SWAP_info(const union swap_header *swap_header,t_partition *partition)
 {
+  partition->name[0]='\0';
   switch(partition->upart_type)
   {
     case UP_LINSWAP:
@@ -52,6 +53,7 @@ static int set_Linux_SWAP_info(const union swap_header *swap_header,t_partition
       break;
     case UP_LINSWAP2:
       snprintf(partition->info,sizeof(partition->info),"SWAP2 version %u",swap_header->info.version);
+/*      set_part_name(partition,swap_header->info.volume_name,16); */
       break;
     default:
       partition->info[0]='\0';
@@ -69,6 +71,8 @@ int test_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_header
   }
   if(memcmp(swap_header->magic.magic,"SWAPSPACE2",10)==0)
   {
+    if(swap_header->info.last_page==0)
+      return 1;
     partition->upart_type=UP_LINSWAP2;
     return 0;
   }
@@ -80,7 +84,9 @@ int recover_Linux_SWAP(t_param_disk *disk_car, const union swap_header *swap_hea
   if(test_Linux_SWAP(disk_car,swap_header,partition,debug,dump_ind)!=0)
     return 1;
   set_Linux_SWAP_info(swap_header,partition);
-  partition->part_type=(unsigned char)P_LINSWAP;
+  partition->part_type_i386=P_LINSWAP;
+  partition->part_type_sun=PSUN_LINSWAP;
+  partition->part_type_mac=PMAC_SWAP;
   switch(partition->upart_type)
   {
     case UP_LINSWAP:
