@@ -2,7 +2,7 @@
 
     File: sysv.c
 
-    Copyright (C) 2004 Christophe GRENIER <grenier@cgsecurity.org>
+    Copyright (C) 2004-2005 Christophe GRENIER <grenier@cgsecurity.org>
   
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -78,7 +78,7 @@ static int test_sysv4(const t_param_disk *disk_car, const struct sysv4_super_blo
   }
   else
     return 1;
-  if(debug!=0)
+  if(debug>0)
     ecrit_rapport("\nSYSV4 Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
   return 0;
 }
@@ -88,7 +88,7 @@ int recover_sysv(t_param_disk *disk_car, const unsigned char *buffer, t_partitio
   const struct sysv4_super_block *sbd=(const struct sysv4_super_block *)buffer;
   if(test_sysv4(disk_car,sbd,partition,debug,dump_ind)==0)
   {
-    if(debug!=0 || dump_ind!=0)
+    if(debug>0 || dump_ind!=0)
     {
       ecrit_rapport("\nrecover_sysv4\n");
       if(dump_ind!=0)
@@ -106,7 +106,7 @@ int recover_sysv(t_param_disk *disk_car, const unsigned char *buffer, t_partitio
 	break;
     }
     set_sysv4_info(disk_car,sbd,partition,debug,dump_ind);
-    partition->part_type = (unsigned char)0x63;
+    partition->part_type_i386 = P_SYSV;
     return 0;
   }
   return 1;
