@@ -39,11 +39,11 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
-#include <ctype.h>      /* toupper, tolower */
+#include <ctype.h>      /* tolower */
 #ifdef HAVE_LOCALE_H
 #include <locale.h>	/* setlocale */
 #endif
-#ifdef HAVE_SIGNAL
+#ifdef HAVE_SIGNAL_H
 #include <signal.h>
 #endif
 #include <errno.h>
@@ -55,7 +55,11 @@
 #include "intrface.h"
 #include "godmode.h"
 #include "fnctdsk.h"
+#include "ext2_dir.h"
+#include "rfs_dir.h"
+#include "ntfs_dir.h"
 extern const t_arch_fnct arch_i386;
+extern const t_arch_fnct arch_mac;
 extern const t_arch_fnct arch_sun;
 
 static FILE* init_log(const char*filename,int argc, char**argv);
@@ -149,19 +153,23 @@ int main( int argc, char **argv )
   int help=0, create_log=0, debug=0, dump_ind=0;
   int do_list=0;
   int test_recovery=0;
-  int write_used=0;
+  int write_used;
   int saveheader=0;
   int create_backup=0;
+  int use_direct_io=0;
   t_list_disk *list_disk=NULL;
   t_list_disk *element_disk;
   const char *cmd_device=NULL;
-#ifdef SOLARIS
+  char *cmd_run=NULL;
+#ifdef TARGET_SOLARIS
   const t_arch_fnct *arch=&arch_sun;
+#elif defined __APPLE__
+  const t_arch_fnct *arch=&arch_mac;
 #else
   const t_arch_fnct *arch=&arch_i386;
 #endif
 #ifdef HAVE_SIGACTION
-  struct sigaction action, old_action;
+  struct sigaction action;
 #endif
 #ifdef TESTING
   srand(1);
@@ -170,7 +178,7 @@ int main( int argc, char **argv )
   /* set up the signal handler for SIGHUP */
   action.sa_handler  = sighup_hdlr;
   action.sa_flags = 0;
-  if(sigaction(SIGHUP, &action, &old_action)==-1)
+  if(sigaction(SIGHUP, &action, NULL)==-1)
   {
     printf("Error on SIGACTION call\n");
     return -1;
@@ -209,105 +217,82 @@ int main( int argc, char **argv )
       saveheader=1;
     else if((strcmp(argv[i],"/backup")==0) || (strcmp(argv[i],"-backup")==0))
       create_backup=1;
+    else if((strcmp(argv[i],"/direct")==0) || (strcmp(argv[i],"-direct")==0))
+      use_direct_io=1;
     else if((strcmp(argv[i],"/help")==0) || (strcmp(argv[i],"-help")==0) || (strcmp(argv[i],"--help")==0) ||
       (strcmp(argv[i],"/h")==0) || (strcmp(argv[i],"-h")==0))
       help=1;
-      /*
     else if(strcmp(argv[i],"/cmd")==0)
     {
       if(i+2>=argc)
-	help=1
+	help=1;
       else
       {
-	cmd_device=argv[i+1];
-	 TODO: handle
-	  analyse
-	  geometry,C,H,S
-	  write
-	  backup
-	  restore
-	  search
-	  rebuildbs
-	  rebuildbs,write
-	  cleanfat
-	  searchsb
+	cmd_device=argv[++i];
+	cmd_run=argv[++i];
       }
     }
-       */
     else
     {
-      list_disk=insert_new_disk(list_disk,file_test_availability(argv[i],debug,arch));
+      list_disk=insert_new_disk(list_disk,file_test_availability(argv[i],debug,arch,use_direct_io));
       if(list_disk==NULL)
       {
 	help=1;
       }
     }
   }
-  if(cmd_device!=NULL && list_disk!=NULL)
-    help=1;
+  if(cmd_device!=NULL)
+  {
+    create_log=1;
+    if(debug<1)
+      debug=1;
+  }
   printf("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
   if(help!=0)
   {
-    printf("\nUsage: testdisk [/log] [/debug] [/dump] [/saveheader] [file or device]"\
-		"\n       testdisk /list [/log] [/backup] [file or device]" \
-	    "\n" \
-	    "\n/log          : create a testdisk.log file" \
-	    "\n/debug        : add debug information" \
-	    "\n/dump         : dump raw sectors" \
-	    "\n/list         : display current partitions" \
-	    "\n/saveheader   : save unknown filesystem header" \
-	    "\n/backup       : backup partition structure" \
-	    "\n" \
-	    "\nTestDisk checks and recovers lost partitions" \
-	    "\nIt works with :" \
-	    "\n- FAT12, FAT16 <32M, FAT16 >32M, FAT32" \
-	    "\n- NTFS" \
-	    "\n- LINUX ext2fs/ext3fs, LINUX swap" \
-	    "\n- BFS (BeOS)" \
-	    "\n- UFS (BSD)" \
-	    "\n- ReiserFS (1 & 2)" \
-	    "\n- JFS" \
-	    "\n" \
-	    "\nIf you have problems with TestDisk or bug reports, please contacte me.\n");
+    printf("\n" \
+	"Usage: testdisk [/log] [/debug] [file or device]\n"\
+	"       testdisk /list  [/log]   [file or device]\n" \
+	"\n" \
+	"/log          : create a testdisk.log file\n" \
+	"/debug        : add debug information\n" \
+	"/list         : display current partitions\n" \
+	"\n" \
+	"TestDisk checks and recovers lost partitions\n" \
+	"It works with :\n" \
+	"- BeFS (BeOS)                           - BSD disklabel (Free/Open/Net BSD)\n" \
+	"- CramFS, Compressed File System        - DOS/Windows FAT12, FAT16 and FAT32\n" \
+	"- HFS, HFS+, Hierarchical File System   - JFS, IBM's Journaled File System\n" \
+	"- Linux Ext2 and Ext3                   - Linux Raid\n" \
+	"- Linux Swap                            - LVM, LVM2, Logical Volume Manager\n" \
+	"- Netware NSS                           - NTFS (Windows NT/2K/XP/2003)\n" \
+	"- ReiserFS 3.5 and 3.6                  - Sun Solaris i386 disklabel\n" \
+	"- UFS and UFS2 (Sun/BSD/...)            - XFS, SGI's Journaled File System\n" \
+	"\n" \
+	"If you have problems with TestDisk or bug reports, please contacte me.\n");
     return 0;
   }
   if(create_log==1 && f_rapport==NULL)
   {
-    /*    const char *ext2fs_version=NULL; */
     f_rapport=init_log("testdisk.log",argc,argv);
     ecrit_rapport("TestDisk %s, Data Recovery Utility, %s\nChristophe GRENIER <grenier@cgsecurity.org>\nhttp://www.cgsecurity.org\n",VERSION,TESTDISKDATE);
 #ifdef DJGPP
     ecrit_rapport("Dos version");
-#elif defined(BSD)
+#elif defined(TARGET_BSD)
     ecrit_rapport("BSD version");
-#elif defined(LINUX)
+#elif defined(TARGET_LINUX)
     ecrit_rapport("Linux version");
+#elif defined(TARGET_SOLARIS)
+    ecrit_rapport("Solaris version");
 #elif defined(__CYGWIN__) || defined(__MINGW32__)
     ecrit_rapport("Windows version");
 #else
     ecrit_rapport("Undefined OS");
 #endif
-#ifdef COMPILE_BY
-#ifdef COMPILE_HOST
-#ifdef COMPILE_TIME
-    ecrit_rapport(" (%s@%s, %s)",COMPILE_BY,COMPILE_HOST,COMPILE_TIME);
-#endif
-#endif
-#endif
-    ecrit_rapport("\n");
-    /*    ext2fs_get_library_version(ext2fs_version,NULL); */
-    /*    ecrit_rapport("%s\n\n",ext2fs_version); */
-#ifdef DEBUG
-    ecrit_rapport("Key down:      0x%x\n", KEY_DOWN);
-    ecrit_rapport("Key up:        0x%x\n", KEY_UP);
-    ecrit_rapport("Key left:      0x%x\n", KEY_LEFT);
-    ecrit_rapport("Key right:     0x%x\n", KEY_RIGHT);
-    ecrit_rapport("Key page down: 0x%x\n", KEY_NPAGE);
-    ecrit_rapport("Key page up:   0x%x\n", KEY_PPAGE);
-    ecrit_rapport("Key enter:     0x%x\n", KEY_ENTER);
-#endif
+    ecrit_rapport(" (ext2fs lib: %s, ntfs lib: %s, reiserfs lib: %s)\n",td_ext2fs_version(),td_ntfs_version(),td_reiserfs_version());
   }
-printf("Please wait...\n");
+  printf("Please wait...\n");
+#ifdef HAVE_SETLOCALE
   {
     const char *locale;
     locale = setlocale (LC_ALL, "");
@@ -318,8 +303,9 @@ printf("Please wait...\n");
       ecrit_rapport("Using locale '%s'.\n", locale);
     }
   }
+#endif
   aff_buffer(BUFFER_RESET,"Q");
-  list_disk=hd_parse(list_disk,debug,arch);
+  list_disk=hd_parse(list_disk,debug,arch,use_direct_io);
   if(list_disk==NULL)
   {
     printf("No harddisk found\n");
@@ -345,7 +331,7 @@ printf("Please wait...\n");
     printf("%s\n",element_disk->disk->description(element_disk->disk));
   }
 #endif
-  hd_parse_bis(list_disk,0,arch);
+  hd_parse_bis(list_disk,0,arch,debug);
   /* save disk parameters to rapport */
   ecrit_rapport("Hard disk list\n");
   for(element_disk=list_disk;element_disk!=NULL;element_disk=element_disk->next)
@@ -363,7 +349,7 @@ printf("Please wait...\n");
       if(test_recovery!=0)
       {
 	t_list_part *list_part=search_part(element_disk->disk,0,debug,0,0,0);
-	if(debug!=0)
+	if(debug>0)
 	{
 	  t_list_part *element;
 	  ecrit_rapport("search_part() results\n");
@@ -376,18 +362,11 @@ printf("Please wait...\n");
   }
   else
   {
-    do_curses_testdisk(debug,dump_ind,list_disk,saveheader,&arch);
-  }
-  for(element_disk=list_disk;element_disk!=NULL;)
-  {
-    t_list_disk *element_disk_next=element_disk->next;
-    write_used|=element_disk->disk->write_used;
-    if(element_disk->disk->clean!=NULL)
-      element_disk->disk->clean(element_disk->disk);
-    FREE(element_disk->disk);
-    FREE(element_disk);
-    element_disk=element_disk_next;
+    do_curses_testdisk(debug,dump_ind,list_disk,saveheader,&arch,cmd_device,cmd_run);
+    cmd_device=NULL;
+    cmd_run=NULL;
   }
+  write_used=delete_list_disk(list_disk);
   if(f_rapport!=NULL)
   {
     ecrit_rapport("TestDisk exited normally.\n");
