@@ -29,6 +29,8 @@
 #include <ncurses.h>
 #elif defined(HAVE_NCURSES_NCURSES_H)
 #include <ncurses/ncurses.h>
+#elif defined(HAVE_NCURSES_CURSES_H)
+#include <ncurses/curses.h>
 #elif defined(HAVE_CURSES_H)
 #include <curses.h>
 #endif
