@@ -32,7 +32,6 @@
 #include "ufs.h"
 #include "fnctdsk.h"
 #include "intrf.h"
-extern const t_arch_fnct arch_sun;
 
 static int set_ufs_info(const t_param_disk *disk_car, const struct ufs_super_block *sb,t_partition *partition, const int debug, const int dump_ind);
 static int test_ufs(const t_param_disk *disk_car, const struct ufs_super_block *sb,t_partition *partition,const int debug, const int dump_ind);
@@ -54,14 +53,14 @@ static int test_ufs(const t_param_disk *disk_car, const struct ufs_super_block *
   if ((sb->fs_magic==UFS_MAGIC) || (sb->fs_magic==UFS_CIGAM))
   {
     partition->upart_type = UP_UFS;
-    if(debug!=0)
+    if(debug>0)
       ecrit_rapport("\nUFS Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
     return 0;
   }
   else if ((sb->fs_magic==UFS2_MAGIC) || (sb->fs_magic==UFS2_CIGAM))
   {
     partition->upart_type = UP_UFS2;
-    if(debug!=0)
+    if(debug>0)
       ecrit_rapport("\nUFS2 Marker at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
     return 0;
   }
@@ -72,7 +71,7 @@ int recover_ufs(t_param_disk *disk_car, const struct ufs_super_block *sb, t_part
 {
   if(test_ufs(disk_car,sb,partition,debug,dump_ind)!=0)
     return 1;
-  if(debug!=0 || dump_ind!=0)
+  if(debug>0 || dump_ind!=0)
   {
     ecrit_rapport("\nrecover_ufs\n");
     if(dump_ind!=0)
@@ -112,22 +111,22 @@ int recover_ufs(t_param_disk *disk_car, const struct ufs_super_block *sb, t_part
       break;
   }
   set_ufs_info(disk_car,sb,partition,debug,dump_ind);
-  partition->arch=&arch_sun;
   if(strcmp(partition->name,"/")==0)
-    partition->part_type = (unsigned char)PSUN_ROOT;
+    partition->part_type_sun = (unsigned char)PSUN_ROOT;
   else if(strcmp(partition->name,"/var")==0)
-    partition->part_type = (unsigned char)PSUN_VAR;
+    partition->part_type_sun = (unsigned char)PSUN_VAR;
   else if(strcmp(partition->name,"/usr")==0)
-    partition->part_type = (unsigned char)PSUN_USR;
+    partition->part_type_sun = (unsigned char)PSUN_USR;
   else if(strcmp(partition->name,"/export/home")==0)
-    partition->part_type = (unsigned char)PSUN_HOME;
+    partition->part_type_sun = (unsigned char)PSUN_HOME;
   else
-    partition->part_type = (unsigned char)PSUN_ROOT;
+    partition->part_type_sun = (unsigned char)PSUN_ROOT;
   return 0;
 }
 
 static int set_ufs_info(const t_param_disk *disk_car,const struct ufs_super_block *sb,t_partition *partition, const int debug, const int dump_ind)
 {
+  partition->name[0]='\0';
   partition->info[0]='\0';
   switch(partition->upart_type)
   {
